@@ -8,12 +8,12 @@
 #include <memory>
 
 #include "base/bind.h"
+#include "base/logging.h"
 #include "base/message_loop/message_pump_default.h"
 #include "base/message_loop/message_pump_type.h"
 #include "base/run_loop.h"
 #include "base/sequence_checker.h"
 #include "base/single_thread_task_runner.h"
-#include "base/strings/stringprintf.h"
 #include "base/synchronization/condition_variable.h"
 #include "base/task/post_task.h"
 #include "base/task/sequence_manager/task_queue_impl.h"
@@ -32,6 +32,7 @@
 #include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/perf/perf_result_reporter.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace base {
 namespace sequence_manager {
@@ -55,12 +56,14 @@ perf_test::PerfResultReporter SetUpReporter(const std::string& story_name) {
 class PerfTestTimeDomain : public MockTimeDomain {
  public:
   PerfTestTimeDomain() : MockTimeDomain(TimeTicks::Now()) {}
+  PerfTestTimeDomain(const PerfTestTimeDomain&) = delete;
+  PerfTestTimeDomain& operator=(const PerfTestTimeDomain&) = delete;
   ~PerfTestTimeDomain() override = default;
 
-  Optional<TimeDelta> DelayTillNextTask(LazyNow* lazy_now) override {
-    Optional<TimeTicks> run_time = NextScheduledRunTime();
+  absl::optional<TimeDelta> DelayTillNextTask(LazyNow* lazy_now) override {
+    absl::optional<TimeTicks> run_time = NextScheduledRunTime();
     if (!run_time)
-      return nullopt;
+      return absl::nullopt;
     SetNowTicks(*run_time);
     // Makes SequenceManager to continue immediately.
     return TimeDelta();
@@ -71,9 +74,6 @@ class PerfTestTimeDomain : public MockTimeDomain {
     if (NumberOfScheduledWakeUps() == 1u)
       RequestDoWork();
   }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(PerfTestTimeDomain);
 };
 
 enum class PerfTestType {
@@ -126,7 +126,7 @@ class BaseSequenceManagerPerfTestDelegate : public PerfTestDelegate {
   }
 
   void WaitUntilDone() override {
-    run_loop_.reset(new RunLoop());
+    run_loop_ = std::make_unique<RunLoop>();
     run_loop_->Run();
   }
 
