@@ -52,16 +52,16 @@ class RealTimeDomainFake : public RealTimeDomain {
 class WorkQueueTest : public testing::Test {
  public:
   void SetUp() override {
-    time_domain_.reset(new RealTimeDomainFake());
+    time_domain_ = std::make_unique<RealTimeDomainFake>();
     task_queue_ = std::make_unique<TaskQueueImpl>(/*sequence_manager=*/nullptr,
                                                   time_domain_.get(),
                                                   TaskQueue::Spec("test"));
 
-    work_queue_.reset(new WorkQueue(task_queue_.get(), "test",
-                                    WorkQueue::QueueType::kImmediate));
-    mock_observer_.reset(new MockObserver);
-    work_queue_sets_.reset(new WorkQueueSets("test", mock_observer_.get(),
-                                             SequenceManager::Settings()));
+    work_queue_ = std::make_unique<WorkQueue>(task_queue_.get(), "test",
+                                              WorkQueue::QueueType::kImmediate);
+    mock_observer_ = std::make_unique<MockObserver>();
+    work_queue_sets_ = std::make_unique<WorkQueueSets>(
+        "test", mock_observer_.get(), SequenceManager::Settings());
     work_queue_sets_->AddQueue(work_queue_.get(), 0);
   }
 
@@ -73,23 +73,23 @@ class WorkQueueTest : public testing::Test {
  protected:
   Task FakeCancelableTaskWithEnqueueOrder(int enqueue_order,
                                           WeakPtr<Cancelable> weak_ptr) {
-    Task fake_task(
-        PostedTask(BindOnce(&Cancelable::NopTask, weak_ptr), FROM_HERE),
-        TimeTicks(), EnqueueOrder(),
-        EnqueueOrder::FromIntForTesting(enqueue_order));
+    Task fake_task(PostedTask(nullptr, BindOnce(&Cancelable::NopTask, weak_ptr),
+                              FROM_HERE),
+                   TimeTicks(), EnqueueOrder(),
+                   EnqueueOrder::FromIntForTesting(enqueue_order));
     return fake_task;
   }
 
   Task FakeTaskWithEnqueueOrder(int enqueue_order) {
-    Task fake_task(PostedTask(BindOnce(&NopTask), FROM_HERE), TimeTicks(),
-                   EnqueueOrder(),
+    Task fake_task(PostedTask(nullptr, BindOnce(&NopTask), FROM_HERE),
+                   TimeTicks(), EnqueueOrder(),
                    EnqueueOrder::FromIntForTesting(enqueue_order));
     return fake_task;
   }
 
   Task FakeNonNestableTaskWithEnqueueOrder(int enqueue_order) {
-    Task fake_task(PostedTask(BindOnce(&NopTask), FROM_HERE), TimeTicks(),
-                   EnqueueOrder(),
+    Task fake_task(PostedTask(nullptr, BindOnce(&NopTask), FROM_HERE),
+                   TimeTicks(), EnqueueOrder(),
                    EnqueueOrder::FromIntForTesting(enqueue_order));
     fake_task.nestable = Nestable::kNonNestable;
     return fake_task;
@@ -555,6 +555,20 @@ TEST_F(WorkQueueTest, RemoveAllCanceledTasksFromFrontQueueBlockedByFence) {
   EXPECT_FALSE(work_queue_->GetFrontTaskEnqueueOrder(&enqueue_order));
 }
 
+TEST_F(WorkQueueTest, CollectTasksOlderThan) {
+  work_queue_->Push(FakeTaskWithEnqueueOrder(2));
+  work_queue_->Push(FakeTaskWithEnqueueOrder(3));
+  work_queue_->Push(FakeTaskWithEnqueueOrder(4));
+
+  std::vector<const Task*> result;
+  work_queue_->CollectTasksOlderThan(EnqueueOrder::FromIntForTesting(4),
+                                     &result);
+
+  ASSERT_EQ(2u, result.size());
+  EXPECT_EQ(2u, result[0]->enqueue_order());
+  EXPECT_EQ(3u, result[1]->enqueue_order());
+}
+
 }  // namespace internal
 }  // namespace sequence_manager
 }  // namespace base
