@@ -5,11 +5,8 @@
 #ifndef COMPONENTS_AUTOFILL_ASSISTANT_BROWSER_ACTIONS_WAIT_FOR_NAVIGATION_ACTION_H_
 #define COMPONENTS_AUTOFILL_ASSISTANT_BROWSER_ACTIONS_WAIT_FOR_NAVIGATION_ACTION_H_
 
-#include <string>
-
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
-#include "base/time/time.h"
 #include "base/timer/timer.h"
 #include "components/autofill_assistant/browser/actions/action.h"
 
@@ -22,9 +19,6 @@ class WaitForNavigationAction : public Action {
   ~WaitForNavigationAction() override;
 
  private:
-  static constexpr base::TimeDelta kDefaultTimeout =
-      base::TimeDelta::FromSeconds(20);
-
   // Overrides Action:
   void InternalProcessAction(ProcessActionCallback callback) override;
 
@@ -34,7 +28,7 @@ class WaitForNavigationAction : public Action {
 
   ProcessActionCallback callback_;
   base::OneShotTimer timer_;
-  base::WeakPtrFactory<WaitForNavigationAction> weak_ptr_factory_;
+  base::WeakPtrFactory<WaitForNavigationAction> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(WaitForNavigationAction);
 };
