@@ -9,11 +9,13 @@
 #include "base/json/json_writer.h"
 #include "base/logging.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/strings/string_piece.h"
 #include "base/strings/string_util.h"
 #include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
 #include "components/cast_channel/cast_auth_util.h"
 #include "components/cast_channel/enum_table.h"
-#include "components/cast_channel/proto/cast_channel.pb.h"
+#include "third_party/openscreen/src/cast/common/channel/proto/cast_channel.pb.h"
 
 using base::Value;
 using cast_util::EnumToString;
@@ -21,62 +23,87 @@ using cast_util::StringToEnum;
 
 namespace cast_util {
 
-using namespace cast_channel;
+using ::cast::channel::AuthChallenge;
+using ::cast::channel::CastMessage;
+using cast_channel::CastMessageType;
+using cast_channel::GetAppAvailabilityResult;
 
 template <>
-const EnumTable<CastMessageType> EnumTable<CastMessageType>::instance(
-    {
-        {CastMessageType::kPing, "PING"},
-        {CastMessageType::kPong, "PONG"},
-        {CastMessageType::kGetAppAvailability, "GET_APP_AVAILABILITY"},
-        {CastMessageType::kReceiverStatusRequest, "GET_STATUS"},
-        {CastMessageType::kConnect, "CONNECT"},
-        {CastMessageType::kCloseConnection, "CLOSE"},
-        {CastMessageType::kBroadcast, "APPLICATION_BROADCAST"},
-        {CastMessageType::kLaunch, "LAUNCH"},
-        {CastMessageType::kStop, "STOP"},
-        {CastMessageType::kReceiverStatus, "RECEIVER_STATUS"},
-        {CastMessageType::kMediaStatus, "MEDIA_STATUS"},
-        {CastMessageType::kLaunchError, "LAUNCH_ERROR"},
-        {CastMessageType::kOffer, "OFFER"},
-        {CastMessageType::kAnswer, "ANSWER"},
-        {CastMessageType::kOther},
-    },
-    CastMessageType::kMaxValue);
+const EnumTable<CastMessageType>& EnumTable<CastMessageType>::GetInstance() {
+  static const EnumTable<CastMessageType> kInstance(
+      {
+          {CastMessageType::kPing, "PING"},
+          {CastMessageType::kPong, "PONG"},
+          {CastMessageType::kRpc, "RPC"},
+          {CastMessageType::kGetAppAvailability, "GET_APP_AVAILABILITY"},
+          {CastMessageType::kGetStatus, "GET_STATUS"},
+          {CastMessageType::kConnect, "CONNECT"},
+          {CastMessageType::kCloseConnection, "CLOSE"},
+          {CastMessageType::kBroadcast, "APPLICATION_BROADCAST"},
+          {CastMessageType::kLaunch, "LAUNCH"},
+          {CastMessageType::kStop, "STOP"},
+          {CastMessageType::kReceiverStatus, "RECEIVER_STATUS"},
+          {CastMessageType::kMediaStatus, "MEDIA_STATUS"},
+          {CastMessageType::kLaunchError, "LAUNCH_ERROR"},
+          {CastMessageType::kOffer, "OFFER"},
+          {CastMessageType::kAnswer, "ANSWER"},
+          {CastMessageType::kCapabilitiesResponse, "CAPABILITIES_RESPONSE"},
+          {CastMessageType::kStatusResponse, "STATUS_RESPONSE"},
+          {CastMessageType::kMultizoneStatus, "MULTIZONE_STATUS"},
+          {CastMessageType::kInvalidPlayerState, "INVALID_PLAYER_STATE"},
+          {CastMessageType::kLoadFailed, "LOAD_FAILED"},
+          {CastMessageType::kLoadCancelled, "LOAD_CANCELLED"},
+          {CastMessageType::kInvalidRequest, "INVALID_REQUEST"},
+          {CastMessageType::kPresentation, "PRESENTATION"},
+          {CastMessageType::kGetCapabilities, "GET_CAPABILITIES"},
+          {CastMessageType::kOther},
+      },
+      CastMessageType::kMaxValue);
+  return kInstance;
+}
 
 template <>
-const EnumTable<V2MessageType> EnumTable<V2MessageType>::instance(
-    {
-        {V2MessageType::kEditTracksInfo, "EDIT_TRACKS_INFO"},
-        {V2MessageType::kGetStatus, "GET_STATUS"},
-        {V2MessageType::kLoad, "LOAD"},
-        {V2MessageType::kMediaGetStatus, "MEDIA_GET_STATUS"},
-        {V2MessageType::kMediaSetVolume, "MEDIA_SET_VOLUME"},
-        {V2MessageType::kPause, "PAUSE"},
-        {V2MessageType::kPlay, "PLAY"},
-        {V2MessageType::kPrecache, "PRECACHE"},
-        {V2MessageType::kQueueInsert, "QUEUE_INSERT"},
-        {V2MessageType::kQueueLoad, "QUEUE_LOAD"},
-        {V2MessageType::kQueueRemove, "QUEUE_REMOVE"},
-        {V2MessageType::kQueueReorder, "QUEUE_REORDER"},
-        {V2MessageType::kQueueUpdate, "QUEUE_UPDATE"},
-        {V2MessageType::kSeek, "SEEK"},
-        {V2MessageType::kSetVolume, "SET_VOLUME"},
-        {V2MessageType::kStop, "STOP"},
-        {V2MessageType::kStopMedia, "STOP_MEDIA"},
-        {V2MessageType::kOther},
-    },
-    V2MessageType::kMaxValue);
+const EnumTable<cast_channel::V2MessageType>&
+EnumTable<cast_channel::V2MessageType>::GetInstance() {
+  static const EnumTable<cast_channel::V2MessageType> kInstance(
+      {
+          {cast_channel::V2MessageType::kEditTracksInfo, "EDIT_TRACKS_INFO"},
+          {cast_channel::V2MessageType::kGetStatus, "GET_STATUS"},
+          {cast_channel::V2MessageType::kLoad, "LOAD"},
+          {cast_channel::V2MessageType::kMediaGetStatus, "MEDIA_GET_STATUS"},
+          {cast_channel::V2MessageType::kMediaSetVolume, "MEDIA_SET_VOLUME"},
+          {cast_channel::V2MessageType::kPause, "PAUSE"},
+          {cast_channel::V2MessageType::kPlay, "PLAY"},
+          {cast_channel::V2MessageType::kPrecache, "PRECACHE"},
+          {cast_channel::V2MessageType::kQueueInsert, "QUEUE_INSERT"},
+          {cast_channel::V2MessageType::kQueueLoad, "QUEUE_LOAD"},
+          {cast_channel::V2MessageType::kQueueRemove, "QUEUE_REMOVE"},
+          {cast_channel::V2MessageType::kQueueReorder, "QUEUE_REORDER"},
+          {cast_channel::V2MessageType::kQueueUpdate, "QUEUE_UPDATE"},
+          {cast_channel::V2MessageType::kQueueNext, "QUEUE_NEXT"},
+          {cast_channel::V2MessageType::kQueuePrev, "QUEUE_PREV"},
+          {cast_channel::V2MessageType::kSeek, "SEEK"},
+          {cast_channel::V2MessageType::kSetVolume, "SET_VOLUME"},
+          {cast_channel::V2MessageType::kStop, "STOP"},
+          {cast_channel::V2MessageType::kStopMedia, "STOP_MEDIA"},
+          {cast_channel::V2MessageType::kOther},
+      },
+      cast_channel::V2MessageType::kMaxValue);
+  return kInstance;
+}
 
 template <>
-const EnumTable<GetAppAvailabilityResult>
-    EnumTable<GetAppAvailabilityResult>::instance(
-        {
-            {GetAppAvailabilityResult::kAvailable, "APP_AVAILABLE"},
-            {GetAppAvailabilityResult::kUnavailable, "APP_UNAVAILABLE"},
-            {GetAppAvailabilityResult::kUnknown},
-        },
-        GetAppAvailabilityResult::kMaxValue);
+const EnumTable<GetAppAvailabilityResult>&
+EnumTable<GetAppAvailabilityResult>::GetInstance() {
+  static const EnumTable<GetAppAvailabilityResult> kInstance(
+      {
+          {GetAppAvailabilityResult::kAvailable, "APP_AVAILABLE"},
+          {GetAppAvailabilityResult::kUnavailable, "APP_UNAVAILABLE"},
+          {GetAppAvailabilityResult::kUnknown},
+      },
+      GetAppAvailabilityResult::kMaxValue);
+  return kInstance;
+}
 
 }  // namespace cast_util
 
@@ -84,6 +111,24 @@ namespace cast_channel {
 
 namespace {
 
+constexpr base::StringPiece kCastReservedNamespacePrefix =
+    "urn:x-cast:com.google.cast.";
+
+constexpr const char* kReservedNamespaces[] = {
+    kAuthNamespace,
+    kHeartbeatNamespace,
+    kConnectionNamespace,
+    kReceiverNamespace,
+    kBroadcastNamespace,
+    kMediaNamespace,
+
+    // mirroring::mojom::kRemotingNamespace
+    "urn:x-cast:com.google.cast.remoting",
+
+    // mirroring::mojom::kWebRtcNamespace
+    "urn:x-cast:com.google.cast.webrtc",
+};
+
 // The value used for "sdkType" in a virtual connect request. Historically, this
 // value is used in the Media Router extension, but here it is reused in Chrome.
 constexpr int kVirtualConnectSdkType = 2;
@@ -92,6 +137,10 @@ constexpr int kVirtualConnectSdkType = 2;
 // stands for CONNECTION_TYPE_LOCAL, which is the only type used in Chrome.
 constexpr int kVirtualConnectTypeLocal = 1;
 
+// The reason code passed to the virtual connection CLOSE message indicating
+// that the connection has been gracefully closed by the sender.
+constexpr int kVirtualConnectionClosedByPeer = 5;
+
 void FillCommonCastMessageFields(CastMessage* message,
                                  const std::string& source_id,
                                  const std::string& destination_id,
@@ -115,11 +164,11 @@ CastMessage CreateKeepAliveMessage(base::StringPiece keep_alive_type) {
 int GetVirtualConnectPlatformValue() {
 #if defined(OS_WIN)
   return 3;
-#elif defined(OS_MACOSX)
+#elif defined(OS_APPLE)
   return 4;
-#elif defined(OS_CHROMEOS)
+#elif BUILDFLAG(IS_CHROMEOS_ASH)
   return 5;
-#elif defined(OS_LINUX)
+#elif defined(OS_LINUX) || BUILDFLAG(IS_CHROMEOS_LACROS)
   return 6;
 #else
   return 0;
@@ -132,7 +181,7 @@ int GetVirtualConnectPlatformValue() {
 // messages are passed using a CastInternalMessage object.
 base::StringPiece GetRemappedMediaRequestType(
     base::StringPiece v2_message_type) {
-  base::Optional<V2MessageType> type =
+  absl::optional<V2MessageType> type =
       StringToEnum<V2MessageType>(v2_message_type);
   DCHECK(type && IsMediaRequestMessageType(*type));
   switch (*type) {
@@ -168,7 +217,7 @@ std::ostream& operator<<(std::ostream& lhs, const CastMessage& rhs) {
     lhs << "payload_utf8: " << rhs.payload_utf8();
   }
   if (rhs.has_payload_binary()) {
-    lhs << "payload_binary: ...";
+    lhs << "payload_binary: (" << rhs.payload_binary().size() << " bytes)";
   }
   lhs << "}";
   return lhs;
@@ -182,17 +231,39 @@ bool IsCastMessageValid(const CastMessage& message_proto) {
       message_proto.destination_id().empty()) {
     return false;
   }
-  return (message_proto.payload_type() == CastMessage_PayloadType_STRING &&
+  return (message_proto.payload_type() ==
+              cast::channel::CastMessage_PayloadType_STRING &&
           message_proto.has_payload_utf8()) ||
-         (message_proto.payload_type() == CastMessage_PayloadType_BINARY &&
+         (message_proto.payload_type() ==
+              cast::channel::CastMessage_PayloadType_BINARY &&
           message_proto.has_payload_binary());
 }
 
-bool IsCastInternalNamespace(const std::string& message_namespace) {
-  // Note: any namespace with the prefix is assumed to be reserved for internal
-  // messages.
-  return base::StartsWith(message_namespace, kCastInternalNamespacePrefix,
-                          base::CompareCase::SENSITIVE);
+bool IsCastReservedNamespace(base::StringPiece message_namespace) {
+  // Note: Any namespace with the prefix is theoretically reserved for internal
+  // messages, but there is at least one namespace in widespread use that uses
+  // the "reserved" prefix for app-level messages, so after matching the main
+  // prefix, we look for longer prefixes that really need to be reserved.
+  if (!base::StartsWith(message_namespace, kCastReservedNamespacePrefix))
+    return false;
+
+  const auto prefix_length = kCastReservedNamespacePrefix.length();
+  for (base::StringPiece reserved_namespace : kReservedNamespaces) {
+    DCHECK(base::StartsWith(reserved_namespace, kCastReservedNamespacePrefix));
+    // This comparison skips the first |prefix_length| characters
+    // because we already know they match.
+    if (base::StartsWith(message_namespace.substr(prefix_length),
+                         reserved_namespace.substr(prefix_length)) &&
+        // This condition allows |reserved_namespace| to be equal
+        // |message_namespace| or be a prefix of it, but if it's a
+        // prefix, it must be followed by a dot.  The subscript is
+        // never out of bounds because |message_namespace| must be
+        // at least as long as |reserved_namespace|.
+        (message_namespace.length() == reserved_namespace.length() ||
+         message_namespace[reserved_namespace.length()] == '.'))
+      return true;
+  }
+  return false;
 }
 
 CastMessageType ParseMessageTypeFromPayload(const base::Value& payload) {
@@ -208,7 +279,7 @@ const char* ToString(CastMessageType message_type) {
 
 // TODO(jrw): Eliminate this function.
 const char* ToString(V2MessageType message_type) {
-  return EnumToString(message_type).value_or(nullptr).data();
+  return EnumToString(message_type).value_or("").data();
 }
 
 // TODO(jrw): Eliminate this function.
@@ -251,17 +322,18 @@ void CreateAuthChallengeMessage(CastMessage* message_proto,
   CHECK(message_proto);
   DeviceAuthMessage auth_message;
 
-  AuthChallenge* challenge = auth_message.mutable_challenge();
+  cast::channel::AuthChallenge* challenge = auth_message.mutable_challenge();
   DCHECK(challenge);
   challenge->set_sender_nonce(auth_context.nonce());
-  challenge->set_hash_algorithm(SHA256);
+  challenge->set_hash_algorithm(cast::channel::SHA256);
 
   std::string auth_message_string;
   auth_message.SerializeToString(&auth_message_string);
 
   FillCommonCastMessageFields(message_proto, kPlatformSenderId,
                               kPlatformReceiverId, kAuthNamespace);
-  message_proto->set_payload_type(CastMessage_PayloadType_BINARY);
+  message_proto->set_payload_type(
+      cast::channel::CastMessage_PayloadType_BINARY);
   message_proto->set_payload_binary(auth_message_string);
 }
 
@@ -293,8 +365,6 @@ CastMessage CreateVirtualConnectionRequest(
     VirtualConnectionType connection_type,
     const std::string& user_agent,
     const std::string& browser_version) {
-  DCHECK(destination_id == kPlatformReceiverId || connection_type == kStrong);
-
   // Parse system_version from user agent string. It contains platform, OS and
   // CPU info and is contained in the first set of parentheses of the user
   // agent string (e.g., X11; Linux x86_64).
@@ -331,6 +401,18 @@ CastMessage CreateVirtualConnectionRequest(
                            destination_id);
 }
 
+CastMessage CreateVirtualConnectionClose(const std::string& source_id,
+                                         const std::string& destination_id) {
+  Value dict(Value::Type::DICTIONARY);
+  dict.SetKey(
+      "type",
+      Value(
+          EnumToString<CastMessageType, CastMessageType::kCloseConnection>()));
+  dict.SetKey("reasonCode", Value(kVirtualConnectionClosedByPeer));
+  return CreateCastMessage(kConnectionNamespace, dict, source_id,
+                           destination_id);
+}
+
 CastMessage CreateGetAppAvailabilityRequest(const std::string& source_id,
                                             int request_id,
                                             const std::string& app_id) {
@@ -339,7 +421,7 @@ CastMessage CreateGetAppAvailabilityRequest(const std::string& source_id,
               Value(EnumToString<CastMessageType,
                                  CastMessageType::kGetAppAvailability>()));
   Value app_id_value(Value::Type::LIST);
-  app_id_value.GetList().push_back(Value(app_id));
+  app_id_value.Append(Value(app_id));
   dict.SetKey("appId", std::move(app_id_value));
   dict.SetKey("requestId", Value(request_id));
 
@@ -350,9 +432,9 @@ CastMessage CreateGetAppAvailabilityRequest(const std::string& source_id,
 CastMessage CreateReceiverStatusRequest(const std::string& source_id,
                                         int request_id) {
   Value dict(Value::Type::DICTIONARY);
-  dict.SetKey("type",
-              Value(EnumToString<CastMessageType,
-                                 CastMessageType::kReceiverStatusRequest>()));
+  dict.SetKey(
+      "type",
+      Value(EnumToString<CastMessageType, CastMessageType::kGetStatus>()));
   dict.SetKey("requestId", Value(request_id));
   return CreateCastMessage(kReceiverNamespace, dict, source_id,
                            kPlatformReceiverId);
@@ -387,17 +469,27 @@ CastMessage CreateBroadcastRequest(const std::string& source_id,
                            kPlatformReceiverId);
 }
 
-CastMessage CreateLaunchRequest(const std::string& source_id,
-                                int request_id,
-                                const std::string& app_id,
-                                const std::string& locale) {
+CastMessage CreateLaunchRequest(
+    const std::string& source_id,
+    int request_id,
+    const std::string& app_id,
+    const std::string& locale,
+    const std::vector<std::string>& supported_app_types,
+    const absl::optional<base::Value>& app_params) {
   Value dict(Value::Type::DICTIONARY);
   dict.SetKey("type",
               Value(EnumToString<CastMessageType, CastMessageType::kLaunch>()));
   dict.SetKey("requestId", Value(request_id));
   dict.SetKey("appId", Value(app_id));
   dict.SetKey("language", Value(locale));
+  std::vector<Value> supported_app_types_value;
+  for (const std::string& type : supported_app_types)
+    supported_app_types_value.push_back(Value(type));
 
+  dict.SetKey("supportedAppTypes", Value(supported_app_types_value));
+  if (app_params) {
+    dict.SetKey("appParams", app_params.value().Clone());
+  }
   return CreateCastMessage(kReceiverNamespace, dict, source_id,
                            kPlatformReceiverId);
 }
@@ -423,7 +515,19 @@ CastMessage CreateCastMessage(const std::string& message_namespace,
                               message_namespace);
   output.set_payload_type(
       CastMessage::PayloadType::CastMessage_PayloadType_STRING);
-  CHECK(base::JSONWriter::Write(message, output.mutable_payload_utf8()));
+  if (message.is_string()) {
+    // NOTE(jrw): This case is needed to fix crbug.com/149843471, which affects
+    // the ability to cast the Shaka player.  Without it, the payload of the
+    // first message sent with the urn:x-cast:com.google.shaka.v2 namespace is
+    // given an extra level of quoting.  It's not clear whether switching on the
+    // JSON type of the message is the right thing to do here, or if this case
+    // is simply compensating for some other problem that occurs between here
+    // and PresentationConnection::send(string, ...), which receives a value
+    // with the correct amount of quotation.
+    output.set_payload_utf8(message.GetString());
+  } else {
+    CHECK(base::JSONWriter::Write(message, output.mutable_payload_utf8()));
+  }
   return output;
 }
 
@@ -444,8 +548,7 @@ CastMessage CreateSetVolumeRequest(const base::Value& body,
                                    const std::string& source_id) {
   DCHECK(body.FindKeyOfType("type", Value::Type::STRING) &&
          body.FindKeyOfType("type", Value::Type::STRING)->GetString() ==
-             (EnumToString<V2MessageType, V2MessageType::kSetVolume>())
-                 .as_string());
+             (EnumToString<V2MessageType, V2MessageType::kSetVolume>()));
   Value dict = body.Clone();
   dict.RemoveKey("sessionId");
   dict.SetKey("requestId", Value(request_id));
@@ -467,6 +570,8 @@ bool IsMediaRequestMessageType(V2MessageType type) {
     case V2MessageType::kQueueRemove:
     case V2MessageType::kQueueReorder:
     case V2MessageType::kQueueUpdate:
+    case V2MessageType::kQueueNext:
+    case V2MessageType::kQueuePrev:
     case V2MessageType::kSeek:
     case V2MessageType::kStopMedia:
       return true;
@@ -477,16 +582,16 @@ bool IsMediaRequestMessageType(V2MessageType type) {
 
 // TODO(jrw): Eliminate this function.
 const char* ToString(GetAppAvailabilityResult result) {
-  return EnumToString(result).value_or(nullptr).data();
+  return EnumToString(result).value_or("").data();
 }
 
-base::Optional<int> GetRequestIdFromResponse(const Value& payload) {
+absl::optional<int> GetRequestIdFromResponse(const Value& payload) {
   DCHECK(payload.is_dict());
 
   const Value* request_id_value =
       payload.FindKeyOfType("requestId", Value::Type::INTEGER);
   if (!request_id_value)
-    return base::nullopt;
+    return absl::nullopt;
   return request_id_value->GetInt();
 }
 
@@ -503,11 +608,20 @@ GetAppAvailabilityResult GetAppAvailabilityResultFromResponse(
       .value_or(GetAppAvailabilityResult::kUnknown);
 }
 
-LaunchSessionResponse::LaunchSessionResponse() {}
+LaunchSessionResponse::LaunchSessionResponse() = default;
 LaunchSessionResponse::LaunchSessionResponse(LaunchSessionResponse&& other) =
     default;
+LaunchSessionResponse& LaunchSessionResponse::operator=(
+    LaunchSessionResponse&& other) = default;
 LaunchSessionResponse::~LaunchSessionResponse() = default;
 
+LaunchSessionResponse GetLaunchSessionResponseError(std::string error_msg) {
+  LaunchSessionResponse response;
+  response.result = LaunchSessionResponse::Result::kError;
+  response.error_msg = std::move(error_msg);
+  return response;
+}
+
 LaunchSessionResponse GetLaunchSessionResponse(const base::Value& payload) {
   const Value* type_value = payload.FindKeyOfType("type", Value::Type::STRING);
   if (!type_value)
@@ -534,4 +648,13 @@ LaunchSessionResponse GetLaunchSessionResponse(const base::Value& payload) {
   return response;
 }
 
+VirtualConnectionType GetConnectionType(const std::string& destination_id) {
+  // VCs to recevier-0 are invisible to the receiver application by design.
+  // We create a strong connection because some commands (e.g. LAUNCH) are
+  // not accepted from invisible connections.
+  return destination_id == kPlatformReceiverId
+             ? VirtualConnectionType::kStrong
+             : VirtualConnectionType::kInvisible;
+}
+
 }  // namespace cast_channel
