@@ -7,6 +7,7 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <memory>
 #include <string>
 #include <utility>
 
@@ -15,19 +16,32 @@
 #include "base/location.h"
 #include "base/numerics/safe_conversions.h"
 #include "base/single_thread_task_runner.h"
-#include "base/strings/stringprintf.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "components/cast_channel/cast_framer.h"
 #include "components/cast_channel/cast_message_util.h"
 #include "components/cast_channel/logger.h"
-#include "components/cast_channel/proto/cast_channel.pb.h"
 #include "net/base/net_errors.h"
+#include "third_party/openscreen/src/cast/common/channel/proto/cast_channel.pb.h"
 
 #define VLOG_WITH_CONNECTION(level) \
   VLOG(level) << "[" << ip_endpoint_.ToString() << ", auth=SSL_VERIFIED] "
 
 namespace cast_channel {
 
+namespace {
+
+#if DCHECK_IS_ON()
+// Used to filter out PING and PONG message from logs, since there are a lot of
+// them and they're not interesting.
+bool IsPingPong(const CastMessage& message) {
+  return message.has_payload_utf8() &&
+         (message.payload_utf8() == R"({"type":"PING"})" ||
+          message.payload_utf8() == R"({"type":"PONG"})");
+}
+#endif  // DCHECK_IS_ON()
+
+}  // namespace
+
 CastTransportImpl::CastTransportImpl(Channel* channel,
                                      int channel_id,
                                      const net::IPEndPoint& ip_endpoint,
@@ -40,13 +54,11 @@ CastTransportImpl::CastTransportImpl(Channel* channel,
       channel_id_(channel_id),
       ip_endpoint_(ip_endpoint),
       logger_(logger) {
-  DCHECK(socket);
-
   // Buffer is reused across messages to minimize unnecessary buffer
   // [re]allocations.
   read_buffer_ = base::MakeRefCounted<net::GrowableIOBuffer>();
   read_buffer_->SetCapacity(MessageFramer::MessageHeader::max_message_size());
-  framer_.reset(new MessageFramer(read_buffer_));
+  framer_ = std::make_unique<MessageFramer>(read_buffer_);
 }
 
 CastTransportImpl::~CastTransportImpl() {
@@ -63,7 +75,6 @@ bool CastTransportImpl::IsTerminalReadState(ReadState read_state) {
   return read_state == ReadState::READ_ERROR;
 }
 
-
 void CastTransportImpl::SetReadDelegate(std::unique_ptr<Delegate> delegate) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(delegate);
@@ -75,27 +86,26 @@ void CastTransportImpl::SetReadDelegate(std::unique_ptr<Delegate> delegate) {
 
 void CastTransportImpl::FlushWriteQueue() {
   for (; !write_queue_.empty(); write_queue_.pop()) {
-    net::CompletionCallback& callback = write_queue_.front().callback;
     base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::BindOnce(callback, net::ERR_FAILED));
-    callback.Reset();
+        FROM_HERE, base::BindOnce(std::move(write_queue_.front().callback),
+                                  net::ERR_FAILED));
   }
 }
 
 void CastTransportImpl::SendMessage(const CastMessage& message,
-                                    const net::CompletionCallback& callback) {
+                                    net::CompletionOnceCallback callback) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(IsCastMessageValid(message));
+  DVLOG_IF(1, !IsPingPong(message)) << "Sending: " << message;
   std::string serialized_message;
   if (!MessageFramer::Serialize(message, &serialized_message)) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::BindOnce(callback, net::ERR_FAILED));
+        FROM_HERE, base::BindOnce(std::move(callback), net::ERR_FAILED));
     return;
   }
-  WriteRequest write_request(message.namespace_(), serialized_message,
-                             callback);
 
-  write_queue_.push(write_request);
+  write_queue_.emplace(message.namespace_(), serialized_message,
+                       std::move(callback));
   if (write_state_ == WriteState::IDLE) {
     SetWriteState(WriteState::WRITE);
     OnWriteResult(net::OK);
@@ -105,15 +115,14 @@ void CastTransportImpl::SendMessage(const CastMessage& message,
 CastTransportImpl::WriteRequest::WriteRequest(
     const std::string& namespace_,
     const std::string& payload,
-    const net::CompletionCallback& callback)
-    : message_namespace(namespace_), callback(callback) {
+    net::CompletionOnceCallback callback)
+    : message_namespace(namespace_), callback(std::move(callback)) {
   VLOG(2) << "WriteRequest size: " << payload.size();
   io_buffer = base::MakeRefCounted<net::DrainableIOBuffer>(
       base::MakeRefCounted<net::StringIOBuffer>(payload), payload.size());
 }
 
-CastTransportImpl::WriteRequest::WriteRequest(const WriteRequest& other) =
-    default;
+CastTransportImpl::WriteRequest::WriteRequest(WriteRequest&& other) = default;
 
 CastTransportImpl::WriteRequest::~WriteRequest() {}
 
@@ -188,15 +197,16 @@ void CastTransportImpl::OnWriteResult(int result) {
 
 int CastTransportImpl::DoWrite() {
   DCHECK(!write_queue_.empty());
-  WriteRequest& request = write_queue_.front();
+  net::DrainableIOBuffer* io_buffer = write_queue_.front().io_buffer.get();
 
-  VLOG_WITH_CONNECTION(2) << "WriteData byte_count = "
-                          << request.io_buffer->size() << " bytes_written "
-                          << request.io_buffer->BytesConsumed();
+  VLOG_WITH_CONNECTION(2) << "WriteData byte_count = " << io_buffer->size()
+                          << " bytes_written " << io_buffer->BytesConsumed();
 
   SetWriteState(WriteState::WRITE_COMPLETE);
 
-  channel_->Write(request.io_buffer.get(), request.io_buffer->BytesRemaining(),
+  // TODO(mfoltz): Improve APIs for CastTransportImpl::Channel::{Read|Write} so
+  // that they don't expect raw pointers but handle movable parameters instead.
+  channel_->Write(io_buffer, io_buffer->BytesRemaining(),
                   base::BindOnce(&CastTransportImpl::OnWriteResult,
                                  base::Unretained(this)));
   return net::ERR_IO_PENDING;
@@ -214,8 +224,7 @@ int CastTransportImpl::DoWriteComplete(int result) {
   }
 
   // Some bytes were successfully written
-  WriteRequest& request = write_queue_.front();
-  scoped_refptr<net::DrainableIOBuffer> io_buffer = request.io_buffer;
+  net::DrainableIOBuffer* io_buffer = write_queue_.front().io_buffer.get();
   io_buffer->DidConsume(result);
   if (io_buffer->BytesRemaining() == 0) {  // Message fully sent
     SetWriteState(WriteState::DO_CALLBACK);
@@ -230,11 +239,11 @@ int CastTransportImpl::DoWriteCallback() {
   VLOG_WITH_CONNECTION(2) << "DoWriteCallback";
   DCHECK(!write_queue_.empty());
 
-  WriteRequest& request = write_queue_.front();
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(request.callback, net::OK));
-
+      FROM_HERE,
+      base::BindOnce(std::move(write_queue_.front().callback), net::OK));
   write_queue_.pop();
+
   if (write_queue_.empty()) {
     SetWriteState(WriteState::IDLE);
   } else {
@@ -361,6 +370,8 @@ int CastTransportImpl::DoReadCallback() {
     return net::ERR_INVALID_RESPONSE;
   }
   SetReadState(ReadState::READ);
+  DVLOG_IF(1, !IsPingPong(*current_message_))
+      << "Received: " << *current_message_;
   delegate_->OnMessage(*current_message_);
   current_message_.reset();
   return net::OK;
