@@ -4,8 +4,13 @@
 
 #include "components/cdm/renderer/widevine_key_system_properties.h"
 
+#include "base/command_line.h"
+#include "base/compiler_specific.h"
 #include "base/feature_list.h"
+#include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
 #include "media/base/media_switches.h"
+#include "media/media_buildflags.h"
 #include "third_party/widevine/cdm/buildflags.h"
 #include "third_party/widevine/cdm/widevine_cdm_common.h"
 
@@ -44,13 +49,12 @@ Robustness ConvertRobustness(const std::string& robustness) {
 
 WidevineKeySystemProperties::WidevineKeySystemProperties(
     media::SupportedCodecs codecs,
-    base::flat_set<media::EncryptionMode> encryption_schemes,
+    base::flat_set<media::EncryptionScheme> encryption_schemes,
     media::SupportedCodecs hw_secure_codecs,
-    base::flat_set<media::EncryptionMode> hw_secure_encryption_schemes,
+    base::flat_set<media::EncryptionScheme> hw_secure_encryption_schemes,
     Robustness max_audio_robustness,
     Robustness max_video_robustness,
     media::EmeSessionTypeSupport persistent_license_support,
-    media::EmeSessionTypeSupport persistent_release_message_support,
     media::EmeFeatureSupport persistent_state_support,
     media::EmeFeatureSupport distinctive_identifier_support)
     : codecs_(codecs),
@@ -60,7 +64,6 @@ WidevineKeySystemProperties::WidevineKeySystemProperties(
       max_audio_robustness_(max_audio_robustness),
       max_video_robustness_(max_video_robustness),
       persistent_license_support_(persistent_license_support),
-      persistent_release_message_support_(persistent_release_message_support),
       persistent_state_support_(persistent_state_support),
       distinctive_identifier_support_(distinctive_identifier_support) {}
 
@@ -84,7 +87,7 @@ bool WidevineKeySystemProperties::IsSupportedInitDataType(
 }
 
 EmeConfigRule WidevineKeySystemProperties::GetEncryptionSchemeConfigRule(
-    media::EncryptionMode encryption_scheme) const {
+    media::EncryptionScheme encryption_scheme) const {
   bool is_supported = encryption_schemes_.count(encryption_scheme);
   bool is_hw_secure_supported =
       hw_secure_encryption_schemes_.count(encryption_scheme);
@@ -100,23 +103,18 @@ EmeConfigRule WidevineKeySystemProperties::GetEncryptionSchemeConfigRule(
 }
 
 SupportedCodecs WidevineKeySystemProperties::GetSupportedCodecs() const {
-  // Disable AV1 if feature kWidevineAv1 is disabled
-  return base::FeatureList::IsEnabled(media::kWidevineAv1)
-             ? codecs_
-             : (codecs_ & ~media::EME_CODEC_AV1);
+  return codecs_;
 }
 
 SupportedCodecs WidevineKeySystemProperties::GetSupportedHwSecureCodecs()
     const {
-  // Disable AV1 if feature kWidevineAv1 is disabled
-  return base::FeatureList::IsEnabled(media::kWidevineAv1)
-             ? hw_secure_codecs_
-             : (hw_secure_codecs_ & ~media::EME_CODEC_AV1);
+  return hw_secure_codecs_;
 }
 
 EmeConfigRule WidevineKeySystemProperties::GetRobustnessConfigRule(
     EmeMediaType media_type,
-    const std::string& requested_robustness) const {
+    const std::string& requested_robustness,
+    const bool* hw_secure_requirement) const {
   Robustness robustness = ConvertRobustness(requested_robustness);
   if (robustness == Robustness::INVALID)
     return EmeConfigRule::NOT_SUPPORTED;
@@ -142,10 +140,24 @@ EmeConfigRule WidevineKeySystemProperties::GetRobustnessConfigRule(
     return EmeConfigRule::NOT_SUPPORTED;
   }
 
+  bool hw_secure_codecs_required =
+      hw_secure_requirement && *hw_secure_requirement;
 #if defined(OS_CHROMEOS)
-  // Hardware security requires remote attestation.
-  if (robustness >= Robustness::HW_SECURE_CRYPTO)
+  // Hardware security requires HWDRM or remote attestation, both of these
+  // require an identifier.
+  if (robustness >= Robustness::HW_SECURE_CRYPTO || hw_secure_codecs_required) {
+#if BUILDFLAG(USE_CHROMEOS_PROTECTED_MEDIA)
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+    if (!base::CommandLine::ForCurrentProcess()->HasSwitch(
+            switches::kLacrosUseChromeosProtectedMedia)) {
+      return EmeConfigRule::IDENTIFIER_REQUIRED;
+    }
+#endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
+    return EmeConfigRule::IDENTIFIER_AND_HW_SECURE_CODECS_REQUIRED;
+#else
     return EmeConfigRule::IDENTIFIER_REQUIRED;
+#endif
+  }
 
   // For video, recommend remote attestation if HW_SECURE_ALL is available,
   // regardless of the value of |robustness|, because it enables hardware
@@ -158,18 +170,22 @@ EmeConfigRule WidevineKeySystemProperties::GetRobustnessConfigRule(
   }
 #elif defined(OS_ANDROID)
   // On Android, require hardware secure codecs for SW_SECURE_DECODE and above.
-  if (robustness >= Robustness::SW_SECURE_DECODE) {
+  if (robustness >= Robustness::SW_SECURE_DECODE || hw_secure_codecs_required)
     return EmeConfigRule::HW_SECURE_CODECS_REQUIRED;
-  }
+#elif defined(OS_WIN)
+  // On Windows, hardware security uses MediaFoundation-based CDM which requires
+  // identifier and persistent state.
+  if (robustness >= Robustness::HW_SECURE_CRYPTO || hw_secure_codecs_required)
+    return EmeConfigRule::IDENTIFIER_PERSISTENCE_AND_HW_SECURE_CODECS_REQUIRED;
 #else
-  // On Linux/Mac/Win, require hardware secure codecs for HW_SECURE_CRYPTO and
+  // On other platforms, require hardware secure codecs for HW_SECURE_CRYPTO and
   // above.
-  if (robustness >= Robustness::HW_SECURE_CRYPTO) {
+  if (robustness >= Robustness::HW_SECURE_CRYPTO)
     return EmeConfigRule::HW_SECURE_CODECS_REQUIRED;
-  }
+
+  ALLOW_UNUSED_LOCAL(hw_secure_codecs_required);
 #endif  // defined(OS_CHROMEOS)
 
-  // TODO(crbug.com/848532): Handle HW_SECURE* levels for Windows.
   return EmeConfigRule::SUPPORTED;
 }
 
@@ -178,11 +194,6 @@ WidevineKeySystemProperties::GetPersistentLicenseSessionSupport() const {
   return persistent_license_support_;
 }
 
-EmeSessionTypeSupport
-WidevineKeySystemProperties::GetPersistentUsageRecordSessionSupport() const {
-  return persistent_release_message_support_;
-}
-
 EmeFeatureSupport WidevineKeySystemProperties::GetPersistentStateSupport()
     const {
   return persistent_state_support_;
