@@ -16,18 +16,18 @@
 #include <algorithm>
 #include <map>
 #include <memory>
+#include <string>
 #include <vector>
 
 #include "base/base_switches.h"
 #include "base/command_line.h"
+#include "base/cxx17_backports.h"
 #include "base/debug/crash_logging.h"
 #include "base/debug/dump_without_crashing.h"
 #include "base/environment.h"
 #include "base/files/file_path.h"
 #include "base/no_destructor.h"
 #include "base/numerics/safe_conversions.h"
-#include "base/stl_util.h"
-#include "base/strings/string16.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
@@ -35,8 +35,8 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/win/pe_image.h"
 #include "base/win/win_util.h"
-#include "components/crash/content/app/crash_reporter_client.h"
 #include "components/crash/content/app/hard_error_handler_win.h"
+#include "components/crash/core/app/crash_reporter_client.h"
 #include "components/crash/core/common/crash_keys.h"
 #include "content/public/common/result_codes.h"
 #include "sandbox/win/src/nt_internals.h"
@@ -113,7 +113,7 @@ google_breakpad::CustomClientInfo* GetCustomInfo(
     const std::wstring& profile_type,
     base::CommandLine* cmd_line,
     crash_reporter::CrashReporterClient* crash_client) {
-  base::string16 version, product, special_build, channel_name;
+  std::wstring version, product, special_build, channel_name;
   crash_client->GetProductNameAndVersion(exe_path, &product, &version,
                                          &special_build, &channel_name);
 
@@ -130,7 +130,7 @@ google_breakpad::CustomClientInfo* GetCustomInfo(
   custom_entries->push_back(
       google_breakpad::CustomInfoEntry(L"ptype", type.c_str()));
   custom_entries->push_back(google_breakpad::CustomInfoEntry(
-      L"pid", base::IntToString16(::GetCurrentProcessId()).c_str()));
+      L"pid", base::NumberToWString(::GetCurrentProcessId()).c_str()));
   custom_entries->push_back(
       google_breakpad::CustomInfoEntry(L"channel", channel_name.c_str()));
   custom_entries->push_back(
@@ -149,19 +149,18 @@ google_breakpad::CustomClientInfo* GetCustomInfo(
                            (cmd_line->HasSwitch(switches::kNoErrorDialogs) ||
                             crash_client->IsRunningUnattended());
   if (use_crash_service) {
-    base::string16 crash_dumps_dir_path;
+    std::wstring crash_dumps_dir_path;
     if (crash_client->GetAlternativeCrashDumpLocation(&crash_dumps_dir_path)) {
       custom_entries->push_back(google_breakpad::CustomInfoEntry(
           L"breakpad-dump-location", crash_dumps_dir_path.c_str()));
     }
   }
 
-  static base::NoDestructor<google_breakpad::CustomClientInfo>
-      custom_client_info;
-  custom_client_info->entries = &custom_entries->front();
-  custom_client_info->count = custom_entries->size();
+  static google_breakpad::CustomClientInfo custom_client_info;
+  custom_client_info.entries = &custom_entries->front();
+  custom_client_info.count = custom_entries->size();
 
-  return custom_client_info.get();
+  return &custom_client_info;
 }
 
 }  // namespace
@@ -338,8 +337,8 @@ static bool WrapMessageBoxWithSEH(const wchar_t* text, const wchar_t* caption,
 // spawned and basically just shows the 'chrome has crashed' dialog if
 // the CHROME_CRASHED environment variable is present.
 bool ShowRestartDialogIfCrashed(bool* exit_now) {
-  base::string16 message;
-  base::string16 title;
+  std::wstring message;
+  std::wstring title;
   bool is_rtl_locale;
   if (!GetCrashReporterClient()->ShouldShowRestartDialog(
           &title, &message, &is_rtl_locale)) {
@@ -372,9 +371,6 @@ extern "C" void __declspec(dllexport) TerminateProcessWithoutDump() {
 // Crashes the process after generating a dump for the provided exception. Note
 // that the crash reporter should be initialized before calling this function
 // for it to do anything.
-// NOTE: This function is used by SyzyASAN to invoke a crash. If you change the
-// the name or signature of this function you will break SyzyASAN instrumented
-// releases of Chrome. Please contact syzygy-team@chromium.org before doing so!
 extern "C" int __declspec(dllexport) CrashForException(
     EXCEPTION_POINTERS* info) {
   if (g_breakpad) {
@@ -502,7 +498,7 @@ static void InitPipeNameEnvVar(bool is_per_user_install) {
     pipe_name = kGoogleUpdatePipeName;
     pipe_name += user_sid;
   }
-  env->SetVar(kPipeNameVar, base::UTF16ToASCII(pipe_name));
+  env->SetVar(kPipeNameVar, base::WideToASCII(pipe_name));
 }
 
 void InitDefaultCrashCallback(LPTOP_LEVEL_EXCEPTION_FILTER filter) {
@@ -517,7 +513,7 @@ void InitCrashReporter(const std::string& process_type_switch) {
   // Disable the message box for assertions.
   _CrtSetReportMode(_CRT_ASSERT, 0);
 
-  base::string16 process_type = base::ASCIIToUTF16(process_type_switch);
+  std::wstring process_type = base::ASCIIToWide(process_type_switch);
   if (process_type.empty())
     process_type = L"browser";
 
@@ -555,7 +551,7 @@ void InitCrashReporter(const std::string& process_type_switch) {
       InitDefaultCrashCallback(default_filter);
     return;
   }
-  base::string16 pipe_name = base::ASCIIToUTF16(pipe_name_ascii);
+  std::wstring pipe_name = base::ASCIIToWide(pipe_name_ascii);
 
 #ifdef _WIN64
   // The protocol for connecting to the out-of-process Breakpad crash
@@ -634,74 +630,4 @@ ClearBreakpadPipeEnvironmentVariable() {
   env->UnSetVar(kPipeNameVar);
 }
 
-#ifdef _WIN64
-int CrashForExceptionInNonABICompliantCodeRange(
-    PEXCEPTION_RECORD ExceptionRecord,
-    ULONG64 EstablisherFrame,
-    PCONTEXT ContextRecord,
-    PDISPATCHER_CONTEXT DispatcherContext) {
-  EXCEPTION_POINTERS info = { ExceptionRecord, ContextRecord };
-  return CrashForException(&info);
-}
-
-struct ExceptionHandlerRecord {
-  RUNTIME_FUNCTION runtime_function;
-  UNWIND_INFO unwind_info;
-  unsigned char thunk[12];
-};
-
-extern "C" void __declspec(dllexport) __cdecl
-RegisterNonABICompliantCodeRange(void* start, size_t size_in_bytes) {
-  ExceptionHandlerRecord* record =
-      reinterpret_cast<ExceptionHandlerRecord*>(start);
-
-  // We assume that the first page of the code range is executable and
-  // committed and reserved for breakpad. What could possibly go wrong?
-
-  // All addresses are 32bit relative offsets to start.
-  record->runtime_function.BeginAddress = 0;
-  record->runtime_function.EndAddress =
-      base::checked_cast<DWORD>(size_in_bytes);
-  record->runtime_function.UnwindData =
-      offsetof(ExceptionHandlerRecord, unwind_info);
-
-  // Create unwind info that only specifies an exception handler.
-  record->unwind_info.Version = 1;
-  record->unwind_info.Flags = UNW_FLAG_EHANDLER;
-  record->unwind_info.SizeOfProlog = 0;
-  record->unwind_info.CountOfCodes = 0;
-  record->unwind_info.FrameRegister = 0;
-  record->unwind_info.FrameOffset = 0;
-  record->unwind_info.ExceptionHandler =
-      offsetof(ExceptionHandlerRecord, thunk);
-
-  // Hardcoded thunk.
-  // mov imm64, rax
-  record->thunk[0] = 0x48;
-  record->thunk[1] = 0xb8;
-  void* handler =
-      reinterpret_cast<void*>(&CrashForExceptionInNonABICompliantCodeRange);
-  memcpy(&record->thunk[2], &handler, 8);
-
-  // jmp rax
-  record->thunk[10] = 0xff;
-  record->thunk[11] = 0xe0;
-
-  // Protect reserved page against modifications.
-  DWORD old_protect;
-  CHECK(VirtualProtect(
-      start, sizeof(ExceptionHandlerRecord), PAGE_EXECUTE_READ, &old_protect));
-  CHECK(RtlAddFunctionTable(
-      &record->runtime_function, 1, reinterpret_cast<DWORD64>(start)));
-}
-
-extern "C" void __declspec(dllexport) __cdecl
-UnregisterNonABICompliantCodeRange(void* start) {
-  ExceptionHandlerRecord* record =
-      reinterpret_cast<ExceptionHandlerRecord*>(start);
-
-  CHECK(RtlDeleteFunctionTable(&record->runtime_function));
-}
-#endif
-
 }  // namespace breakpad
