@@ -10,10 +10,8 @@
 namespace data_reduction_proxy {
 namespace features {
 
-extern const base::Feature kDataReductionMainMenu;
-extern const base::Feature kDataReductionSiteBreakdown;
-extern const base::Feature kDataReductionProxyDecidesTransform;
 extern const base::Feature kDataReductionProxyLowMemoryDevicePromo;
+extern const base::Feature kReportSaveDataSavings;
 
 }  // namespace features
 }  // namespace data_reduction_proxy
