@@ -4,7 +4,7 @@
 
 #include "components/domain_reliability/quic_error_mapping.h"
 
-#include "base/stl_util.h"
+#include "base/cxx17_backports.h"
 
 namespace domain_reliability {
 
@@ -307,6 +307,123 @@ const struct QuicErrorMapping {
     {quic::QUIC_IETF_GQUIC_ERROR_MISSING, "quic.ietf.gquic.error_missing"},
     {quic::QUIC_WINDOW_UPDATE_RECEIVED_ON_READ_UNIDIRECTIONAL_STREAM,
      "quic.window.update.received.on.read.unidirectional.stream"},
+    {quic::QUIC_TOO_MANY_BUFFERED_CONTROL_FRAMES,
+     "quic.too.many.buffered.control.frames"},
+    {quic::QUIC_TRANSPORT_INVALID_CLIENT_INDICATION,
+     "quic.transport.invalid.client.indication"},
+
+    {quic::QUIC_QPACK_DECOMPRESSION_FAILED, "quic.qpack.decompression.failed"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_ERROR, "quic.qpack.encoder.stream.error"},
+    {quic::QUIC_QPACK_DECODER_STREAM_ERROR, "quic.qpack.decoder.stream.error"},
+
+    {quic::QUIC_QPACK_ENCODER_STREAM_INTEGER_TOO_LARGE,
+     "quic.qpack.encoder.stream.integer.too.large"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_STRING_LITERAL_TOO_LONG,
+     "quic.qpack.encoder.stream.string.literal.too.long"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_HUFFMAN_ENCODING_ERROR,
+     "quic.qpack.encoder.stream.huffman.encoding.error"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_INVALID_STATIC_ENTRY,
+     "quic.qpack.encoder.stream.invalid.static.entry"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_ERROR_INSERTING_STATIC,
+     "quic.qpack.encoder.stream.error.inserting.static"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_INSERTION_INVALID_RELATIVE_INDEX,
+     "quic.qpack.encoder.stream.insertion.invalid.relative.index"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_INSERTION_DYNAMIC_ENTRY_NOT_FOUND,
+     "quic.qpack.encoder.stream.insertion.dynamic.entry.not.found"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_ERROR_INSERTING_DYNAMIC,
+     "quic.qpack.encoder.stream.error.inserting.dynamic"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_ERROR_INSERTING_LITERAL,
+     "quic.qpack.encoder.stream.error.inserting.literal"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_DUPLICATE_INVALID_RELATIVE_INDEX,
+     "quic.qpack.encoder.stream.duplicate.invalid.relative.index"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_DUPLICATE_DYNAMIC_ENTRY_NOT_FOUND,
+     "quic.qpack.encoder.stream.duplicate.dynamic.entry.not.found"},
+    {quic::QUIC_QPACK_ENCODER_STREAM_SET_DYNAMIC_TABLE_CAPACITY,
+     "quic.qpack.encoder.stream.set.dynamic.table.capacity"},
+    {quic::QUIC_QPACK_DECODER_STREAM_INTEGER_TOO_LARGE,
+     "quic.qpack.decoder.stream.integer.too.large"},
+    {quic::QUIC_QPACK_DECODER_STREAM_INVALID_ZERO_INCREMENT,
+     "quic.qpack.decoder.stream.invalid.zero.increment"},
+    {quic::QUIC_QPACK_DECODER_STREAM_INCREMENT_OVERFLOW,
+     "quic.qpack.decoder.stream.increment.overflow"},
+    {quic::QUIC_QPACK_DECODER_STREAM_IMPOSSIBLE_INSERT_COUNT,
+     "quic.qpack.decoder.stream.impossible.insert.count"},
+    {quic::QUIC_QPACK_DECODER_STREAM_INCORRECT_ACKNOWLEDGEMENT,
+     "quic.qpack.decoder.stream.incorrect.acknowledgement"},
+
+    {quic::QUIC_STREAM_DATA_BEYOND_CLOSE_OFFSET,
+     "quic.stream.data.beyond.close.offset"},
+    {quic::QUIC_STREAM_MULTIPLE_OFFSET, "quic.stream.multiple.offset"},
+
+    {quic::QUIC_HTTP_FRAME_TOO_LARGE, "quic.http.frame.too.large,"},
+    {quic::QUIC_HTTP_FRAME_ERROR, "quic.http.frame.error"},
+    {quic::QUIC_HTTP_FRAME_UNEXPECTED_ON_SPDY_STREAM,
+     "quic.http.frame.unexpected.on.spdy.stream"},
+    {quic::QUIC_HTTP_FRAME_UNEXPECTED_ON_CONTROL_STREAM,
+     "quic.http.frame.unexpected.on.control.stream"},
+
+    {quic::QUIC_HPACK_INDEX_VARINT_ERROR, "quic.hpack.index_varint_error"},
+    {quic::QUIC_HPACK_NAME_LENGTH_VARINT_ERROR,
+     "quic.hpack.name_length_varint_error"},
+    {quic::QUIC_HPACK_VALUE_LENGTH_VARINT_ERROR,
+     "quic.hpack.value_length_varint_error"},
+    {quic::QUIC_HPACK_NAME_TOO_LONG, "quic.hpack.name_too_long"},
+    {quic::QUIC_HPACK_VALUE_TOO_LONG, "quic.hpack.value_too_long"},
+    {quic::QUIC_HPACK_NAME_HUFFMAN_ERROR, "quic.hpack.name_huffman_error"},
+    {quic::QUIC_HPACK_VALUE_HUFFMAN_ERROR, "quic.hpack.value_huffman_error"},
+    {quic::QUIC_HPACK_MISSING_DYNAMIC_TABLE_SIZE_UPDATE,
+     "quic.hpack.missing_dynamic_table_size_update"},
+    {quic::QUIC_HPACK_INVALID_INDEX, "quic.hpack.invalid_index"},
+    {quic::QUIC_HPACK_INVALID_NAME_INDEX, "quic.hpack.invalid_name_index"},
+    {quic::QUIC_HPACK_DYNAMIC_TABLE_SIZE_UPDATE_NOT_ALLOWED,
+     "quic.hpack.dynamic_table_size_update_not_allowed"},
+    {quic::QUIC_HPACK_INITIAL_TABLE_SIZE_UPDATE_IS_ABOVE_LOW_WATER_MARK,
+     "quic.hpack.initial_table_size_update_is_above_low_water_mark"},
+    {quic::QUIC_HPACK_TABLE_SIZE_UPDATE_IS_ABOVE_ACKNOWLEDGED_SETTING,
+     "quic.hpack.table_size_update_is_above_acknowledge_setting"},
+    {quic::QUIC_HPACK_TRUNCATED_BLOCK, "quic.hpack.truncated_block"},
+    {quic::QUIC_HPACK_FRAGMENT_TOO_LONG, "quic.hpack.fragment_too_long"},
+    {quic::QUIC_HPACK_COMPRESSED_HEADER_SIZE_EXCEEDS_LIMIT,
+     "quic.hpack.compressed_header_size_exceeds_limit"},
+    {quic::QUIC_HTTP_INVALID_FRAME_SEQUENCE_ON_SPDY_STREAM,
+     "quic.http_invalid_frame_sequence_on_spdy_stream"},
+    {quic::QUIC_HTTP_INVALID_FRAME_SEQUENCE_ON_CONTROL_STREAM,
+     "quic.http_invalid_frame_sequence_on_control_stream"},
+    {quic::QUIC_HTTP_DUPLICATE_UNIDIRECTIONAL_STREAM,
+     "quic.http_duplicate_unidirectional_stream"},
+    {quic::QUIC_HTTP_SERVER_INITIATED_BIDIRECTIONAL_STREAM,
+     "quic.http_server_initiated_bidirectional_stream"},
+    {quic::QUIC_HTTP_STREAM_WRONG_DIRECTION,
+     "quic.http_stream_wrong_direction"},
+    {quic::QUIC_HTTP_CLOSED_CRITICAL_STREAM,
+     "quic.http_closed_critical_stream"},
+    {quic::QUIC_HTTP_MISSING_SETTINGS_FRAME,
+     "quic.http_missing_settings_frame"},
+    {quic::QUIC_HTTP_DUPLICATE_SETTING_IDENTIFIER,
+     "quic.http_duplicate_setting_identifier"},
+    {quic::QUIC_HTTP_INVALID_MAX_PUSH_ID, "quic.http_invalid_max_push_id"},
+    {quic::QUIC_HTTP_STREAM_LIMIT_TOO_LOW, "quic.http_stream_limit_too_low"},
+    {quic::QUIC_ZERO_RTT_UNRETRANSMITTABLE, "quic.zero_rtt_unretransmittable"},
+    {quic::QUIC_ZERO_RTT_REJECTION_LIMIT_REDUCED,
+     "quic.zero_rtt_rejection_limit_reduced"},
+    {quic::QUIC_ZERO_RTT_RESUMPTION_LIMIT_REDUCED,
+     "quic.zero_rtt_resumption_limit_reduced"},
+    {quic::QUIC_HTTP_ZERO_RTT_RESUMPTION_SETTINGS_MISMATCH,
+     "quic.http_zero_rtt_resumption_settings_mismatch"},
+    {quic::QUIC_HTTP_ZERO_RTT_REJECTION_SETTINGS_MISMATCH,
+     "quic.http_zero_rtt_rejection_settings_mismatch"},
+    {quic::QUIC_HTTP_GOAWAY_INVALID_STREAM_ID,
+     "quic.quic_http_goaway_invalid_stream_id"},
+    {quic::QUIC_HTTP_GOAWAY_ID_LARGER_THAN_PREVIOUS,
+     "quic.quic_http_goaway_id_larger_than_previous"},
+    {quic::QUIC_SILENT_IDLE_TIMEOUT, "quic.silent_idle_timeout"},
+    {quic::QUIC_HTTP_RECEIVE_SPDY_SETTING, "quic.http_receive_spdy_setting"},
+    {quic::QUIC_MISSING_WRITE_KEYS, "quic.missing_write_keys"},
+    {quic::QUIC_HTTP_RECEIVE_SPDY_FRAME, "quic.http_receive_spdy_frame"},
+    {quic::QUIC_HTTP_RECEIVE_SERVER_PUSH, "quic.http_receive_server_push"},
+
+    {quic::QUIC_KEY_UPDATE_ERROR, "quic.quic_key_update_error"},
+    {quic::QUIC_AEAD_LIMIT_REACHED, "quic.quic_aead_limit_reached"},
 
     // QUIC_INVALID_APPLICATION_CLOSE_DATA was code 101. The code has been
     // deprecated, but to keep the assert below happy, there needs to be
@@ -314,6 +431,28 @@ const struct QuicErrorMapping {
     {static_cast<quic::QuicErrorCode>(101),
      "quic.invalid.application_close_data"},
 
+    {quic::QUIC_MAX_AGE_TIMEOUT, "quic.quic_max_age_timeout"},
+    {quic::QUIC_INVALID_0RTT_PACKET_NUMBER_OUT_OF_ORDER,
+     "quic.quic_invalid_0rtt_packet_number_out_of_order"},
+    {quic::QUIC_INVALID_PRIORITY_UPDATE, "quic::quic_invalid_priority_update"},
+    {quic::QUIC_PEER_PORT_CHANGE_HANDSHAKE_UNCONFIRMED,
+     "quic.peer_port_change_handshake_unconfirmed"},
+
+    {quic::QUIC_TLS_BAD_CERTIFICATE, "quic::quic_tls_bad_certificate"},
+    {quic::QUIC_TLS_UNSUPPORTED_CERTIFICATE,
+     "quic::quic_tls_unsupported_certificate"},
+    {quic::QUIC_TLS_CERTIFICATE_REVOKED, "quic::quic_tls_certificate_revoked"},
+    {quic::QUIC_TLS_CERTIFICATE_EXPIRED, "quic::quic_tls_certificate_expired"},
+    {quic::QUIC_TLS_CERTIFICATE_UNKNOWN, "quic::quic_tls_certificate_unknown"},
+    {quic::QUIC_TLS_INTERNAL_ERROR, "quic::quic_tls_internal_error"},
+    {quic::QUIC_TLS_UNRECOGNIZED_NAME, "quic::quic_tls_unrecognized_name"},
+    {quic::QUIC_TLS_CERTIFICATE_REQUIRED,
+     "quic::quic_tls_certificate_required"},
+    {quic::QUIC_CONNECTION_ID_LIMIT_ERROR,
+     "quic::quic_connection_id_limit_error"},
+    {quic::QUIC_TOO_MANY_CONNECTION_ID_WAITING_TO_RETIRE,
+     "quic::quic_too_many_connection_id_waiting_to_retire"},
+
     // No error. Used as bound while iterating.
     {quic::QUIC_LAST_ERROR, "quic.last_error"}};
 
