@@ -7,17 +7,12 @@
 #include <memory>
 
 #include "base/bind.h"
-#include "components/download/public/common/download_url_loader_factory_getter.h"
 #include "components/download/public/common/stream_handle_input_stream.h"
-#include "components/download/public/common/url_download_request_handle.h"
-#include "services/device/public/mojom/constants.mojom.h"
+#include "components/download/public/common/url_loader_factory_provider.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "services/device/public/mojom/wake_lock_provider.mojom.h"
 #include "services/network/public/cpp/shared_url_loader_factory.h"
-#include "services/service_manager/public/cpp/connector.h"
-
-namespace network {
-struct ResourceResponseHead;
-}
 
 namespace download {
 
@@ -31,9 +26,9 @@ class URLLoaderStatusMonitor : public network::mojom::URLLoaderClient {
   ~URLLoaderStatusMonitor() override = default;
 
   // network::mojom::URLLoaderClient
-  void OnReceiveResponse(const network::ResourceResponseHead& head) override {}
+  void OnReceiveResponse(network::mojom::URLResponseHeadPtr head) override {}
   void OnReceiveRedirect(const net::RedirectInfo& redirect_info,
-                         const network::ResourceResponseHead& head) override {}
+                         network::mojom::URLResponseHeadPtr head) override {}
   void OnUploadProgress(int64_t current_position,
                         int64_t total_size,
                         OnUploadProgressCallback callback) override {}
@@ -62,31 +57,29 @@ std::unique_ptr<ResourceDownloader> ResourceDownloader::BeginDownload(
     base::WeakPtr<UrlDownloadHandler::Delegate> delegate,
     std::unique_ptr<DownloadUrlParameters> params,
     std::unique_ptr<network::ResourceRequest> request,
-    scoped_refptr<download::DownloadURLLoaderFactoryGetter>
-        url_loader_factory_getter,
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
     const URLSecurityPolicy& url_security_policy,
     const GURL& site_url,
     const GURL& tab_url,
     const GURL& tab_referrer_url,
     bool is_new_download,
     bool is_parallel_request,
-    std::unique_ptr<service_manager::Connector> connector,
+    mojo::PendingRemote<device::mojom::WakeLockProvider> wake_lock_provider,
     bool is_background_mode,
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner) {
   auto downloader = std::make_unique<ResourceDownloader>(
       delegate, std::move(request), params->render_process_host_id(),
       params->render_frame_host_routing_id(), site_url, tab_url,
       tab_referrer_url, is_new_download, task_runner,
-      std::move(url_loader_factory_getter), url_security_policy,
-      std::move(connector));
+      std::move(url_loader_factory), url_security_policy,
+      std::move(wake_lock_provider));
 
   downloader->Start(std::move(params), is_parallel_request, is_background_mode);
   return downloader;
 }
 
 // static
-std::unique_ptr<ResourceDownloader>
-ResourceDownloader::InterceptNavigationResponse(
+void ResourceDownloader::InterceptNavigationResponse(
     base::WeakPtr<UrlDownloadHandler::Delegate> delegate,
     std::unique_ptr<network::ResourceRequest> resource_request,
     int render_process_id,
@@ -96,23 +89,29 @@ ResourceDownloader::InterceptNavigationResponse(
     const GURL& tab_referrer_url,
     std::vector<GURL> url_chain,
     net::CertStatus cert_status,
-    const scoped_refptr<network::ResourceResponse>& response_head,
+    network::mojom::URLResponseHeadPtr response_head,
     mojo::ScopedDataPipeConsumerHandle response_body,
     network::mojom::URLLoaderClientEndpointsPtr url_loader_client_endpoints,
-    scoped_refptr<download::DownloadURLLoaderFactoryGetter>
-        url_loader_factory_getter,
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
     const URLSecurityPolicy& url_security_policy,
-    std::unique_ptr<service_manager::Connector> connector,
+    mojo::PendingRemote<device::mojom::WakeLockProvider> wake_lock_provider,
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner) {
   auto downloader = std::make_unique<ResourceDownloader>(
       delegate, std::move(resource_request), render_process_id, render_frame_id,
       site_url, tab_url, tab_referrer_url, true, task_runner,
-      std::move(url_loader_factory_getter), url_security_policy,
-      std::move(connector));
-  downloader->InterceptResponse(
+      std::move(url_loader_factory), url_security_policy,
+      std::move(wake_lock_provider));
+  ResourceDownloader* raw_downloader = downloader.get();
+  task_runner->PostTask(
+      FROM_HERE,
+      base::BindOnce(
+          &UrlDownloadHandler::Delegate::OnUrlDownloadHandlerCreated, delegate,
+          UrlDownloadHandler::UniqueUrlDownloadHandlerPtr(
+              std::move(downloader).release(),
+              base::OnTaskRunnerDeleter(base::ThreadTaskRunnerHandle::Get()))));
+  raw_downloader->InterceptResponse(
       std::move(url_chain), cert_status, std::move(response_head),
       std::move(response_body), std::move(url_loader_client_endpoints));
-  return downloader;
 }
 
 ResourceDownloader::ResourceDownloader(
@@ -125,10 +124,9 @@ ResourceDownloader::ResourceDownloader(
     const GURL& tab_referrer_url,
     bool is_new_download,
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
-    scoped_refptr<download::DownloadURLLoaderFactoryGetter>
-        url_loader_factory_getter,
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
     const URLSecurityPolicy& url_security_policy,
-    std::unique_ptr<service_manager::Connector> connector)
+    mojo::PendingRemote<device::mojom::WakeLockProvider> wake_lock_provider)
     : delegate_(delegate),
       resource_request_(std::move(resource_request)),
       is_new_download_(is_new_download),
@@ -138,10 +136,14 @@ ResourceDownloader::ResourceDownloader(
       tab_url_(tab_url),
       tab_referrer_url_(tab_referrer_url),
       delegate_task_runner_(task_runner),
-      url_loader_factory_getter_(std::move(url_loader_factory_getter)),
+      url_loader_factory_(url_loader_factory),
       url_security_policy_(url_security_policy),
       is_content_initiated_(false) {
-  RequestWakeLock(connector.get());
+  if (wake_lock_provider) {
+    mojo::Remote<device::mojom::WakeLockProvider> provider(
+        std::move(wake_lock_provider));
+    RequestWakeLock(provider.get());
+  }
 }
 
 ResourceDownloader::~ResourceDownloader() = default;
@@ -150,7 +152,7 @@ void ResourceDownloader::Start(
     std::unique_ptr<DownloadUrlParameters> download_url_parameters,
     bool is_parallel_request,
     bool is_background_mode) {
-  callback_ = download_url_parameters->callback();
+  callback_ = std::move(download_url_parameters->callback());
   upload_callback_ = download_url_parameters->upload_callback();
   guid_ = download_url_parameters->guid();
   is_content_initiated_ = download_url_parameters->content_initiated();
@@ -162,26 +164,25 @@ void ResourceDownloader::Start(
           download_url_parameters->GetSaveInfo()),
       is_parallel_request, download_url_parameters->is_transient(),
       download_url_parameters->fetch_error_body(),
-      download_url_parameters->follow_cross_origin_redirects(),
+      download_url_parameters->cross_origin_redirects(),
       download_url_parameters->request_headers(),
       download_url_parameters->request_origin(),
       download_url_parameters->download_source(),
-      download_url_parameters->ignore_content_length_mismatch(),
       std::vector<GURL>(1, resource_request_->url), is_background_mode);
-  network::mojom::URLLoaderClientPtr url_loader_client_ptr;
-  url_loader_client_binding_ =
-      std::make_unique<mojo::Binding<network::mojom::URLLoaderClient>>(
-          url_loader_client_.get(), mojo::MakeRequest(&url_loader_client_ptr));
+
+  mojo::PendingRemote<network::mojom::URLLoaderClient> url_loader_client_remote;
+  url_loader_client_receiver_ =
+      std::make_unique<mojo::Receiver<network::mojom::URLLoaderClient>>(
+          url_loader_client_.get(),
+          url_loader_client_remote.InitWithNewPipeAndPassReceiver());
 
   // Set up the URLLoader
-  network::mojom::URLLoaderRequest url_loader_request =
-      mojo::MakeRequest(&url_loader_);
-  url_loader_factory_getter_->GetURLLoaderFactory()->CreateLoaderAndStart(
-      std::move(url_loader_request),
-      0,  // routing_id
+  url_loader_factory_->CreateLoaderAndStart(
+      url_loader_.BindNewPipeAndPassReceiver(),
       0,  // request_id
-      network::mojom::kURLLoadOptionSendSSLInfoWithResponse,
-      *(resource_request_.get()), std::move(url_loader_client_ptr),
+      network::mojom::kURLLoadOptionSendSSLInfoWithResponse |
+          network::mojom::kURLLoadOptionSniffMimeType,
+      *(resource_request_.get()), std::move(url_loader_client_remote),
       net::MutableNetworkTrafficAnnotationTag(
           download_url_parameters->GetNetworkTrafficAnnotation()));
   url_loader_->SetPriority(net::RequestPriority::IDLE,
@@ -191,7 +192,7 @@ void ResourceDownloader::Start(
 void ResourceDownloader::InterceptResponse(
     std::vector<GURL> url_chain,
     net::CertStatus cert_status,
-    const scoped_refptr<network::ResourceResponse>& response_head,
+    network::mojom::URLResponseHeadPtr response_head,
     mojo::ScopedDataPipeConsumerHandle response_body,
     network::mojom::URLLoaderClientEndpointsPtr endpoints) {
   // Set the URLLoader.
@@ -203,32 +204,26 @@ void ResourceDownloader::InterceptResponse(
       false, /* is_parallel_request */
       false, /* is_transient */
       false, /* fetch_error_body */
-      true,  /* follow_cross_origin_redirects */
+      network::mojom::RedirectMode::kFollow,
       download::DownloadUrlParameters::RequestHeadersType(),
       std::string(), /* request_origin */
-      download::DownloadSource::NAVIGATION,
-      false /* ignore_content_length_mismatch */, std::move(url_chain),
+      download::DownloadSource::NAVIGATION, std::move(url_chain),
       false /* is_background_mode */);
 
   // Simulate on the new URLLoaderClient calls that happened on the old client.
-  response_head->head.cert_status = cert_status;
-  url_loader_client_->OnReceiveResponse(response_head->head);
-
-  // Available when NavigationImmediateResponse is enabled.
-  if (response_body)
-    url_loader_client_->OnStartLoadingResponseBody(std::move(response_body));
+  response_head->cert_status = cert_status;
+  url_loader_client_->OnReceiveResponse(std::move(response_head));
+  url_loader_client_->OnStartLoadingResponseBody(std::move(response_body));
 
   // Bind the new client.
-  url_loader_client_binding_ =
-      std::make_unique<mojo::Binding<network::mojom::URLLoaderClient>>(
+  url_loader_client_receiver_ =
+      std::make_unique<mojo::Receiver<network::mojom::URLLoaderClient>>(
           url_loader_client_.get(), std::move(endpoints->url_loader_client));
 }
 
 void ResourceDownloader::OnResponseStarted(
     std::unique_ptr<DownloadCreateInfo> download_create_info,
     mojom::DownloadStreamHandlePtr stream_handle) {
-  download_create_info->request_handle.reset(new UrlDownloadRequestHandle(
-      weak_ptr_factory_.GetWeakPtr(), base::SequencedTaskRunnerHandle::Get()));
   download_create_info->is_new_download = is_new_download_;
   download_create_info->guid = guid_;
   download_create_info->site_url = site_url_;
@@ -238,6 +233,8 @@ void ResourceDownloader::OnResponseStarted(
   download_create_info->render_frame_id = render_frame_id_;
   download_create_info->has_user_gesture = resource_request_->has_user_gesture;
   download_create_info->is_content_initiated = is_content_initiated_;
+  download_create_info->transition_type =
+      ui::PageTransitionFromInt(resource_request_->transition_type);
 
   delegate_task_runner_->PostTask(
       FROM_HERE,
@@ -245,13 +242,18 @@ void ResourceDownloader::OnResponseStarted(
           &UrlDownloadHandler::Delegate::OnUrlDownloadStarted, delegate_,
           std::move(download_create_info),
           std::make_unique<StreamHandleInputStream>(std::move(stream_handle)),
-          std::move(url_loader_factory_getter_), callback_));
+          URLLoaderFactoryProvider::URLLoaderFactoryProviderPtr(
+              new URLLoaderFactoryProvider(url_loader_factory_),
+              base::OnTaskRunnerDeleter(base::ThreadTaskRunnerHandle::Get())),
+          this, std::move(callback_)));
 }
 
 void ResourceDownloader::OnReceiveRedirect() {
-  url_loader_->FollowRedirect(std::vector<std::string>() /* removed_headers */,
-                              net::HttpRequestHeaders() /* modified_headers */,
-                              base::nullopt);
+  url_loader_->FollowRedirect(
+      std::vector<std::string>() /* removed_headers */,
+      net::HttpRequestHeaders() /* modified_headers */,
+      net::HttpRequestHeaders() /* modified_cors_exempt_headers */,
+      absl::nullopt);
 }
 
 void ResourceDownloader::OnResponseCompleted() {
@@ -272,10 +274,6 @@ void ResourceDownloader::OnUploadProgress(uint64_t bytes_uploaded) {
       FROM_HERE, base::BindOnce(upload_callback_, bytes_uploaded));
 }
 
-void ResourceDownloader::CancelRequest() {
-  Destroy();
-}
-
 void ResourceDownloader::Destroy() {
   if (wake_lock_)
     wake_lock_->CancelWakeLock();
@@ -286,16 +284,11 @@ void ResourceDownloader::Destroy() {
 }
 
 void ResourceDownloader::RequestWakeLock(
-    service_manager::Connector* connector) {
-  if (!connector)
-    return;
-  device::mojom::WakeLockProviderPtr wake_lock_provider;
-  connector->BindInterface(device::mojom::kServiceName,
-                           mojo::MakeRequest(&wake_lock_provider));
-  wake_lock_provider->GetWakeLockWithoutContext(
+    device::mojom::WakeLockProvider* provider) {
+  provider->GetWakeLockWithoutContext(
       device::mojom::WakeLockType::kPreventAppSuspension,
       device::mojom::WakeLockReason::kOther, "Download in progress",
-      mojo::MakeRequest(&wake_lock_));
+      wake_lock_.BindNewPipeAndPassReceiver());
 
   wake_lock_->RequestWakeLock();
 }
