@@ -13,17 +13,18 @@
 #include "base/callback_helpers.h"
 #include "base/logging.h"
 #include "base/memory/weak_ptr.h"
-#include "base/stl_util.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "base/trace_event/trace_event.h"
 #include "base/trace_event/traced_value.h"
+#include "build/build_config.h"
 #include "components/exo/frame_sink_resource_manager.h"
 #include "components/viz/common/gpu/context_lost_observer.h"
 #include "components/viz/common/gpu/context_provider.h"
 #include "components/viz/common/resources/resource_format.h"
 #include "components/viz/common/resources/resource_format_utils.h"
-#include "components/viz/common/resources/single_release_callback.h"
+#include "components/viz/common/resources/resource_id.h"
+#include "components/viz/common/resources/returned_resource.h"
 #include "gpu/GLES2/gl2extchromium.h"
 #include "gpu/command_buffer/client/context_support.h"
 #include "gpu/command_buffer/client/raster_interface.h"
@@ -33,6 +34,7 @@
 #include "gpu/command_buffer/common/sync_token.h"
 #include "ui/aura/env.h"
 #include "ui/compositor/compositor.h"
+#include "ui/gfx/gpu_fence_handle.h"
 #include "ui/gfx/gpu_memory_buffer.h"
 
 namespace exo {
@@ -70,9 +72,8 @@ class Buffer::Texture : public viz::ContextLostObserver {
 
   // Allow texture to be reused after |sync_token| has passed and runs
   // |callback|.
-  void Release(base::OnceClosure callback,
-               const gpu::SyncToken& sync_token,
-               bool is_lost);
+  void Release(base::OnceCallback<void(gfx::GpuFenceHandle)> callback,
+               viz::ReturnedResource resource);
 
   // Updates the contents referenced by |gpu_memory_buffer_| returned by
   // mailbox().
@@ -83,9 +84,9 @@ class Buffer::Texture : public viz::ContextLostObserver {
 
   // Releases the contents referenced by |mailbox_| after |sync_token| has
   // passed and runs |callback| when completed.
-  void ReleaseSharedImage(base::OnceClosure callback,
-                          const gpu::SyncToken& sync_token,
-                          bool is_lost);
+  void ReleaseSharedImage(
+      base::OnceCallback<void(gfx::GpuFenceHandle)> callback,
+      viz::ReturnedResource resource);
 
   // Copy the contents of texture to |destination| and runs |callback| when
   // completed. Returns a sync token that can be used when accessing texture
@@ -115,7 +116,7 @@ class Buffer::Texture : public viz::ContextLostObserver {
   const base::TimeDelta wait_for_release_delay_;
   base::TimeTicks wait_for_release_time_;
   bool wait_for_release_pending_ = false;
-  base::WeakPtrFactory<Texture> weak_ptr_factory_;
+  base::WeakPtrFactory<Texture> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(Texture);
 };
@@ -127,14 +128,15 @@ Buffer::Texture::Texture(
       size_(size),
       context_provider_(std::move(context_provider)),
       texture_target_(GL_TEXTURE_2D),
-      query_type_(GL_COMMANDS_COMPLETED_CHROMIUM),
-      weak_ptr_factory_(this) {
+      query_type_(GL_COMMANDS_COMPLETED_CHROMIUM) {
   gpu::SharedImageInterface* sii = context_provider_->SharedImageInterface();
   const uint32_t usage =
       gpu::SHARED_IMAGE_USAGE_RASTER | gpu::SHARED_IMAGE_USAGE_DISPLAY;
 
   mailbox_ = sii->CreateSharedImage(viz::ResourceFormat::RGBA_8888, size,
-                                    gfx::ColorSpace(), usage);
+                                    gfx::ColorSpace(), kTopLeft_GrSurfaceOrigin,
+                                    kPremul_SkAlphaType, usage,
+                                    gpu::kNullSurfaceHandle);
   DCHECK(!mailbox_.IsZero());
   gpu::raster::RasterInterface* ri = context_provider_->RasterInterface();
   ri->WaitSyncTokenCHROMIUM(sii->GenUnverifiedSyncToken().GetConstData());
@@ -155,15 +157,15 @@ Buffer::Texture::Texture(
       context_provider_(std::move(context_provider)),
       texture_target_(texture_target),
       query_type_(query_type),
-      wait_for_release_delay_(wait_for_release_delay),
-      weak_ptr_factory_(this) {
+      wait_for_release_delay_(wait_for_release_delay) {
   gpu::SharedImageInterface* sii = context_provider_->SharedImageInterface();
   const uint32_t usage = gpu::SHARED_IMAGE_USAGE_RASTER |
                          gpu::SHARED_IMAGE_USAGE_DISPLAY |
                          gpu::SHARED_IMAGE_USAGE_SCANOUT;
 
   mailbox_ = sii->CreateSharedImage(
-      gpu_memory_buffer_, gpu_memory_buffer_manager, gfx::ColorSpace(), usage);
+      gpu_memory_buffer_, gpu_memory_buffer_manager, gfx::ColorSpace(),
+      kTopLeft_GrSurfaceOrigin, kPremul_SkAlphaType, usage);
   DCHECK(!mailbox_.IsZero());
   gpu::raster::RasterInterface* ri = context_provider_->RasterInterface();
   ri->WaitSyncTokenCHROMIUM(sii->GenUnverifiedSyncToken().GetConstData());
@@ -193,19 +195,20 @@ bool Buffer::Texture::IsLost() {
   return true;
 }
 
-void Buffer::Texture::Release(base::OnceClosure callback,
-                              const gpu::SyncToken& sync_token,
-                              bool is_lost) {
+void Buffer::Texture::Release(
+    base::OnceCallback<void(gfx::GpuFenceHandle)> callback,
+    viz::ReturnedResource resource) {
   if (context_provider_) {
-    if (sync_token.HasData()) {
+    // Only need to wait on the sync token if we don't have a release fence.
+    if (resource.sync_token.HasData() && resource.release_fence.is_null()) {
       gpu::raster::RasterInterface* ri = context_provider_->RasterInterface();
-      ri->WaitSyncTokenCHROMIUM(sync_token.GetConstData());
+      ri->WaitSyncTokenCHROMIUM(resource.sync_token.GetConstData());
     }
   }
 
   // Run callback as texture can be reused immediately after waiting for sync
   // token.
-  std::move(callback).Run();
+  std::move(callback).Run(std::move(resource.release_fence));
 }
 
 gpu::SyncToken Buffer::Texture::UpdateSharedImage(
@@ -227,25 +230,28 @@ gpu::SyncToken Buffer::Texture::UpdateSharedImage(
   return sync_token;
 }
 
-void Buffer::Texture::ReleaseSharedImage(base::OnceClosure callback,
-                                         const gpu::SyncToken& sync_token,
-                                         bool is_lost) {
-  if (context_provider_) {
+void Buffer::Texture::ReleaseSharedImage(
+    base::OnceCallback<void(gfx::GpuFenceHandle)> callback,
+    viz::ReturnedResource resource) {
+  // Only need to wait on the sync token and query if we don't have a release
+  // fence.
+  if (context_provider_ && resource.release_fence.is_null()) {
     gpu::raster::RasterInterface* ri = context_provider_->RasterInterface();
-    if (sync_token.HasData())
-      ri->WaitSyncTokenCHROMIUM(sync_token.GetConstData());
+    if (resource.sync_token.HasData())
+      ri->WaitSyncTokenCHROMIUM(resource.sync_token.GetConstData());
     ri->BeginQueryEXT(query_type_, query_id_);
     ri->EndQueryEXT(query_type_);
     // Run callback when query result is available (i.e., when all operations on
     // the shared image have completed and it's ready to be reused) if sync
     // token has data and buffer has been used. If buffer was never used then
     // run the callback immediately.
-    if (sync_token.HasData()) {
-      ReleaseWhenQueryResultIsAvailable(std::move(callback));
+    if (resource.sync_token.HasData()) {
+      ReleaseWhenQueryResultIsAvailable(base::BindOnce(
+          std::move(callback), /*release_fence=*/gfx::GpuFenceHandle()));
       return;
     }
   }
-  std::move(callback).Run();
+  std::move(callback).Run(std::move(resource.release_fence));
 }
 
 gpu::SyncToken Buffer::Texture::CopyTexImage(
@@ -266,7 +272,8 @@ gpu::SyncToken Buffer::Texture::CopyTexImage(
     ri->BeginQueryEXT(query_type_, query_id_);
     ri->CopySubTexture(mailbox_, destination->mailbox_,
                        destination->texture_target_, 0, 0, 0, 0, size_.width(),
-                       size_.height());
+                       size_.height(), /*unpack_flip_y=*/false,
+                       /*unpack_premultiply_alpha=*/false);
     ri->EndQueryEXT(query_type_);
     // Run callback when query result is available.
     ReleaseWhenQueryResultIsAvailable(std::move(callback));
@@ -350,6 +357,21 @@ void Buffer::Texture::WaitForRelease() {
   std::move(callback).Run();
 }
 
+Buffer::BufferRelease::BufferRelease(
+    gfx::GpuFenceHandle release_fence,
+    std::unique_ptr<base::FileDescriptorWatcher::Controller> controller,
+    base::OnceClosure buffer_release_callback)
+    : release_fence(std::move(release_fence)),
+      controller(std::move(controller)),
+      buffer_release_callback(std::move(buffer_release_callback)) {}
+
+Buffer::BufferRelease::~BufferRelease() = default;
+
+Buffer::BufferRelease::BufferRelease(BufferRelease&&) = default;
+
+Buffer::BufferRelease& Buffer::BufferRelease::operator=(BufferRelease&&) =
+    default;
+
 ////////////////////////////////////////////////////////////////////////////////
 // Buffer, public:
 
@@ -382,10 +404,12 @@ bool Buffer::ProduceTransferableResource(
     FrameSinkResourceManager* resource_manager,
     std::unique_ptr<gfx::GpuFence> acquire_fence,
     bool secure_output_only,
-    viz::TransferableResource* resource) {
+    viz::TransferableResource* resource,
+    PerCommitExplicitReleaseCallback per_commit_explicit_release_callback) {
   TRACE_EVENT1("exo", "Buffer::ProduceTransferableResource", "buffer_id",
-               gfx_buffer());
+               static_cast<const void*>(gfx_buffer()));
   DCHECK(attach_count_);
+  next_commit_id_++;
 
   // If textures are lost, destroy them to ensure that we create new ones below.
   if (contents_texture_ && contents_texture_->IsLost())
@@ -400,11 +424,18 @@ bool Buffer::ProduceTransferableResource(
       context_factory->SharedMainThreadRasterContextProvider();
   if (!context_provider) {
     DLOG(WARNING) << "Failed to acquire a context provider";
-    resource->id = 0;
+    resource->id = viz::kInvalidResourceId;
     resource->size = gfx::Size();
+    if (per_commit_explicit_release_callback)
+      std::move(per_commit_explicit_release_callback)
+          .Run(/*release_fence=*/gfx::GpuFenceHandle());
     return false;
   }
 
+  if (per_commit_explicit_release_callback)
+    pending_explicit_releases_.emplace(
+        next_commit_id_, std::move(per_commit_explicit_release_callback));
+
   resource->id = resource_manager->AllocateResourceId();
   resource->format = viz::RGBA_8888;
   resource->filter = GL_LINEAR;
@@ -423,7 +454,8 @@ bool Buffer::ProduceTransferableResource(
 
   if (release_contents_callback_.IsCancelled())
     TRACE_EVENT_ASYNC_BEGIN1("exo", kBufferInUse, gpu_memory_buffer_.get(),
-                             "buffer_id", gfx_buffer());
+                             "buffer_id",
+                             static_cast<const void*>(gfx_buffer()));
 
   // Cancel pending contents release callback.
   release_contents_callback_.Reset(
@@ -447,7 +479,8 @@ bool Buffer::ProduceTransferableResource(
                        base::Unretained(contents_texture),
                        base::BindOnce(&Buffer::ReleaseContentsTexture,
                                       AsWeakPtr(), std::move(contents_texture_),
-                                      release_contents_callback_.callback())));
+                                      release_contents_callback_.callback(),
+                                      next_commit_id_)));
     return true;
   }
 
@@ -465,7 +498,8 @@ bool Buffer::ProduceTransferableResource(
       std::move(acquire_fence), texture,
       base::BindOnce(&Buffer::ReleaseContentsTexture, AsWeakPtr(),
                      std::move(contents_texture_),
-                     release_contents_callback_.callback()));
+                     release_contents_callback_.callback(), next_commit_id_,
+                     /*release_fence=*/gfx::GpuFenceHandle()));
   resource->mailbox_holder =
       gpu::MailboxHolder(texture->mailbox(), sync_token, GL_TEXTURE_2D);
   resource->is_overlay_candidate = false;
@@ -483,15 +517,15 @@ bool Buffer::ProduceTransferableResource(
 void Buffer::OnAttach() {
   DLOG_IF(WARNING, attach_count_)
       << "Reattaching a buffer that is already attached to another surface.";
-  TRACE_EVENT2("exo", "Buffer::OnAttach", "buffer_id", gfx_buffer(), "count",
-               attach_count_);
+  TRACE_EVENT2("exo", "Buffer::OnAttach", "buffer_id",
+               static_cast<const void*>(gfx_buffer()), "count", attach_count_);
   ++attach_count_;
 }
 
 void Buffer::OnDetach() {
   DCHECK_GT(attach_count_, 0u);
-  TRACE_EVENT2("exo", "Buffer::OnAttach", "buffer_id", gfx_buffer(), "count",
-               attach_count_);
+  TRACE_EVENT2("exo", "Buffer::OnAttach", "buffer_id",
+               static_cast<const void*>(gfx_buffer()), "count", attach_count_);
   --attach_count_;
 
   // Release buffer if no longer attached to a surface and content has been
@@ -519,18 +553,25 @@ void Buffer::Release() {
     release_callback_.Run();
 }
 
-void Buffer::ReleaseTexture(std::unique_ptr<Texture> texture) {
+void Buffer::ReleaseTexture(std::unique_ptr<Texture> texture,
+                            gfx::GpuFenceHandle release_fence) {
+  // Buffer was composited - we should not receive a release fence.
+  DCHECK(release_fence.is_null());
   texture_ = std::move(texture);
 }
 
 void Buffer::ReleaseContentsTexture(std::unique_ptr<Texture> texture,
-                                    base::OnceClosure callback) {
+                                    base::OnceClosure callback,
+                                    uint64_t commit_id,
+                                    gfx::GpuFenceHandle release_fence) {
   contents_texture_ = std::move(texture);
-  std::move(callback).Run();
+  MaybeRunPerCommitRelease(commit_id, std::move(release_fence),
+                           std::move(callback));
 }
 
 void Buffer::ReleaseContents() {
-  TRACE_EVENT1("exo", "Buffer::ReleaseContents", "buffer_id", gfx_buffer());
+  TRACE_EVENT1("exo", "Buffer::ReleaseContents", "buffer_id",
+               static_cast<const void*>(gfx_buffer()));
 
   // Cancel callback to indicate that buffer has been released.
   release_contents_callback_.Cancel();
@@ -544,4 +585,40 @@ void Buffer::ReleaseContents() {
   }
 }
 
+void Buffer::MaybeRunPerCommitRelease(
+    uint64_t commit_id,
+    gfx::GpuFenceHandle release_fence,
+    base::OnceClosure buffer_release_callback) {
+  auto iter = pending_explicit_releases_.find(commit_id);
+  if (iter != pending_explicit_releases_.end()) {
+    std::move(iter->second).Run(release_fence.Clone());
+    pending_explicit_releases_.erase(iter);
+  }
+
+  // We are still required to send these wl_buffer.release events even if
+  // the client supports explicit synchronization.
+  if (release_fence.is_null()) {
+    std::move(buffer_release_callback).Run();
+  } else {
+#if defined(OS_POSIX)
+    auto controller = base::FileDescriptorWatcher::WatchReadable(
+        release_fence.owned_fd.get(),
+        base::BindRepeating(&Buffer::FenceSignalled, AsWeakPtr(), commit_id));
+    buffer_releases_.emplace(
+        commit_id,
+        BufferRelease(std::move(release_fence), std::move(controller),
+                      std::move(buffer_release_callback)));
+#else
+    NOTREACHED();
+#endif
+  }
+}
+
+void Buffer::FenceSignalled(uint64_t commit_id) {
+  auto iter = buffer_releases_.find(commit_id);
+  DCHECK(iter != buffer_releases_.end());
+  std::move(iter->second.buffer_release_callback).Run();
+  buffer_releases_.erase(iter);
+}
+
 }  // namespace exo
