@@ -4,7 +4,8 @@
 
 #include "components/history/content/browser/download_conversions.h"
 
-#include "base/logging.h"
+#include "base/check_op.h"
+#include "base/notreached.h"
 #include "components/download/public/common/download_danger_type.h"
 #include "components/history/core/browser/download_constants.h"
 
@@ -70,8 +71,8 @@ download::DownloadDangerType ToContentDownloadDangerType(
       return download::DOWNLOAD_DANGER_TYPE_DANGEROUS_HOST;
     case DownloadDangerType::POTENTIALLY_UNWANTED:
       return download::DOWNLOAD_DANGER_TYPE_POTENTIALLY_UNWANTED;
-    case DownloadDangerType::WHITELISTED_BY_POLICY:
-      return download::DOWNLOAD_DANGER_TYPE_WHITELISTED_BY_POLICY;
+    case DownloadDangerType::ALLOWLISTED_BY_POLICY:
+      return download::DOWNLOAD_DANGER_TYPE_ALLOWLISTED_BY_POLICY;
     case DownloadDangerType::ASYNC_SCANNING:
       return download::DOWNLOAD_DANGER_TYPE_ASYNC_SCANNING;
     case DownloadDangerType::BLOCKED_PASSWORD_PROTECTED:
@@ -86,6 +87,12 @@ download::DownloadDangerType ToContentDownloadDangerType(
       return download::DOWNLOAD_DANGER_TYPE_DEEP_SCANNED_SAFE;
     case DownloadDangerType::DEEP_SCANNED_OPENED_DANGEROUS:
       return download::DOWNLOAD_DANGER_TYPE_DEEP_SCANNED_OPENED_DANGEROUS;
+    case DownloadDangerType::PROMPT_FOR_SCANNING:
+      return download::DOWNLOAD_DANGER_TYPE_PROMPT_FOR_SCANNING;
+    case DownloadDangerType::BLOCKED_UNSUPPORTED_FILETYPE:
+      return download::DOWNLOAD_DANGER_TYPE_BLOCKED_UNSUPPORTED_FILETYPE;
+    case DownloadDangerType::DANGEROUS_ACCOUNT_COMRPOMISE:
+      return download::DOWNLOAD_DANGER_TYPE_DANGEROUS_ACCOUNT_COMPROMISE;
     case DownloadDangerType::INVALID:
       NOTREACHED();
       return download::DOWNLOAD_DANGER_TYPE_MAX;
@@ -115,8 +122,8 @@ DownloadDangerType ToHistoryDownloadDangerType(
       return DownloadDangerType::DANGEROUS_HOST;
     case download::DOWNLOAD_DANGER_TYPE_POTENTIALLY_UNWANTED:
       return DownloadDangerType::POTENTIALLY_UNWANTED;
-    case download::DOWNLOAD_DANGER_TYPE_WHITELISTED_BY_POLICY:
-      return DownloadDangerType::WHITELISTED_BY_POLICY;
+    case download::DOWNLOAD_DANGER_TYPE_ALLOWLISTED_BY_POLICY:
+      return DownloadDangerType::ALLOWLISTED_BY_POLICY;
     case download::DOWNLOAD_DANGER_TYPE_ASYNC_SCANNING:
       return DownloadDangerType::ASYNC_SCANNING;
     case download::DOWNLOAD_DANGER_TYPE_BLOCKED_PASSWORD_PROTECTED:
@@ -131,7 +138,12 @@ DownloadDangerType ToHistoryDownloadDangerType(
       return DownloadDangerType::DEEP_SCANNED_SAFE;
     case download::DOWNLOAD_DANGER_TYPE_DEEP_SCANNED_OPENED_DANGEROUS:
       return DownloadDangerType::DEEP_SCANNED_OPENED_DANGEROUS;
-
+    case download::DOWNLOAD_DANGER_TYPE_PROMPT_FOR_SCANNING:
+      return DownloadDangerType::PROMPT_FOR_SCANNING;
+    case download::DOWNLOAD_DANGER_TYPE_BLOCKED_UNSUPPORTED_FILETYPE:
+      return DownloadDangerType::BLOCKED_UNSUPPORTED_FILETYPE;
+    case download::DOWNLOAD_DANGER_TYPE_DANGEROUS_ACCOUNT_COMPROMISE:
+      return DownloadDangerType::DANGEROUS_ACCOUNT_COMRPOMISE;
     default:
       NOTREACHED();
       return DownloadDangerType::INVALID;
