@@ -17,12 +17,11 @@ class HistoryService;
 class TypedURLModelTypeController : public syncer::ModelTypeController {
  public:
   TypedURLModelTypeController(HistoryService* history_service,
-                              PrefService* pref_service,
-                              const char* history_disabled_pref_name);
+                              PrefService* pref_service);
   ~TypedURLModelTypeController() override;
 
   // syncer::DataTypeController implementation.
-  bool ReadyForStart() const override;
+  PreconditionState GetPreconditionState() const override;
 
  private:
   void OnSavingBrowserHistoryDisabledChanged();
@@ -30,9 +29,6 @@ class TypedURLModelTypeController : public syncer::ModelTypeController {
   HistoryService* const history_service_;
   PrefService* const pref_service_;
 
-  // Name of the pref that indicates whether saving history is disabled.
-  const char* const history_disabled_pref_name_;
-
   PrefChangeRegistrar pref_registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(TypedURLModelTypeController);
