@@ -7,8 +7,6 @@
 #include <set>
 #include <vector>
 
-#include "base/files/file_path.h"
-#include "base/files/scoped_temp_dir.h"
 #include "base/strings/string_util.h"
 #include "base/time/time.h"
 #include "components/history/core/browser/url_database.h"
@@ -17,6 +15,7 @@
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
+#include "url/origin.h"
 
 using base::Time;
 using base::TimeDelta;
@@ -29,10 +28,8 @@ namespace history {
 
 namespace {
 
-bool IsVisitInfoEqual(const VisitRow& a,
-                      const VisitRow& b) {
-  return a.visit_id == b.visit_id &&
-         a.url_id == b.url_id &&
+bool IsVisitInfoEqual(const VisitRow& a, const VisitRow& b) {
+  return a.visit_id == b.visit_id && a.url_id == b.url_id &&
          a.visit_time == b.visit_time &&
          a.referring_visit == b.referring_visit &&
          ui::PageTransitionTypeIncludingQualifiersIs(a.transition,
@@ -45,17 +42,14 @@ class VisitDatabaseTest : public PlatformTest,
                           public URLDatabase,
                           public VisitDatabase {
  public:
-  VisitDatabaseTest() {
-  }
+  VisitDatabaseTest() {}
 
  private:
   // Test setup.
   void SetUp() override {
     PlatformTest::SetUp();
-    ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
-    base::FilePath db_file = temp_dir_.GetPath().AppendASCII("VisitTest.db");
 
-    EXPECT_TRUE(db_.Open(db_file));
+    EXPECT_TRUE(db_.OpenInMemory());
 
     // Initialize the tables for this test.
     CreateURLTable(false);
@@ -70,24 +64,24 @@ class VisitDatabaseTest : public PlatformTest,
   // Provided for URL/VisitDatabase.
   sql::Database& GetDB() override { return db_; }
 
-  base::ScopedTempDir temp_dir_;
   sql::Database db_;
 };
 
 TEST_F(VisitDatabaseTest, Add) {
   // Add one visit.
-  VisitRow visit_info1(1, Time::Now(), 0, ui::PAGE_TRANSITION_LINK, 0, false);
+  VisitRow visit_info1(1, Time::Now(), 0, ui::PAGE_TRANSITION_LINK, 0, false,
+                       false);
   EXPECT_TRUE(AddVisit(&visit_info1, SOURCE_BROWSED));
 
   // Add second visit for the same page.
   VisitRow visit_info2(visit_info1.url_id,
                        visit_info1.visit_time + TimeDelta::FromSeconds(1), 1,
-                       ui::PAGE_TRANSITION_TYPED, 0, true);
+                       ui::PAGE_TRANSITION_TYPED, 0, true, false);
   EXPECT_TRUE(AddVisit(&visit_info2, SOURCE_BROWSED));
 
   // Add third visit for a different page.
   VisitRow visit_info3(2, visit_info1.visit_time + TimeDelta::FromSeconds(2), 0,
-                       ui::PAGE_TRANSITION_LINK, 0, false);
+                       ui::PAGE_TRANSITION_LINK, 0, false, false);
   EXPECT_TRUE(AddVisit(&visit_info3, SOURCE_BROWSED));
 
   // Query the first two.
@@ -106,17 +100,17 @@ TEST_F(VisitDatabaseTest, Delete) {
   // should link them.
   static const int kTime1 = 1000;
   VisitRow visit_info1(1, Time::FromInternalValue(kTime1), 0,
-                       ui::PAGE_TRANSITION_LINK, 0, false);
+                       ui::PAGE_TRANSITION_LINK, 0, false, false);
   EXPECT_TRUE(AddVisit(&visit_info1, SOURCE_BROWSED));
 
   static const int kTime2 = kTime1 + 1;
   VisitRow visit_info2(1, Time::FromInternalValue(kTime2), visit_info1.visit_id,
-                       ui::PAGE_TRANSITION_LINK, 0, false);
+                       ui::PAGE_TRANSITION_LINK, 0, false, false);
   EXPECT_TRUE(AddVisit(&visit_info2, SOURCE_BROWSED));
 
   static const int kTime3 = kTime2 + 1;
   VisitRow visit_info3(1, Time::FromInternalValue(kTime3), visit_info2.visit_id,
-                       ui::PAGE_TRANSITION_LINK, 0, false);
+                       ui::PAGE_TRANSITION_LINK, 0, false, false);
   EXPECT_TRUE(AddVisit(&visit_info3, SOURCE_BROWSED));
 
   // First make sure all the visits are there.
@@ -142,7 +136,7 @@ TEST_F(VisitDatabaseTest, Delete) {
 
 TEST_F(VisitDatabaseTest, Update) {
   // Make something in the database.
-  VisitRow original(1, Time::Now(), 23, ui::PageTransitionFromInt(0), 19,
+  VisitRow original(1, Time::Now(), 23, ui::PageTransitionFromInt(0), 19, false,
                     false);
   AddVisit(&original, SOURCE_BROWSED);
 
@@ -175,7 +169,7 @@ std::vector<VisitRow> GetTestVisitRows() {
       ui::PageTransitionFromInt(ui::PAGE_TRANSITION_LINK |
                                 ui::PAGE_TRANSITION_CHAIN_START |
                                 ui::PAGE_TRANSITION_CHAIN_END),
-      0, false);
+      0, false, false);
   visit_info1.visit_id = 1;
 
   // Add second visit for the same page.
@@ -184,7 +178,7 @@ std::vector<VisitRow> GetTestVisitRows() {
       ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
                                 ui::PAGE_TRANSITION_CHAIN_START |
                                 ui::PAGE_TRANSITION_CHAIN_END),
-      0, true);
+      0, true, false);
   visit_info2.visit_id = 2;
 
   // Add third visit for a different page.
@@ -192,7 +186,7 @@ std::vector<VisitRow> GetTestVisitRows() {
       2, visit_info1.visit_time + TimeDelta::FromSeconds(2), 0,
       ui::PageTransitionFromInt(ui::PAGE_TRANSITION_LINK |
                                 ui::PAGE_TRANSITION_CHAIN_START),
-      0, false);
+      0, false, false);
   visit_info3.visit_id = 3;
 
   // Add a redirect visit from the last page.
@@ -201,7 +195,7 @@ std::vector<VisitRow> GetTestVisitRows() {
       visit_info3.visit_id,
       ui::PageTransitionFromInt(ui::PAGE_TRANSITION_SERVER_REDIRECT |
                                 ui::PAGE_TRANSITION_CHAIN_END),
-      0, false);
+      0, false, false);
   visit_info4.visit_id = 4;
 
   // Add a subframe visit.
@@ -211,7 +205,7 @@ std::vector<VisitRow> GetTestVisitRows() {
       ui::PageTransitionFromInt(ui::PAGE_TRANSITION_AUTO_SUBFRAME |
                                 ui::PAGE_TRANSITION_CHAIN_START |
                                 ui::PAGE_TRANSITION_CHAIN_END),
-      0, false);
+      0, false, false);
   visit_info5.visit_id = 5;
 
   // Add third visit for the same URL as visit 1 and 2, but exactly a day
@@ -221,7 +215,7 @@ std::vector<VisitRow> GetTestVisitRows() {
       ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
                                 ui::PAGE_TRANSITION_CHAIN_START |
                                 ui::PAGE_TRANSITION_CHAIN_END),
-      0, true);
+      0, true, false);
   visit_info6.visit_id = 6;
 
   std::vector<VisitRow> test_visit_rows;
@@ -283,8 +277,7 @@ TEST_F(VisitDatabaseTest, GetAllVisitsInRange) {
   // Query a time range and make sure beginning is inclusive and ending is
   // exclusive.
   GetAllVisitsInRange(test_visit_rows[1].visit_time,
-                      test_visit_rows[3].visit_time, 0,
-                      &results);
+                      test_visit_rows[3].visit_time, 0, &results);
   ASSERT_EQ(static_cast<size_t>(2), results.size());
   EXPECT_TRUE(IsVisitInfoEqual(results[0], test_visit_rows[1]));
   EXPECT_TRUE(IsVisitInfoEqual(results[1], test_visit_rows[2]));
@@ -369,13 +362,16 @@ TEST_F(VisitDatabaseTest, GetAllURLIDsForTransition) {
 
 TEST_F(VisitDatabaseTest, VisitSource) {
   // Add visits.
-  VisitRow visit_info1(111, Time::Now(), 0, ui::PAGE_TRANSITION_LINK, 0, false);
+  VisitRow visit_info1(111, Time::Now(), 0, ui::PAGE_TRANSITION_LINK, 0, false,
+                       false);
   ASSERT_TRUE(AddVisit(&visit_info1, SOURCE_BROWSED));
 
-  VisitRow visit_info2(112, Time::Now(), 1, ui::PAGE_TRANSITION_TYPED, 0, true);
+  VisitRow visit_info2(112, Time::Now(), 1, ui::PAGE_TRANSITION_TYPED, 0, true,
+                       false);
   ASSERT_TRUE(AddVisit(&visit_info2, SOURCE_SYNCED));
 
-  VisitRow visit_info3(113, Time::Now(), 0, ui::PAGE_TRANSITION_TYPED, 0, true);
+  VisitRow visit_info3(113, Time::Now(), 0, ui::PAGE_TRANSITION_TYPED, 0, true,
+                       false);
   ASSERT_TRUE(AddVisit(&visit_info3, SOURCE_EXTENSION));
 
   // Query each visit.
@@ -443,33 +439,32 @@ TEST_F(VisitDatabaseTest, GetHistoryCount) {
   Time now = two_days_ago;
 
   ui::PageTransition standard_transition = ui::PageTransitionFromInt(
-      ui::PAGE_TRANSITION_TYPED |
-      ui::PAGE_TRANSITION_CHAIN_START |
+      ui::PAGE_TRANSITION_TYPED | ui::PAGE_TRANSITION_CHAIN_START |
       ui::PAGE_TRANSITION_CHAIN_END);
 
   // Add 5 visits (3 distinct URLs) for the day before yesterday.
   // Whether the URL was browsed on this machine or synced has no effect.
-  VisitRow first_day_1(1, now, 0, standard_transition, 0, true);
+  VisitRow first_day_1(1, now, 0, standard_transition, 0, true, false);
   first_day_1.visit_id = 1;
   AddVisit(&first_day_1, SOURCE_BROWSED);
   now += TimeDelta::FromHours(1);
 
-  VisitRow first_day_2(2, now, 0, standard_transition, 0, true);
+  VisitRow first_day_2(2, now, 0, standard_transition, 0, true, false);
   first_day_2.visit_id = 2;
   AddVisit(&first_day_2, SOURCE_BROWSED);
   now += TimeDelta::FromHours(1);
 
-  VisitRow first_day_3(1, now, 0, standard_transition, 0, true);
+  VisitRow first_day_3(1, now, 0, standard_transition, 0, true, false);
   first_day_3.visit_id = 3;
   AddVisit(&first_day_3, SOURCE_SYNCED);
   now += TimeDelta::FromHours(1);
 
-  VisitRow first_day_4(3, now, 0, standard_transition, 0, true);
+  VisitRow first_day_4(3, now, 0, standard_transition, 0, true, false);
   first_day_4.visit_id = 4;
   AddVisit(&first_day_4, SOURCE_SYNCED);
   now += TimeDelta::FromHours(1);
 
-  VisitRow first_day_5(2, now, 0, standard_transition, 0, true);
+  VisitRow first_day_5(2, now, 0, standard_transition, 0, true, false);
   first_day_5.visit_id = 5;
   AddVisit(&first_day_5, SOURCE_BROWSED);
   now += TimeDelta::FromHours(1);
@@ -478,22 +473,23 @@ TEST_F(VisitDatabaseTest, GetHistoryCount) {
   // a user-visible navigation. Of the remaining 3, only 2 are unique.
   now = yesterday;
 
-  VisitRow second_day_1(1, now, 0, standard_transition, 0, true);
+  VisitRow second_day_1(1, now, 0, standard_transition, 0, true, false);
   second_day_1.visit_id = 6;
   AddVisit(&second_day_1, SOURCE_BROWSED);
   now += TimeDelta::FromHours(1);
 
-  VisitRow second_day_2(1, now, 0, standard_transition, 0, true);
+  VisitRow second_day_2(1, now, 0, standard_transition, 0, true, false);
   second_day_2.visit_id = 7;
   AddVisit(&second_day_2, SOURCE_BROWSED);
   now += TimeDelta::FromHours(1);
 
-  VisitRow second_day_3(2, now, 0, ui::PAGE_TRANSITION_AUTO_SUBFRAME, 0, false);
+  VisitRow second_day_3(2, now, 0, ui::PAGE_TRANSITION_AUTO_SUBFRAME, 0, false,
+                        false);
   second_day_3.visit_id = 8;
   AddVisit(&second_day_3, SOURCE_BROWSED);
   now += TimeDelta::FromHours(1);
 
-  VisitRow second_day_4(3, now, 0, standard_transition, 0, true);
+  VisitRow second_day_4(3, now, 0, standard_transition, 0, true, false);
   second_day_4.visit_id = 9;
   AddVisit(&second_day_4, SOURCE_BROWSED);
   now += TimeDelta::FromHours(1);
@@ -513,29 +509,27 @@ TEST_F(VisitDatabaseTest, GetHistoryCount) {
   EXPECT_TRUE(GetHistoryCount(Time(), Time::Max(), &result));
   EXPECT_EQ(5, result);
 
-  // Narrowing the range to exclude |first_day_1| will still return 5,
-  // because |first_day_1| is not unique.
-  EXPECT_TRUE(GetHistoryCount(
-      two_days_ago + TimeDelta::FromHours(2), today, &result));
+  // Narrowing the range to exclude `first_day_1` will still return 5,
+  // because `first_day_1` is not unique.
+  EXPECT_TRUE(
+      GetHistoryCount(two_days_ago + TimeDelta::FromHours(2), today, &result));
   EXPECT_EQ(5, result);
 
-  // Narrowing the range to exclude |second_day_4| will return 4,
-  // because |second_day_4| is unique.
-  EXPECT_TRUE(GetHistoryCount(
-      two_days_ago, yesterday + TimeDelta::FromHours(3), &result));
+  // Narrowing the range to exclude `second_day_4` will return 4,
+  // because `second_day_4` is unique.
+  EXPECT_TRUE(GetHistoryCount(two_days_ago, yesterday + TimeDelta::FromHours(3),
+                              &result));
   EXPECT_EQ(4, result);
 
-  // Narrowing the range to exclude both |first_day_1| and |second_day_4| will
+  // Narrowing the range to exclude both `first_day_1` and `second_day_4` will
   // still return 4.
   EXPECT_TRUE(GetHistoryCount(two_days_ago + TimeDelta::FromHours(2),
-                              yesterday + TimeDelta::FromHours(3),
-                              &result));
+                              yesterday + TimeDelta::FromHours(3), &result));
   EXPECT_EQ(4, result);
 
   // A range that contains no visits will return 0.
   EXPECT_TRUE(GetHistoryCount(two_days_ago + TimeDelta::FromMicroseconds(1),
-                              two_days_ago + TimeDelta::FromHours(1),
-                              &result));
+                              two_days_ago + TimeDelta::FromHours(1), &result));
   EXPECT_EQ(0, result);
 
   // If this timezone uses DST, test the behavior on days when the time
@@ -543,7 +537,7 @@ TEST_F(VisitDatabaseTest, GetHistoryCount) {
   // TimeDelta::FromDays(1) to move one day, as this simply removes 24 hours and
   // thus does not work correctly with DST shifts. Instead, we'll go back
   // 1 second (i.e. somewhere in the middle of the previous day), and use
-  // |LocalMidnight()| to round down to the beginning of the day in the local
+  // `LocalMidnight()` to round down to the beginning of the day in the local
   // time, taking timezones and DST into account. This is necessary to achieve
   // the same equivalence class on days as the DATE(..., 'localtime') function
   // in SQL.
@@ -570,18 +564,18 @@ TEST_F(VisitDatabaseTest, GetHistoryCount) {
   // 24 hours later. The count should be 1, not 2, because the day is longer
   // than 24 hours, and the two visits will be regarded as duplicate.
   if (!shift_backward.is_null()) {
-    VisitRow backward_1(1, shift_backward, 0, standard_transition, 0, true);
+    VisitRow backward_1(1, shift_backward, 0, standard_transition, 0, true,
+                        false);
     backward_1.visit_id = 10;
     AddVisit(&backward_1, SOURCE_BROWSED);
 
     VisitRow backward_2(1, shift_backward + TimeDelta::FromHours(24), 0,
-                        standard_transition, 0, true);
+                        standard_transition, 0, true, false);
     backward_2.visit_id = 11;
     AddVisit(&backward_2, SOURCE_BROWSED);
 
-    EXPECT_TRUE(GetHistoryCount(shift_backward,
-                                shift_backward + TimeDelta::FromHours(25),
-                                &result));
+    EXPECT_TRUE(GetHistoryCount(
+        shift_backward, shift_backward + TimeDelta::FromHours(25), &result));
     EXPECT_EQ(1, result);
   }
 
@@ -590,25 +584,378 @@ TEST_F(VisitDatabaseTest, GetHistoryCount) {
   // regarded as duplicate in a normal 24 hour day, but in this case the second
   // visit is already in the next day.
   if (!shift_forward.is_null()) {
-    VisitRow forward_1(1, shift_forward, 0, standard_transition, 0, true);
+    VisitRow forward_1(1, shift_forward, 0, standard_transition, 0, true,
+                       false);
     forward_1.visit_id = 12;
     AddVisit(&forward_1, SOURCE_BROWSED);
 
-    Time almost_24_hours_later = shift_forward +
-                                 TimeDelta::FromHours(24) -
+    Time almost_24_hours_later = shift_forward + TimeDelta::FromHours(24) -
                                  TimeDelta::FromMicroseconds(1);
     VisitRow forward_2(1, almost_24_hours_later, 0, standard_transition, 0,
-                       true);
+                       true, false);
     forward_2.visit_id = 13;
     AddVisit(&forward_2, SOURCE_BROWSED);
 
-    EXPECT_TRUE(GetHistoryCount(shift_forward,
-                                shift_forward + TimeDelta::FromHours(24),
-                                &result));
+    EXPECT_TRUE(GetHistoryCount(
+        shift_forward, shift_forward + TimeDelta::FromHours(24), &result));
     EXPECT_EQ(2, result);
   }
 }
 
+TEST_F(VisitDatabaseTest, GetLastVisitToOrigin_BadURL) {
+  base::Time last_visit;
+  EXPECT_FALSE(GetLastVisitToOrigin(url::Origin(), base::Time::Min(),
+                                    base::Time::Max(), &last_visit));
+  EXPECT_EQ(last_visit, base::Time());
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToOrigin_NonHttpURL) {
+  base::Time last_visit;
+  EXPECT_FALSE(GetLastVisitToOrigin(url::Origin::Create(GURL("ftp://host/")),
+                                    base::Time::Min(), base::Time::Max(),
+                                    &last_visit));
+  EXPECT_EQ(last_visit, base::Time());
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToOrigin_NoVisits) {
+  base::Time last_visit;
+  EXPECT_TRUE(GetLastVisitToOrigin(
+      url::Origin::Create(GURL("https://www.chromium.org")), base::Time::Min(),
+      base::Time::Max(), &last_visit));
+  EXPECT_EQ(last_visit, base::Time());
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToOrigin_VisitsOutsideRange) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromHours(1);
+
+  VisitRow row1{AddURL(URLRow(GURL("https://www.chromium.org"))),
+                begin_time - base::TimeDelta::FromHours(1),
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row1, SOURCE_BROWSED);
+  VisitRow row2{AddURL(URLRow(GURL("https://www.chromium.org"))),
+                end_time + base::TimeDelta::FromHours(1),
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row2, SOURCE_BROWSED);
+
+  base::Time last_visit;
+  EXPECT_TRUE(GetLastVisitToOrigin(
+      url::Origin::Create(GURL("https://www.chromium.org")), begin_time,
+      end_time, &last_visit));
+  EXPECT_EQ(last_visit, base::Time());
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToOrigin_EndTimeNotIncluded) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromHours(1);
+
+  VisitRow row1{AddURL(URLRow(GURL("https://www.chromium.org"))),
+                begin_time,
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row1, SOURCE_BROWSED);
+  VisitRow row2{AddURL(URLRow(GURL("https://www.chromium.org"))),
+                end_time,
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row2, SOURCE_BROWSED);
+
+  base::Time last_visit;
+  EXPECT_TRUE(GetLastVisitToOrigin(
+      url::Origin::Create(GURL("https://www.chromium.org")), begin_time,
+      end_time, &last_visit));
+  EXPECT_EQ(last_visit, begin_time);
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToOrigin_SameOriginOnly) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromHours(1);
+
+  VisitRow row1{AddURL(URLRow(GURL("https://other.origin.chromium.org"))),
+                begin_time,
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row1, SOURCE_BROWSED);
+  VisitRow row2{AddURL(URLRow(GURL("https://www.chromium.org/path?query=foo"))),
+                begin_time + base::TimeDelta::FromMinutes(1),
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row2, SOURCE_BROWSED);
+
+  base::Time last_visit;
+  EXPECT_TRUE(GetLastVisitToOrigin(
+      url::Origin::Create(GURL("https://www.chromium.org")), begin_time,
+      end_time, &last_visit));
+  EXPECT_EQ(last_visit, begin_time + base::TimeDelta::FromMinutes(1));
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToHost_DifferentScheme) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromHours(1);
+
+  VisitRow row1{AddURL(URLRow(GURL("https://www.chromium.org"))),
+                begin_time,
+                0,
+                ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
+                                          ui::PAGE_TRANSITION_CHAIN_START |
+                                          ui::PAGE_TRANSITION_CHAIN_END),
+                0,
+                false,
+                false};
+  AddVisit(&row1, SOURCE_BROWSED);
+  VisitRow row2{AddURL(URLRow(GURL("http://www.chromium.org"))),
+                begin_time + base::TimeDelta::FromMinutes(1),
+                0,
+                ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
+                                          ui::PAGE_TRANSITION_CHAIN_START |
+                                          ui::PAGE_TRANSITION_CHAIN_END),
+                0,
+                false,
+                false};
+  AddVisit(&row2, SOURCE_BROWSED);
+
+  base::Time last_visit;
+  VisitRow row;
+  EXPECT_TRUE(GetLastVisitToHost(GURL("https://www.chromium.org").host(),
+                                 begin_time, end_time, &last_visit));
+  EXPECT_EQ(last_visit, begin_time + base::TimeDelta::FromMinutes(1));
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToHost_IncludePort) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromHours(1);
+
+  VisitRow row1{AddURL(URLRow(GURL("https://www.chromium.org"))),
+                begin_time,
+                0,
+                ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
+                                          ui::PAGE_TRANSITION_CHAIN_START |
+                                          ui::PAGE_TRANSITION_CHAIN_END),
+                0,
+                false,
+                false};
+  AddVisit(&row1, SOURCE_BROWSED);
+  VisitRow row2{AddURL(URLRow(GURL("https://www.chromium.org:8080"))),
+                begin_time + base::TimeDelta::FromMinutes(1),
+                0,
+                ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
+                                          ui::PAGE_TRANSITION_CHAIN_START |
+                                          ui::PAGE_TRANSITION_CHAIN_END),
+                0,
+                false,
+                false};
+  AddVisit(&row2, SOURCE_BROWSED);
+
+  base::Time last_visit;
+  EXPECT_TRUE(GetLastVisitToHost(GURL("https://www.chromium.org").host(),
+                                 begin_time, end_time, &last_visit));
+  EXPECT_EQ(last_visit, begin_time + base::TimeDelta::FromMinutes(1));
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToHost_DifferentPorts) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromHours(1);
+
+  VisitRow row1{AddURL(URLRow(GURL("https://www.chromium.org:8080"))),
+                begin_time,
+                0,
+                ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
+                                          ui::PAGE_TRANSITION_CHAIN_START |
+                                          ui::PAGE_TRANSITION_CHAIN_END),
+                0,
+                false,
+                false};
+  AddVisit(&row1, SOURCE_BROWSED);
+  VisitRow row2{AddURL(URLRow(GURL("https://www.chromium.org:32256"))),
+                begin_time + base::TimeDelta::FromMinutes(1),
+                0,
+                ui::PageTransitionFromInt(ui::PAGE_TRANSITION_TYPED |
+                                          ui::PAGE_TRANSITION_CHAIN_START |
+                                          ui::PAGE_TRANSITION_CHAIN_END),
+                0,
+                false,
+                false};
+  AddVisit(&row2, SOURCE_BROWSED);
+
+  base::Time last_visit;
+  EXPECT_TRUE(GetLastVisitToHost(GURL("https://www.chromium.org:8080").host(),
+                                 begin_time, end_time, &last_visit));
+  EXPECT_EQ(last_visit, begin_time + base::TimeDelta::FromMinutes(1));
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToOrigin_MostRecentVisitTime) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromHours(1);
+
+  VisitRow row1{AddURL(URLRow(GURL("https://chromium.org/"))),
+                begin_time,
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row1, SOURCE_BROWSED);
+  VisitRow row2{AddURL(URLRow(GURL("https://www.chromium.org/"))),
+                begin_time + base::TimeDelta::FromMinutes(1),
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row2, SOURCE_BROWSED);
+  VisitRow row3{AddURL(URLRow(GURL("https://www.chromium.org/"))),
+                begin_time + base::TimeDelta::FromMinutes(2),
+                0,
+                ui::PageTransitionFromInt(0),
+                0,
+                false,
+                false};
+  AddVisit(&row3, SOURCE_BROWSED);
+
+  base::Time last_visit;
+  EXPECT_TRUE(GetLastVisitToOrigin(
+      url::Origin::Create(GURL("https://www.chromium.org")), begin_time,
+      end_time, &last_visit));
+  EXPECT_EQ(last_visit, begin_time + base::TimeDelta::FromMinutes(2));
+}
+
+TEST_F(VisitDatabaseTest, GetLastVisitToURL) {
+  {
+    base::Time last_visit;
+    EXPECT_TRUE(GetLastVisitToURL(GURL("https://foo.com/bar/baz"),
+                                  base::Time::FromTimeT(1000), &last_visit));
+    EXPECT_EQ(last_visit, base::Time());
+  }
+
+  VisitRow most_recent{AddURL(URLRow(GURL("https://foo.com/bar/baz"))),
+                       base::Time::FromTimeT(200),
+                       0,
+                       ui::PageTransitionFromInt(0),
+                       0,
+                       false,
+                       false};
+  AddVisit(&most_recent, SOURCE_BROWSED);
+  VisitRow older_visit{AddURL(URLRow(GURL("https://foo.com/bar/baz"))),
+                       base::Time::FromTimeT(100),
+                       0,
+                       ui::PageTransitionFromInt(0),
+                       0,
+                       false,
+                       false};
+  AddVisit(&older_visit, SOURCE_BROWSED);
+  VisitRow wrong_url{AddURL(URLRow(GURL("https://foo.com/wrong_url"))),
+                     base::Time::FromTimeT(300),
+                     0,
+                     ui::PageTransitionFromInt(0),
+                     0,
+                     false,
+                     false};
+  AddVisit(&wrong_url, SOURCE_BROWSED);
+
+  {
+    base::Time last_visit;
+    EXPECT_TRUE(GetLastVisitToURL(GURL("https://foo.com/bar/baz"),
+                                  base::Time::FromTimeT(1000), &last_visit));
+    EXPECT_EQ(last_visit, base::Time::FromTimeT(200));
+  }
+  // Test getting the older visit using an `end_time` of 150.
+  {
+    base::Time last_visit;
+    EXPECT_TRUE(GetLastVisitToURL(GURL("https://foo.com/bar/baz"),
+                                  base::Time::FromTimeT(150), &last_visit));
+    EXPECT_EQ(last_visit, base::Time::FromTimeT(100));
+  }
+}
+
+TEST_F(VisitDatabaseTest, GetDailyVisitsToHostWithVisits) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromDays(10);
+
+  base::Time day1_time =
+      begin_time.LocalMidnight() + base::TimeDelta::FromHours(24);
+  base::Time day2_time = day1_time + base::TimeDelta::FromHours(24);
+
+  auto add_visit = [&](const GURL& url, base::Time visit_time) {
+    VisitRow row{AddURL(URLRow(url)),
+                 visit_time,
+                 0,
+                 ui::PageTransitionFromInt(ui::PAGE_TRANSITION_LINK |
+                                           ui::PAGE_TRANSITION_CHAIN_START |
+                                           ui::PAGE_TRANSITION_CHAIN_END),
+                 0,
+                 false,
+                 false};
+    AddVisit(&row, SOURCE_BROWSED);
+  };
+  // One visit before time range.
+  add_visit(GURL("https://foo.com/"),
+            begin_time - base::TimeDelta::FromHours(1));
+  // Two visits on first day.
+  add_visit(GURL("https://foo.com/bar"), day1_time);
+  add_visit(GURL("https://foo.com/baz"), day1_time +
+                                             base::TimeDelta::FromHours(24) -
+                                             base::TimeDelta::FromSeconds(1));
+  // Five visits on the next day.
+  for (int i = 0; i < 5; ++i) {
+    add_visit(GURL("https://foo.com/bar"), day2_time);
+  }
+  // These aren't visits, different scheme/host/port.
+  add_visit(GURL("http://foo.com/bar"), day2_time);
+  add_visit(GURL("https://fun.foo.com"), day2_time);
+  add_visit(GURL("https://foo.com:123/bar"), day2_time);
+
+  // One visit after end_time.
+  add_visit(GURL("https://foo.com/bar"),
+            end_time + base::TimeDelta::FromSeconds(1));
+
+  DailyVisitsResult result =
+      GetDailyVisitsToHost(GURL("https://foo.com"), begin_time, end_time);
+  EXPECT_TRUE(result.success);
+  EXPECT_EQ(2, result.days_with_visits);
+  EXPECT_EQ(7, result.total_visits);
+}
+
+TEST_F(VisitDatabaseTest, GetDailyVisitsToHostNoVisits) {
+  base::Time begin_time = base::Time::Now();
+  base::Time end_time = begin_time + base::TimeDelta::FromDays(10);
+
+  // A non-user visible visit.
+  VisitRow row{AddURL(URLRow(GURL("https://www.chromium.org"))),
+               begin_time,
+               0,
+               ui::PageTransitionFromInt(0),
+               0,
+               false,
+               false};
+  AddVisit(&row, SOURCE_BROWSED);
+
+  DailyVisitsResult result = GetDailyVisitsToHost(
+      GURL("https://www.chromium.org"), begin_time, end_time);
+  EXPECT_TRUE(result.success);
+  EXPECT_EQ(0, result.days_with_visits);
+  EXPECT_EQ(0, result.total_visits);
+}
+
 TEST_F(VisitDatabaseTest, GetGoogleDomainVisitsFromSearchesInRange_NoVisits) {
   const auto begin_time = base::Time::Now();
   EXPECT_THAT(GetGoogleDomainVisitsFromSearchesInRange(
@@ -625,6 +972,7 @@ TEST_F(VisitDatabaseTest,
                0,
                ui::PageTransitionFromInt(0),
                0,
+               false,
                false};
   AddVisit(&row, SOURCE_BROWSED);
   // In range, exactly begin time.
@@ -633,6 +981,7 @@ TEST_F(VisitDatabaseTest,
          0,
          ui::PageTransitionFromInt(0),
          0,
+         false,
          false};
   AddVisit(&row, SOURCE_BROWSED);
   // In range, 23 hours after begin time.
@@ -641,6 +990,7 @@ TEST_F(VisitDatabaseTest,
          0,
          ui::PageTransitionFromInt(0),
          0,
+         false,
          false};
   AddVisit(&row, SOURCE_BROWSED);
   // Out of range, exactly a day after begin time.
@@ -649,6 +999,7 @@ TEST_F(VisitDatabaseTest,
          0,
          ui::PageTransitionFromInt(0),
          0,
+         false,
          false};
   AddVisit(&row, SOURCE_BROWSED);
 
@@ -670,6 +1021,7 @@ TEST_F(VisitDatabaseTest, GetGoogleDomainVisitsFromSearchesInRange_NotSearch) {
                0,
                ui::PageTransitionFromInt(0),
                0,
+               false,
                false};
   AddVisit(&row, SOURCE_BROWSED);
 
@@ -686,6 +1038,7 @@ TEST_F(VisitDatabaseTest,
                0,
                ui::PageTransitionFromInt(0),
                0,
+               false,
                false};
   AddVisit(&row, SOURCE_BROWSED);
 
