@@ -9,6 +9,7 @@
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/containers/flat_map.h"
+#include "base/cxx17_backports.h"
 #include "base/feature_list.h"
 #include "base/files/file.h"
 #include "base/files/file_enumerator.h"
@@ -16,21 +17,25 @@
 #include "base/files/memory_mapped_file.h"
 #include "base/logging.h"
 #include "base/metrics/histogram_base.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/metrics/persistent_histogram_allocator.h"
 #include "base/metrics/persistent_memory_allocator.h"
-#include "base/stl_util.h"
 #include "base/strings/string_piece.h"
 #include "base/task/post_task.h"
 #include "base/task/task_traits.h"
+#include "base/task/thread_pool.h"
 #include "base/task_runner.h"
 #include "base/task_runner_util.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "components/metrics/metrics_pref_names.h"
 #include "components/metrics/metrics_service.h"
+#include "components/metrics/persistent_histograms.h"
 #include "components/metrics/persistent_system_profile.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/pref_service.h"
+#include "components/prefs/scoped_user_pref_update.h"
 
 namespace metrics {
 
@@ -97,7 +102,7 @@ scoped_refptr<base::TaskRunner> CreateBackgroundTaskRunner() {
   if (g_task_runner_for_testing)
     return scoped_refptr<base::TaskRunner>(g_task_runner_for_testing);
 
-  return base::CreateTaskRunnerWithTraits(
+  return base::ThreadPool::CreateTaskRunner(
       {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
        base::TaskShutdownBehavior::SKIP_ON_SHUTDOWN});
 }
@@ -190,7 +195,10 @@ FileMetricsProvider::Params::Params(const base::FilePath& path,
 FileMetricsProvider::Params::~Params() {}
 
 FileMetricsProvider::FileMetricsProvider(PrefService* local_state)
-    : task_runner_(CreateBackgroundTaskRunner()), pref_service_(local_state) {
+    : task_runner_(CreateBackgroundTaskRunner()),
+      pref_service_(local_state),
+      main_task_runner_(base::ThreadTaskRunnerHandle::Get()) {
+  DCHECK(main_task_runner_);
   base::StatisticsRecorder::RegisterHistogramProvider(
       weak_factory_.GetWeakPtr());
 }
@@ -208,14 +216,14 @@ void FileMetricsProvider::RegisterSource(const Params& params) {
   // |prefs_key| may be empty if the caller does not wish to persist the
   // state across instances of the program.
   if (pref_service_ && !params.prefs_key.empty()) {
-    source->last_seen = base::Time::FromInternalValue(
-        pref_service_->GetInt64(metrics::prefs::kMetricsLastSeenPrefix +
-                                source->prefs_key));
+    source->last_seen = pref_service_->GetTime(
+        metrics::prefs::kMetricsLastSeenPrefix + source->prefs_key);
   }
 
   switch (params.association) {
     case ASSOCIATE_CURRENT_RUN:
     case ASSOCIATE_INTERNAL_PROFILE:
+    case ASSOCIATE_INTERNAL_PROFILE_SAMPLES_COUNTER:
       sources_to_check_.push_back(std::move(source));
       break;
     case ASSOCIATE_PREVIOUS_RUN:
@@ -227,10 +235,16 @@ void FileMetricsProvider::RegisterSource(const Params& params) {
 }
 
 // static
-void FileMetricsProvider::RegisterPrefs(PrefRegistrySimple* prefs,
-                                        const base::StringPiece prefs_key) {
-  prefs->RegisterInt64Pref(metrics::prefs::kMetricsLastSeenPrefix +
-                           prefs_key.as_string(), 0);
+void FileMetricsProvider::RegisterSourcePrefs(
+    PrefRegistrySimple* prefs,
+    const base::StringPiece prefs_key) {
+  prefs->RegisterInt64Pref(
+      metrics::prefs::kMetricsLastSeenPrefix + std::string(prefs_key), 0);
+}
+
+//  static
+void FileMetricsProvider::RegisterPrefs(PrefRegistrySimple* prefs) {
+  prefs->RegisterListPref(metrics::prefs::kMetricsFileMetricsMetadata);
 }
 
 // static
@@ -303,7 +317,7 @@ bool FileMetricsProvider::LocateNextFileInDirectory(SourceInfo* source) {
         // have been created by a privileged process like setup.exe. Even if it
         // is not removed, it will continue to be ignored bacuse of the older
         // modification time.
-        base::DeleteFile(found_file.path, /*recursive=*/false);
+        base::DeleteFile(found_file.path);
       }
     }
   }
@@ -323,7 +337,7 @@ bool FileMetricsProvider::LocateNextFileInDirectory(SourceInfo* source) {
         source->max_age != base::TimeDelta() &&
         now_time - found.info.GetLastModifiedTime() > source->max_age;
     if (too_many || too_big || too_old) {
-      base::DeleteFile(found.path, /*recursive=*/false);
+      base::DeleteFile(found.path);
       --file_count;
       total_size_kib -= found.info.GetSize() >> 10;
       RecordAccessResult(too_many ? ACCESS_RESULT_TOO_MANY_FILES
@@ -373,7 +387,6 @@ void FileMetricsProvider::FinishedWithSource(SourceInfo* source,
   }
 }
 
-// static
 void FileMetricsProvider::CheckAndMergeMetricSourcesOnTaskRunner(
     SourceInfoList* sources) {
   // This method has all state information passed in |sources| and is intended
@@ -401,7 +414,11 @@ void FileMetricsProvider::CheckAndMergeMetricSourcesOnTaskRunner(
         if (source->association == ASSOCIATE_INTERNAL_PROFILE)
           break;
 
-        MergeHistogramDeltasFromSource(source.get());
+        if (source->association == ASSOCIATE_INTERNAL_PROFILE_SAMPLES_COUNTER) {
+          RecordFileMetadataOnTaskRunner(source.get());
+        } else {
+          MergeHistogramDeltasFromSource(source.get());
+        }
         DCHECK(source->read_complete);
       }
 
@@ -462,6 +479,41 @@ FileMetricsProvider::AccessResult FileMetricsProvider::CheckAndMapMetricSource(
   if (!file.IsValid())
     return ACCESS_RESULT_NO_OPEN;
 
+  // Check that file is writable if that is expected. If a write is attempted
+  // on an unwritable memory-mapped file, a SIGBUS will cause a crash.
+  const bool read_only = kSourceOptions[source->type].is_read_only;
+  if (!read_only) {
+    constexpr int kTestSize = 16;
+    char header[kTestSize];
+    int amount = file.Read(0, header, kTestSize);
+    if (amount != kTestSize)
+      return ACCESS_RESULT_INVALID_CONTENTS;
+
+    char zeros[kTestSize] = {0};
+    file.Write(0, zeros, kTestSize);
+    file.Flush();
+
+    // A crash here would be unfortunate as the file would be left invalid
+    // and skipped/deleted by later attempts. This is unlikely, however, and
+    // the benefit of avoiding crashes from mapping as read/write a file that
+    // can't be written more than justifies the risk.
+
+    char check[kTestSize];
+    amount = file.Read(0, check, kTestSize);
+    if (amount != kTestSize)
+      return ACCESS_RESULT_INVALID_CONTENTS;
+    if (memcmp(check, zeros, kTestSize) != 0)
+      return ACCESS_RESULT_NOT_WRITABLE;
+
+    file.Write(0, header, kTestSize);
+    file.Flush();
+    amount = file.Read(0, check, kTestSize);
+    if (amount != kTestSize)
+      return ACCESS_RESULT_INVALID_CONTENTS;
+    if (memcmp(check, header, kTestSize) != 0)
+      return ACCESS_RESULT_NOT_WRITABLE;
+  }
+
   std::unique_ptr<base::MemoryMappedFile> mapped(new base::MemoryMappedFile());
   if (!mapped->Initialize(std::move(file),
                           kSourceOptions[source->type].memory_mapped_access)) {
@@ -472,7 +524,6 @@ FileMetricsProvider::AccessResult FileMetricsProvider::CheckAndMapMetricSource(
   source->last_seen = info.last_modified;
 
   // Test the validity of the file contents.
-  const bool read_only = kSourceOptions[source->type].is_read_only;
   if (!base::FilePersistentMemoryAllocator::IsFileAcceptable(*mapped,
                                                              read_only)) {
     return ACCESS_RESULT_INVALID_CONTENTS;
@@ -524,6 +575,14 @@ void FileMetricsProvider::MergeHistogramDeltasFromSource(SourceInfo* source) {
     std::unique_ptr<base::HistogramBase> histogram = histogram_iter.GetNext();
     if (!histogram)
       break;
+
+    // Keep track of which histograms are getting merged from other sources.
+    // TODO(crbug.com/1176977): Consider removing this after bug is fixed.
+    base::UmaHistogramSparse(
+        read_only ? "UMA.FileMetricsProvider.MergeHistogram.ReadOnly"
+                  : "UMA.FileMetricsProvider.MergeHistogram.NotReadOnly",
+        static_cast<base::HistogramBase::Sample>(histogram->name_hash()));
+
     if (read_only) {
       source->allocator->MergeHistogramFinalDeltaToStatisticsRecorder(
           histogram.get());
@@ -627,6 +686,25 @@ bool FileMetricsProvider::ProvideIndependentMetricsOnTaskRunner(
   return false;
 }
 
+void FileMetricsProvider::AppendToSamplesCountPref(size_t samples_count) {
+  ListPrefUpdate update(pref_service_,
+                        metrics::prefs::kMetricsFileMetricsMetadata);
+  update->Append(static_cast<int>(samples_count));
+}
+
+void FileMetricsProvider::RecordFileMetadataOnTaskRunner(SourceInfo* source) {
+  base::HistogramBase::Count samples_count = 0;
+  base::PersistentHistogramAllocator::Iterator it{source->allocator.get()};
+  std::unique_ptr<base::HistogramBase> histogram;
+  while ((histogram = it.GetNext()) != nullptr) {
+    samples_count += histogram->SnapshotFinalDelta()->TotalCount();
+  }
+  main_task_runner_->PostTask(
+      FROM_HERE, base::BindOnce(&FileMetricsProvider::AppendToSamplesCountPref,
+                                base::Unretained(this), samples_count));
+  source->read_complete = true;
+}
+
 void FileMetricsProvider::ScheduleSourcesCheck() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
@@ -643,7 +721,7 @@ void FileMetricsProvider::ScheduleSourcesCheck() {
       FROM_HERE,
       base::BindOnce(
           &FileMetricsProvider::CheckAndMergeMetricSourcesOnTaskRunner,
-          base::Unretained(check_list)),
+          base::Unretained(this), base::Unretained(check_list)),
       base::BindOnce(&FileMetricsProvider::RecordSourcesChecked,
                      weak_factory_.GetWeakPtr(), base::Owned(check_list)));
 }
@@ -696,9 +774,9 @@ void FileMetricsProvider::RecordSourceAsRead(SourceInfo* source) {
   // Persistently record the "last seen" timestamp of the source file to
   // ensure that the file is never read again unless it is modified again.
   if (pref_service_ && !source->prefs_key.empty()) {
-    pref_service_->SetInt64(
+    pref_service_->SetTime(
         metrics::prefs::kMetricsLastSeenPrefix + source->prefs_key,
-        source->last_seen.ToInternalValue());
+        source->last_seen);
   }
 }
 
@@ -723,7 +801,7 @@ void FileMetricsProvider::OnDidCreateMetricsLog() {
 
 bool FileMetricsProvider::HasIndependentMetrics() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  return !sources_with_profile_.empty();
+  return !sources_with_profile_.empty() || SimulateIndependentMetrics();
 }
 
 void FileMetricsProvider::ProvideIndependentMetrics(
@@ -844,4 +922,29 @@ void FileMetricsProvider::MergeHistogramDeltas() {
   }
 }
 
+bool FileMetricsProvider::SimulateIndependentMetrics() {
+  if (!pref_service_->HasPrefPath(
+          metrics::prefs::kMetricsFileMetricsMetadata)) {
+    return false;
+  }
+
+  ListPrefUpdate list_value(pref_service_,
+                            metrics::prefs::kMetricsFileMetricsMetadata);
+  if (list_value->GetList().empty())
+    return false;
+
+  base::Value::ListView mutable_list = list_value->GetList();
+  size_t count = pref_service_->GetInteger(
+      metrics::prefs::kStabilityFileMetricsUnsentSamplesCount);
+  pref_service_->SetInteger(
+      metrics::prefs::kStabilityFileMetricsUnsentSamplesCount,
+      mutable_list[0].GetInt() + count);
+  pref_service_->SetInteger(
+      metrics::prefs::kStabilityFileMetricsUnsentFilesCount,
+      list_value->GetSize() - 1);
+  list_value->EraseListIter(mutable_list.begin());
+
+  return true;
+}
+
 }  // namespace metrics
