@@ -10,18 +10,25 @@ namespace prefs {
 
 // Alphabetical list of preference names specific to the metrics
 // component. Document each in the .cc file.
-extern const char kDeprecatedMetricsInitialLogs[];
-extern const char kDeprecatedMetricsOngoingLogs[];
 extern const char kInstallDate[];
 extern const char kMetricsClientID[];
+extern const char kMetricsFileMetricsMetadata[];
 extern const char kMetricsDefaultOptIn[];
 extern const char kMetricsInitialLogs[];
+extern const char kMetricsInitialLogsMetadata[];
 extern const char kMetricsLowEntropySource[];
 extern const char kMetricsOldLowEntropySource[];
+extern const char kMetricsPseudoLowEntropySource[];
 extern const char kMetricsMachineId[];
 extern const char kMetricsOngoingLogs[];
+extern const char kMetricsOngoingLogsMetadata[];
 extern const char kMetricsResetIds[];
 
+// Preferences for cloned installs.
+extern const char kClonedResetCount[];
+extern const char kFirstClonedResetTimestamp[];
+extern const char kLastClonedResetTimestamp[];
+
 // For finding out whether metrics and crash reporting is enabled use the
 // relevant embedder-specific subclass of MetricsServiceAccessor instead of
 // reading this pref directly; see the comments on metrics_service_accessor.h.
@@ -33,22 +40,16 @@ extern const char kMetricsSessionID[];
 extern const char kMetricsLastSeenPrefix[];
 
 // Preferences for recording stability logs.
-extern const char kStabilityBreakpadRegistrationFail[];
-extern const char kStabilityBreakpadRegistrationSuccess[];
 extern const char kStabilityBrowserLastLiveTimeStamp[];
 extern const char kStabilityChildProcessCrashCount[];
 extern const char kStabilityCrashCount[];
 extern const char kStabilityCrashCountDueToGmsCoreUpdate[];
-extern const char kStabilityCrashCountWithoutGmsCoreUpdateObsolete[];
-extern const char kStabilityDebuggerNotPresent[];
-extern const char kStabilityDebuggerPresent[];
-extern const char kStabilityDeferredCount[];
-extern const char kStabilityDiscardCount[];
-extern const char kStabilityExecutionPhase[];
 extern const char kStabilityExitedCleanly[];
 extern const char kStabilityExtensionRendererCrashCount[];
 extern const char kStabilityExtensionRendererFailedLaunchCount[];
 extern const char kStabilityExtensionRendererLaunchCount[];
+extern const char kStabilityFileMetricsUnsentSamplesCount[];
+extern const char kStabilityFileMetricsUnsentFilesCount[];
 extern const char kStabilityGmsCoreVersion[];
 extern const char kStabilityGpuCrashCount[];
 extern const char kStabilityIncompleteSessionEndCount[];
@@ -64,12 +65,6 @@ extern const char kStabilitySessionEndCompleted[];
 extern const char kStabilityStatsBuildTime[];
 extern const char kStabilityStatsVersion[];
 extern const char kStabilitySystemCrashCount[];
-extern const char kStabilityVersionMismatchCount[];
-
-// Preferences for generating metrics at uninstall time.
-extern const char kUninstallLaunchCount[];
-extern const char kUninstallMetricsPageLoadCount[];
-extern const char kUninstallMetricsUptimeSec[];
 
 // For measuring data use for throttling UMA log uploads on cellular.
 extern const char kUkmCellDataUse[];
