@@ -7,6 +7,7 @@
 
 #include "base/callback.h"
 #include "base/macros.h"
+#include "base/time/time.h"
 
 namespace base {
 class HistogramSnapshotManager;
@@ -24,15 +25,18 @@ class MetricsProvider {
   MetricsProvider();
   virtual ~MetricsProvider();
 
-  // Called after initialiazation of MetricsService and field trials.
+  // Called after initialization of MetricsService and field trials.
   virtual void Init();
 
   // Called during service initialization to allow the provider to start any
   // async initialization tasks.  The service will wait for the provider to
   // call |done_callback| before generating logs for the current session.
-  virtual void AsyncInit(const base::Closure& done_callback);
+  virtual void AsyncInit(base::OnceClosure done_callback);
 
   // Called when a new MetricsLog is created.
+  // This can be used to log a histogram that will appear in the log. Not safe
+  // for some other uses, like user actions.
+  // TODO(crbug.com/1171830): Improve this.
   virtual void OnDidCreateMetricsLog();
 
   // Called when metrics recording has been enabled.
@@ -41,6 +45,14 @@ class MetricsProvider {
   // Called when metrics recording has been disabled.
   virtual void OnRecordingDisabled();
 
+  // Called when metrics client identifiers have been reset.
+  //
+  // Metrics providers should clean up any persisted state that could be used to
+  // associate the previous identifier with the new one.
+  //
+  // Currently this method is only invoked in UKM.
+  virtual void OnClientStateCleared();
+
   // Called when the application is going into background mode, on platforms
   // where applications may be killed when going into the background (Android,
   // iOS). Providers that buffer histogram data in memory should persist
@@ -55,7 +67,7 @@ class MetricsProvider {
   // Provides a complete and independent uma proto + metrics for uploading.
   // Called once every time HasIndependentMetrics() returns true. The passed in
   // |uma_proto| is by default filled with current session id and core system
-  // profile infomration. This function is called on main thread, but the
+  // profile information. This function is called on main thread, but the
   // provider can do async work to fill in |uma_proto| and run |done_callback|
   // on calling thread when complete. Ownership of the passed objects remains
   // with the caller and those objects will live until the callback is executed.
@@ -64,10 +76,19 @@ class MetricsProvider {
       ChromeUserMetricsExtension* uma_proto,
       base::HistogramSnapshotManager* snapshot_manager);
 
-  // Provides additional metrics into the system profile.
+  // Provides additional metrics into the system profile. This is a convenience
+  // method over ProvideSystemProfileMetricsWithLogCreationTime() without the
+  // |log_creation_time| param. Should not be called directly by services.
   virtual void ProvideSystemProfileMetrics(
       SystemProfileProto* system_profile_proto);
 
+  // Provides additional metrics into the system profile. The log creation
+  // time param provides a timestamp of when the log was opened, which is needed
+  // for some metrics providers.
+  virtual void ProvideSystemProfileMetricsWithLogCreationTime(
+      base::TimeTicks log_creation_time,
+      SystemProfileProto* system_profile_proto);
+
   // Called once at startup to see whether this provider has critical data
   // to provide about the previous session.
   // Returning true will trigger ProvidePreviousSessionData on all other
