@@ -10,13 +10,12 @@
 #include <utility>
 
 #include "base/bind.h"
+#include "base/cxx17_backports.h"
 #include "base/feature_list.h"
 #include "base/location.h"
 #include "base/metrics/field_trial_params.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/stl_util.h"
 #include "base/strings/string_split.h"
-#include "base/strings/stringprintf.h"
 #include "base/time/clock.h"
 #include "components/ntp_snippets/features.h"
 #include "components/ntp_snippets/pref_names.h"
@@ -347,10 +346,10 @@ void ReportTimeUntilPersistentFetch(
 
 class EulaState final : public web_resource::EulaAcceptedNotifier::Observer {
  public:
-  EulaState(PrefService* local_state_prefs, base::Closure eula_accepted)
+  EulaState(PrefService* local_state_prefs, base::OnceClosure eula_accepted)
       : eula_notifier_(
             web_resource::EulaAcceptedNotifier::Create(local_state_prefs)),
-        eula_accepted_(eula_accepted) {
+        eula_accepted_(std::move(eula_accepted)) {
     // EulaNotifier is not constructed on some platforms (such as desktop).
     if (!eula_notifier_) {
       return;
@@ -376,12 +375,13 @@ class EulaState final : public web_resource::EulaAcceptedNotifier::Observer {
     // executed once the setting flips to accepted. Hence, we can assume that
     // at the time this code runs, a background-fetch trigger is queued in the
     // scheduler.
-    eula_accepted_.Run();
+    if (eula_accepted_)
+      std::move(eula_accepted_).Run();
   }
 
  private:
   std::unique_ptr<web_resource::EulaAcceptedNotifier> eula_notifier_;
-  base::Callback<void()> eula_accepted_;
+  base::OnceClosure eula_accepted_;
 
   DISALLOW_COPY_AND_ASSIGN(EulaState);
 };
@@ -444,8 +444,7 @@ RemoteSuggestionsSchedulerImpl::RemoteSuggestionsSchedulerImpl(
     const UserClassifier* user_classifier,
     PrefService* profile_prefs,
     PrefService* local_state_prefs,
-    base::Clock* clock,
-    Logger* debug_logger)
+    base::Clock* clock)
     : persistent_scheduler_(persistent_scheduler),
       provider_(nullptr),
       background_fetch_in_progress_(false),
@@ -466,15 +465,14 @@ RemoteSuggestionsSchedulerImpl::RemoteSuggestionsSchedulerImpl(
       time_until_first_startup_trigger_reported_(false),
       eula_state_(std::make_unique<EulaState>(
           local_state_prefs,
-          base::Bind(&RemoteSuggestionsSchedulerImpl::RunQueuedTriggersIfReady,
-                     base::Unretained(this)))),
+          base::BindOnce(
+              &RemoteSuggestionsSchedulerImpl::RunQueuedTriggersIfReady,
+              base::Unretained(this)))),
       profile_prefs_(profile_prefs),
       clock_(clock),
-      enabled_triggers_(GetEnabledTriggerTypes()),
-      debug_logger_(debug_logger) {
+      enabled_triggers_(GetEnabledTriggerTypes()) {
   DCHECK(user_classifier);
   DCHECK(profile_prefs);
-  DCHECK(debug_logger_);
 }
 
 RemoteSuggestionsSchedulerImpl::~RemoteSuggestionsSchedulerImpl() = default;
@@ -527,7 +525,6 @@ void RemoteSuggestionsSchedulerImpl::OnProviderDeactivated() {
 }
 
 void RemoteSuggestionsSchedulerImpl::OnSuggestionsCleared() {
-  debug_logger_->Log(FROM_HERE, /*message=*/std::string());
   // This should be called by |provider_| so it should exist.
   DCHECK(provider_);
   // Some user action causes suggestions to be cleared, we need to fetch as soon
@@ -567,26 +564,22 @@ void RemoteSuggestionsSchedulerImpl::OnInteractiveFetchFinished(
 }
 
 void RemoteSuggestionsSchedulerImpl::OnPersistentSchedulerWakeUp() {
-  debug_logger_->Log(FROM_HERE, /*message=*/std::string());
   RefetchIfAppropriate(TriggerType::PERSISTENT_SCHEDULER_WAKE_UP);
 }
 
 void RemoteSuggestionsSchedulerImpl::OnBrowserForegrounded() {
-  debug_logger_->Log(FROM_HERE, /*message=*/std::string());
   // TODO(jkrcal): Consider that this is called whenever we open or return to an
   // Activity. Therefore, keep work light for fast start up calls.
   RefetchIfAppropriate(TriggerType::BROWSER_FOREGROUNDED);
 }
 
 void RemoteSuggestionsSchedulerImpl::OnBrowserColdStart() {
-  debug_logger_->Log(FROM_HERE, /*message=*/std::string());
   // TODO(jkrcal): Consider that work here must be kept light for fast
   // cold start ups.
   RefetchIfAppropriate(TriggerType::BROWSER_COLD_START);
 }
 
 void RemoteSuggestionsSchedulerImpl::OnSuggestionsSurfaceOpened() {
-  debug_logger_->Log(FROM_HERE, /*message=*/std::string());
   // TODO(jkrcal): Consider that this is called whenever we open an NTP.
   // Therefore, keep work light for fast start up calls.
   RefetchIfAppropriate(TriggerType::SURFACE_OPENED);
@@ -695,15 +688,12 @@ bool RemoteSuggestionsSchedulerImpl::IsLastSuccessfulFetchStale() const {
 }
 
 void RemoteSuggestionsSchedulerImpl::RefetchIfAppropriate(TriggerType trigger) {
-  debug_logger_->Log(FROM_HERE, /*message=*/std::string());
 
   if (background_fetch_in_progress_) {
-    debug_logger_->Log(FROM_HERE, "stop due to ongoing fetch");
     return;
   }
 
   if (enabled_triggers_.count(trigger) == 0) {
-    debug_logger_->Log(FROM_HERE, "stop due to disabled trigger");
     return;
   }
 
@@ -711,12 +701,10 @@ void RemoteSuggestionsSchedulerImpl::RefetchIfAppropriate(TriggerType trigger) {
     // Do not let a request fail due to lack of internet connection. Then, such
     // a failure would get logged and further requests would be blocked for a
     // while (even after becoming online).
-    debug_logger_->Log(FROM_HERE, "stop due to being offline");
     return;
   }
 
   if (!IsReadyForBackgroundFetches()) {
-    debug_logger_->Log(FROM_HERE, "delay until ready");
     queued_triggers_.insert(trigger);
     return;
   }
@@ -741,12 +729,10 @@ void RemoteSuggestionsSchedulerImpl::RefetchIfAppropriate(TriggerType trigger) {
 
   if (trigger != TriggerType::PERSISTENT_SCHEDULER_WAKE_UP &&
       !ShouldRefetchNow(last_fetch_attempt_time, trigger)) {
-    debug_logger_->Log(FROM_HERE, "stop, because too soon");
     return;
   }
 
   if (!AcquireQuota(/*interactive_request=*/false)) {
-    debug_logger_->Log(FROM_HERE, "stop due to quota");
     return;
   }
 
@@ -770,7 +756,6 @@ void RemoteSuggestionsSchedulerImpl::RefetchIfAppropriate(TriggerType trigger) {
       "NewTabPage.ContentSuggestions.BackgroundFetchTrigger",
       static_cast<int>(trigger), static_cast<int>(TriggerType::COUNT));
 
-  debug_logger_->Log(FROM_HERE, "issuing a fetch");
   background_fetch_in_progress_ = true;
 
   if ((trigger == TriggerType::BROWSER_COLD_START ||
@@ -807,24 +792,6 @@ bool RemoteSuggestionsSchedulerImpl::ShouldRefetchNow(
   }
 
   base::Time now = clock_->Now();
-  if (Logger::IsLoggingEnabled()) {
-    if (background_fetches_allowed_after_ > now) {
-      debug_logger_->Log(
-          FROM_HERE,
-          base::StringPrintf(
-              "due to privacy, next fetch is allowed after %s",
-              Logger::TimeToString(background_fetches_allowed_after_).c_str()));
-    }
-    if (first_allowed_fetch_time > now) {
-      debug_logger_->Log(
-          FROM_HERE,
-          base::StringPrintf(
-              "next fetch is scheduled after %s (as last fetch "
-              "attempt occured at %s)",
-              Logger::TimeToString(first_allowed_fetch_time).c_str(),
-              Logger::TimeToString(last_fetch_attempt_time).c_str()));
-    }
-  }
 
   return background_fetches_allowed_after_ <= now &&
          first_allowed_fetch_time <= now;
