@@ -121,7 +121,7 @@ PopularSites::SitesVector ParseSiteList(const base::ListValue& list) {
     const base::DictionaryValue* item;
     if (!list.GetDictionary(i, &item))
       continue;
-    base::string16 title;
+    std::u16string title;
     std::string url;
     if (!item->GetString("title", &title) || !item->GetString("url", &url))
       continue;
@@ -199,7 +199,7 @@ std::map<SectionType, PopularSites::SitesVector> ParseSites(
 void SetDefaultResourceForSite(size_t index,
                                int resource_id,
                                base::Value* sites) {
-  base::Value::ListStorage& list = sites->GetList();
+  base::Value::ListView list = sites->GetList();
   if (index >= list.size() || !list[index].is_dict())
     return;
 
@@ -215,7 +215,7 @@ base::Value DefaultPopularSites() {
   if (!base::FeatureList::IsEnabled(kPopularSitesBakedInContentFeature))
     return base::Value(base::Value::Type::LIST);
 
-  base::Optional<base::Value> sites = base::JSONReader::Read(
+  absl::optional<base::Value> sites = base::JSONReader::Read(
       ui::ResourceBundle::GetSharedInstance().LoadDataResourceString(
           IDR_DEFAULT_POPULAR_SITES_JSON));
   for (base::Value& site : sites.value().GetList())
@@ -237,7 +237,7 @@ base::Value DefaultPopularSites() {
 
 }  // namespace
 
-PopularSites::Site::Site(const base::string16& title,
+PopularSites::Site::Site(const std::u16string& title,
                          const GURL& url,
                          const GURL& favicon_url,
                          const GURL& large_icon_url,
@@ -271,9 +271,9 @@ PopularSitesImpl::PopularSitesImpl(
 PopularSitesImpl::~PopularSitesImpl() {}
 
 bool PopularSitesImpl::MaybeStartFetch(bool force_download,
-                                       const FinishedCallback& callback) {
+                                       FinishedCallback callback) {
   DCHECK(!callback_);
-  callback_ = callback;
+  callback_ = std::move(callback);
 
   const base::Time last_download_time = base::Time::FromInternalValue(
       prefs_->GetInt64(prefs::kPopularSitesLastDownloadPref));
@@ -484,7 +484,7 @@ void PopularSitesImpl::OnJsonParsed(
   prefs_->SetString(prefs::kPopularSitesURLPref, pending_url_.spec());
 
   sections_ = ParseSites(*list, version_in_pending_url_);
-  callback_.Run(true);
+  std::move(callback_).Run(true);
 }
 
 void PopularSitesImpl::OnDownloadFailed() {
@@ -497,7 +497,7 @@ void PopularSitesImpl::OnDownloadFailed() {
     FetchPopularSites();
   } else {
     DLOG(WARNING) << "Download fallback site list failed";
-    callback_.Run(false);
+    std::move(callback_).Run(false);
   }
 }
 
