@@ -2,22 +2,27 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#ifndef COMPONENTS_OFFLINE_ITEMS_COLLETION_CORE_OFFLINE_CONTENT_PROVIDER_H_
-#define COMPONENTS_OFFLINE_ITEMS_COLLETION_CORE_OFFLINE_CONTENT_PROVIDER_H_
+#ifndef COMPONENTS_OFFLINE_ITEMS_COLLECTION_CORE_OFFLINE_CONTENT_PROVIDER_H_
+#define COMPONENTS_OFFLINE_ITEMS_COLLECTION_CORE_OFFLINE_CONTENT_PROVIDER_H_
 
 #include <string>
 #include <vector>
 
 #include "base/callback.h"
 #include "base/macros.h"
-#include "base/optional.h"
+#include "base/observer_list.h"
 #include "components/offline_items_collection/core/launch_location.h"
+#include "components/offline_items_collection/core/open_params.h"
+#include "components/offline_items_collection/core/rename_result.h"
+#include "components/offline_items_collection/core/update_delta.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "url/gurl.h"
 
 namespace offline_items_collection {
 
 struct ContentId;
 struct OfflineItem;
+struct OfflineItemSchedule;
 struct OfflineItemShareInfo;
 struct OfflineItemVisuals;
 
@@ -33,14 +38,39 @@ class OfflineContentProvider {
                               std::unique_ptr<OfflineItemShareInfo>)>;
   using MultipleItemCallback = base::OnceCallback<void(const OfflineItemList&)>;
   using SingleItemCallback =
-      base::OnceCallback<void(const base::Optional<OfflineItem>&)>;
+      base::OnceCallback<void(const absl::optional<OfflineItem>&)>;
+  using RenameCallback = base::OnceCallback<void(RenameResult)>;
+  using DownloadRenameCallback = base::OnceCallback<RenameCallback>;
+
+  // Used by GetVisualsForItem to specify which visuals are needed.
+  struct GetVisualsOptions {
+    bool get_icon;
+    bool get_custom_favicon;
+    static GetVisualsOptions NoVisuals() { return GetVisualsOptions(); }
+    static GetVisualsOptions IconOnly() {
+      GetVisualsOptions options;
+      options.get_icon = true;
+      return options;
+    }
+    static GetVisualsOptions CustomFaviconOnly() {
+      GetVisualsOptions options;
+      options.get_custom_favicon = true;
+      return options;
+    }
+    static GetVisualsOptions IconAndCustomFavicon() {
+      GetVisualsOptions options;
+      options.get_icon = true;
+      options.get_custom_favicon = true;
+      return options;
+    }
+  };
 
   // An observer class that should be notified of relevant changes to the
   // underlying data source.
   // For the Observer that maintains its own cache of items, populated via
   // GetAllItems method, it is possible to receive notifications that are
   // out-of-sync with the cache content. See notes on the methods.
-  class Observer {
+  class Observer : public base::CheckedObserver {
    public:
     // Called when one or more OfflineItems have been added and should be shown
     // in the UI.  This should only be called for actual new items that are
@@ -62,16 +92,22 @@ class OfflineContentProvider {
     // TODO(dtrainor): Make this take a list of OfflineItems.
     // If Observer maintains a cache of items, the changes may already be
     // applied to the items in the cache, so there is no difference between
-    // items. In this case, this call should be ignored.
-    virtual void OnItemUpdated(const OfflineItem& item) = 0;
+    // items. This can be used in conjunction with the |update_delta| to
+    // determine whether this call should be ignored.
+    virtual void OnItemUpdated(
+        const OfflineItem& item,
+        const absl::optional<UpdateDelta>& update_delta) = 0;
+
+    // Called right before this object gets destroyed, to lets observers
+    // perform cleanup.
+    virtual void OnContentProviderGoingDown() = 0;
 
    protected:
-    virtual ~Observer() = default;
+    ~Observer() override = default;
   };
 
-  // Called to trigger opening an OfflineItem represented by |id|. |location|
-  // denotes where it is opened and is used for logging purpose.
-  virtual void OpenItem(LaunchLocation location, const ContentId& id) = 0;
+  // Called to trigger opening an OfflineItem represented by |id|.
+  virtual void OpenItem(const OpenParams& open_params, const ContentId& id) = 0;
 
   // Called to trigger removal of an OfflineItem represented by |id|.
   virtual void RemoveItem(const ContentId& id) = 0;
@@ -100,10 +136,12 @@ class OfflineContentProvider {
   // Asks for an OfflineItemVisuals struct for an OfflineItem represented by
   // |id| or |nullptr| if one doesn't exist.  The implementer should post any
   // replies even if the results are available immediately to prevent reentrancy
-  // and for consistent behavior.
+  // and for consistent behavior. |options| may be set to let the implementer
+  // know that it need not create the |icon| or |custom_favicon| members.
   // |callback| should be called no matter what (error, unavailable content,
   // etc.).
   virtual void GetVisualsForItem(const ContentId& id,
+                                 GetVisualsOptions options,
                                  VisualsCallback callback) = 0;
 
   // Asks for the right URI to use to share an OfflineItem represented by |id|
@@ -115,16 +153,40 @@ class OfflineContentProvider {
   virtual void GetShareInfoForItem(const ContentId& id,
                                    ShareCallback callback) = 0;
 
+  // Called to rename a downloaded OfflineItem represented by |id|, new name is
+  // given by |name|. Implementer should post and replies the results of rename
+  // attempt.
+  virtual void RenameItem(const ContentId& id,
+                          const std::string& name,
+                          RenameCallback callback) = 0;
+
+  // Called to change when to start the OfflineItem represented by |id|.
+  virtual void ChangeSchedule(const ContentId& id,
+                              absl::optional<OfflineItemSchedule> schedule) = 0;
+
   // Adds an observer that should be notified of OfflineItem list modifications.
-  virtual void AddObserver(Observer* observer) = 0;
+  void AddObserver(Observer* observer);
 
   // Removes an observer.  No further notifications should be sent to it.
-  virtual void RemoveObserver(Observer* observer) = 0;
+  void RemoveObserver(Observer* observer);
 
  protected:
-  virtual ~OfflineContentProvider() = default;
+  OfflineContentProvider();
+  virtual ~OfflineContentProvider();
+
+  // Used in tests.
+  bool HasObserver(Observer* observer);
+
+  // Notify observers via OnItemsAdded(), OnItemRemoved() or OnItemUpdated().
+  void NotifyItemsAdded(const OfflineItemList& items);
+  void NotifyItemRemoved(const ContentId& id);
+  void NotifyItemUpdated(const OfflineItem& item,
+                         const absl::optional<UpdateDelta>& update_delta);
+
+ private:
+  base::ObserverList<Observer> observers_;
 };
 
 }  // namespace offline_items_collection
 
-#endif  // COMPONENTS_OFFLINE_ITEMS_COLLETION_CORE_OFFLINE_CONTENT_PROVIDER_H_
+#endif  // COMPONENTS_OFFLINE_ITEMS_COLLECTION_CORE_OFFLINE_CONTENT_PROVIDER_H_
