@@ -4,6 +4,7 @@
 
 #include "components/offline_items_collection/core/offline_content_aggregator.h"
 
+#include "base/bind.h"
 #include "base/test/test_mock_time_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
@@ -12,17 +13,17 @@
 #include "components/offline_items_collection/core/test_support/scoped_mock_offline_content_provider.h"
 #include "components/offline_items_collection/core/throttled_offline_content_provider.h"
 #include "testing/gmock/include/gmock/gmock.h"
-#include "testing/gmock_mutant.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using testing::_;
-using testing::CallbackToFunctor;
+using testing::Eq;
 using testing::InvokeWithoutArgs;
-using testing::Return;
 
 namespace offline_items_collection {
 namespace {
 
+using GetVisualsOptions = OfflineContentProvider::GetVisualsOptions;
+
 // Helper class to automatically trigger another OnItemUpdated() to the
 // underlying provider when this observer gets notified of OnItemUpdated().
 // This will only happen the first time the ContentId of the udpated OfflineItem
@@ -39,13 +40,14 @@ class TriggerSingleReentrantUpdateHelper
         new_item_(new_item) {}
   ~TriggerSingleReentrantUpdateHelper() override {}
 
-  void OnItemUpdated(const OfflineItem& item) override {
+  void OnItemUpdated(const OfflineItem& item,
+                     const absl::optional<UpdateDelta>& update_delta) override {
     if (wrapped_provider_) {
       if (item.id == new_item_.id)
-        wrapped_provider_->NotifyOnItemUpdated(new_item_);
+        wrapped_provider_->NotifyOnItemUpdated(new_item_, update_delta);
       wrapped_provider_ = nullptr;
     }
-    ScopedMockObserver::OnItemUpdated(item);
+    ScopedMockObserver::OnItemUpdated(item, update_delta);
   }
 
  private:
@@ -59,13 +61,12 @@ class ThrottledOfflineContentProviderTest : public testing::Test {
       : task_runner_(new base::TestMockTimeTaskRunner),
         handle_(task_runner_),
         delay_(base::TimeDelta::FromSeconds(1)),
-        provider_(delay_, &wrapped_provider_),
-        weak_ptr_factory_(this) {}
+        provider_(delay_, &wrapped_provider_) {}
   ~ThrottledOfflineContentProviderTest() override {}
 
   MOCK_METHOD1(OnGetAllItemsDone,
                void(const OfflineContentProvider::OfflineItemList&));
-  MOCK_METHOD1(OnGetItemByIdDone, void(const base::Optional<OfflineItem>&));
+  MOCK_METHOD1(OnGetItemByIdDone, void(const absl::optional<OfflineItem>&));
 
  protected:
   base::TimeTicks GetTimeThatWillAllowAnUpdate() {
@@ -79,7 +80,8 @@ class ThrottledOfflineContentProviderTest : public testing::Test {
   base::TimeDelta delay_;
   MockOfflineContentProvider wrapped_provider_;
   ThrottledOfflineContentProvider provider_;
-  base::WeakPtrFactory<ThrottledOfflineContentProviderTest> weak_ptr_factory_;
+  base::WeakPtrFactory<ThrottledOfflineContentProviderTest> weak_ptr_factory_{
+      this};
 };
 
 TEST_F(ThrottledOfflineContentProviderTest, TestBasicPassthrough) {
@@ -92,26 +94,27 @@ TEST_F(ThrottledOfflineContentProviderTest, TestBasicPassthrough) {
   items.push_back(item);
 
   testing::InSequence sequence;
-  EXPECT_CALL(wrapped_provider_, OpenItem(LaunchLocation::DOWNLOAD_HOME, id));
+  EXPECT_CALL(wrapped_provider_, OpenItem(_, id));
   EXPECT_CALL(wrapped_provider_, RemoveItem(id));
   EXPECT_CALL(wrapped_provider_, CancelDownload(id));
   EXPECT_CALL(wrapped_provider_, PauseDownload(id));
   EXPECT_CALL(wrapped_provider_, ResumeDownload(id, true));
-  EXPECT_CALL(wrapped_provider_, GetVisualsForItem_(id, _));
+  EXPECT_CALL(wrapped_provider_, GetVisualsForItem_(id, _, _));
   wrapped_provider_.SetItems(items);
-  provider_.OpenItem(LaunchLocation::DOWNLOAD_HOME, id);
+  provider_.OpenItem(OpenParams(LaunchLocation::DOWNLOAD_HOME), id);
   provider_.RemoveItem(id);
   provider_.CancelDownload(id);
   provider_.PauseDownload(id);
   provider_.ResumeDownload(id, true);
-  provider_.GetVisualsForItem(id, OfflineContentProvider::VisualsCallback());
+  provider_.GetVisualsForItem(id, GetVisualsOptions::IconOnly(),
+                              OfflineContentProvider::VisualsCallback());
 
   EXPECT_CALL(*this, OnGetAllItemsDone(items)).Times(1);
   provider_.GetAllItems(
       base::BindOnce(&ThrottledOfflineContentProviderTest::OnGetAllItemsDone,
                      weak_ptr_factory_.GetWeakPtr()));
 
-  EXPECT_CALL(*this, OnGetItemByIdDone(base::make_optional(item))).Times(1);
+  EXPECT_CALL(*this, OnGetItemByIdDone(absl::make_optional(item))).Times(1);
   provider_.GetItemById(
       id,
       base::BindOnce(&ThrottledOfflineContentProviderTest::OnGetItemByIdDone,
@@ -125,11 +128,11 @@ TEST_F(ThrottledOfflineContentProviderTest, TestRemoveCancelsUpdate) {
   ContentId id("1", "A");
   OfflineItem item(id);
 
-  EXPECT_CALL(observer, OnItemUpdated(item)).Times(0);
+  EXPECT_CALL(observer, OnItemUpdated(item, Eq(absl::nullopt))).Times(0);
   EXPECT_CALL(observer, OnItemRemoved(id)).Times(1);
 
   provider_.set_last_update_time(base::TimeTicks::Now());
-  wrapped_provider_.NotifyOnItemUpdated(item);
+  wrapped_provider_.NotifyOnItemUpdated(item, absl::nullopt);
   wrapped_provider_.NotifyOnItemRemoved(id);
   task_runner_->FastForwardUntilNoTasksRemain();
 }
@@ -148,14 +151,16 @@ TEST_F(ThrottledOfflineContentProviderTest, TestOnItemUpdatedSquashed) {
   OfflineItem updated_item2(id2);
   updated_item2.title = "updated2";
 
-  EXPECT_CALL(observer, OnItemUpdated(updated_item1)).Times(1);
-  EXPECT_CALL(observer, OnItemUpdated(updated_item2)).Times(1);
+  EXPECT_CALL(observer, OnItemUpdated(updated_item1, Eq(absl::nullopt)))
+      .Times(1);
+  EXPECT_CALL(observer, OnItemUpdated(updated_item2, Eq(absl::nullopt)))
+      .Times(1);
 
   provider_.set_last_update_time(base::TimeTicks::Now());
-  wrapped_provider_.NotifyOnItemUpdated(item1);
-  wrapped_provider_.NotifyOnItemUpdated(item2);
-  wrapped_provider_.NotifyOnItemUpdated(updated_item2);
-  wrapped_provider_.NotifyOnItemUpdated(updated_item1);
+  wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
+  wrapped_provider_.NotifyOnItemUpdated(item2, absl::nullopt);
+  wrapped_provider_.NotifyOnItemUpdated(updated_item2, absl::nullopt);
+  wrapped_provider_.NotifyOnItemUpdated(updated_item1, absl::nullopt);
 
   task_runner_->FastForwardUntilNoTasksRemain();
 }
@@ -175,21 +180,22 @@ TEST_F(ThrottledOfflineContentProviderTest, TestGetItemByIdOverridesUpdate) {
   std::vector<OfflineItem> items = {item1, item2};
   wrapped_provider_.SetItems(items);
 
-  EXPECT_CALL(observer, OnItemUpdated(updated_item1)).Times(1);
-  EXPECT_CALL(observer, OnItemUpdated(item2)).Times(1);
+  EXPECT_CALL(observer, OnItemUpdated(updated_item1, Eq(absl::nullopt)))
+      .Times(1);
+  EXPECT_CALL(observer, OnItemUpdated(item2, Eq(absl::nullopt))).Times(1);
 
   provider_.set_last_update_time(base::TimeTicks::Now());
-  wrapped_provider_.NotifyOnItemUpdated(item1);
-  wrapped_provider_.NotifyOnItemUpdated(item2);
+  wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
+  wrapped_provider_.NotifyOnItemUpdated(item2, absl::nullopt);
 
   items = {updated_item1, item2};
   wrapped_provider_.SetItems(items);
 
-  auto single_item_callback = [](const base::Optional<OfflineItem>& item) {};
+  auto single_item_callback = [](const absl::optional<OfflineItem>& item) {};
   provider_.GetItemById(id1, base::BindOnce(single_item_callback));
 
   provider_.set_last_update_time(GetTimeThatWillAllowAnUpdate());
-  wrapped_provider_.NotifyOnItemUpdated(item2);
+  wrapped_provider_.NotifyOnItemUpdated(item2, absl::nullopt);
 
   task_runner_->FastForwardUntilNoTasksRemain();
 }
@@ -210,13 +216,14 @@ TEST_F(ThrottledOfflineContentProviderTest, TestGetAllItemsOverridesUpdate) {
   items.push_back(updated_item1);
   items.push_back(item2);
 
-  EXPECT_CALL(observer, OnItemUpdated(updated_item1)).Times(1);
-  EXPECT_CALL(observer, OnItemUpdated(item2)).Times(1);
+  EXPECT_CALL(observer, OnItemUpdated(updated_item1, Eq(absl::nullopt)))
+      .Times(1);
+  EXPECT_CALL(observer, OnItemUpdated(item2, Eq(absl::nullopt))).Times(1);
 
   wrapped_provider_.SetItems(items);
   provider_.set_last_update_time(base::TimeTicks::Now());
-  wrapped_provider_.NotifyOnItemUpdated(item1);
-  wrapped_provider_.NotifyOnItemUpdated(item2);
+  wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
+  wrapped_provider_.NotifyOnItemUpdated(item2, absl::nullopt);
 
   auto callback = [](const OfflineContentProvider::OfflineItemList& items) {};
   provider_.GetAllItems(base::BindOnce(callback));
@@ -241,23 +248,23 @@ TEST_F(ThrottledOfflineContentProviderTest, TestThrottleWorksProperly) {
   item4.title = "updated3";
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(GetTimeThatWillAllowAnUpdate());
-    wrapped_provider_.NotifyOnItemUpdated(item1);
+    wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item3)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item3, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(base::TimeTicks::Now());
-    wrapped_provider_.NotifyOnItemUpdated(item2);
-    wrapped_provider_.NotifyOnItemUpdated(item3);
+    wrapped_provider_.NotifyOnItemUpdated(item2, absl::nullopt);
+    wrapped_provider_.NotifyOnItemUpdated(item3, absl::nullopt);
     task_runner_->FastForwardBy(delay_);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item4)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item4, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(GetTimeThatWillAllowAnUpdate());
-    wrapped_provider_.NotifyOnItemUpdated(item4);
+    wrapped_provider_.NotifyOnItemUpdated(item4, absl::nullopt);
     task_runner_->FastForwardUntilNoTasksRemain();
   }
 }
@@ -273,19 +280,20 @@ TEST_F(ThrottledOfflineContentProviderTest, TestInitialRequestGoesThrough) {
   item1_updated.title = "updated1";
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(GetTimeThatWillAllowAnUpdate());
-    wrapped_provider_.NotifyOnItemUpdated(item1);
+    wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(_)).Times(0);
+    EXPECT_CALL(observer, OnItemUpdated(_, _)).Times(0);
     provider_.set_last_update_time(base::TimeTicks::Now());
-    wrapped_provider_.NotifyOnItemUpdated(item1_updated);
+    wrapped_provider_.NotifyOnItemUpdated(item1_updated, absl::nullopt);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1_updated)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1_updated, Eq(absl::nullopt)))
+        .Times(1);
     task_runner_->FastForwardUntilNoTasksRemain();
   }
 }
@@ -300,13 +308,14 @@ TEST_F(ThrottledOfflineContentProviderTest, TestReentrantUpdatesGetQueued) {
   TriggerSingleReentrantUpdateHelper observer(&provider_, &wrapped_provider_,
                                               updated_item);
   {
-    wrapped_provider_.NotifyOnItemUpdated(item);
-    EXPECT_CALL(observer, OnItemUpdated(item)).Times(1);
+    wrapped_provider_.NotifyOnItemUpdated(item, absl::nullopt);
+    EXPECT_CALL(observer, OnItemUpdated(item, Eq(absl::nullopt))).Times(1);
     task_runner_->FastForwardBy(delay_);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(updated_item)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(updated_item, Eq(absl::nullopt)))
+        .Times(1);
     task_runner_->FastForwardUntilNoTasksRemain();
   }
 }
@@ -323,63 +332,65 @@ TEST_F(ThrottledOfflineContentProviderTest, TestPokingProviderFlushesQueue) {
   OfflineItem item5(ContentId("5", "E"));
   OfflineItem item6(ContentId("6", "F"));
 
-  auto updater = base::Bind(&MockOfflineContentProvider::NotifyOnItemUpdated,
-                            base::Unretained(&wrapped_provider_));
-
   // Set up reentrancy calls back into the provider.
   EXPECT_CALL(wrapped_provider_, OpenItem(_, _))
-      .WillRepeatedly(
-          InvokeWithoutArgs(CallbackToFunctor(base::Bind(updater, item2))));
+      .WillRepeatedly(InvokeWithoutArgs([=]() {
+        wrapped_provider_.NotifyOnItemUpdated(item2, absl::nullopt);
+      }));
   EXPECT_CALL(wrapped_provider_, RemoveItem(_))
-      .WillRepeatedly(
-          InvokeWithoutArgs(CallbackToFunctor(base::Bind(updater, item3))));
+      .WillRepeatedly(InvokeWithoutArgs([=]() {
+        wrapped_provider_.NotifyOnItemUpdated(item3, absl::nullopt);
+      }));
   EXPECT_CALL(wrapped_provider_, CancelDownload(_))
-      .WillRepeatedly(
-          InvokeWithoutArgs(CallbackToFunctor(base::Bind(updater, item4))));
+      .WillRepeatedly(InvokeWithoutArgs([=]() {
+        wrapped_provider_.NotifyOnItemUpdated(item4, absl::nullopt);
+      }));
   EXPECT_CALL(wrapped_provider_, PauseDownload(_))
-      .WillRepeatedly(
-          InvokeWithoutArgs(CallbackToFunctor(base::Bind(updater, item5))));
+      .WillRepeatedly(InvokeWithoutArgs([=]() {
+        wrapped_provider_.NotifyOnItemUpdated(item5, absl::nullopt);
+      }));
   EXPECT_CALL(wrapped_provider_, ResumeDownload(_, _))
-      .WillRepeatedly(
-          InvokeWithoutArgs(CallbackToFunctor(base::Bind(updater, item6))));
+      .WillRepeatedly(InvokeWithoutArgs([=]() {
+        wrapped_provider_.NotifyOnItemUpdated(item6, absl::nullopt);
+      }));
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1)).Times(1);
-    EXPECT_CALL(observer, OnItemUpdated(item2)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1, Eq(absl::nullopt))).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item2, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(base::TimeTicks::Now());
-    wrapped_provider_.NotifyOnItemUpdated(item1);
-    provider_.OpenItem(LaunchLocation::DOWNLOAD_HOME, id1);
+    wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
+    provider_.OpenItem(OpenParams(LaunchLocation::DOWNLOAD_HOME), id1);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1)).Times(1);
-    EXPECT_CALL(observer, OnItemUpdated(item3)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1, Eq(absl::nullopt))).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item3, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(base::TimeTicks::Now());
-    wrapped_provider_.NotifyOnItemUpdated(item1);
+    wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
     provider_.RemoveItem(id1);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1)).Times(1);
-    EXPECT_CALL(observer, OnItemUpdated(item4)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1, Eq(absl::nullopt))).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item4, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(base::TimeTicks::Now());
-    wrapped_provider_.NotifyOnItemUpdated(item1);
+    wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
     provider_.CancelDownload(id1);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1)).Times(1);
-    EXPECT_CALL(observer, OnItemUpdated(item5)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1, Eq(absl::nullopt))).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item5, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(base::TimeTicks::Now());
-    wrapped_provider_.NotifyOnItemUpdated(item1);
+    wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
     provider_.PauseDownload(id1);
   }
 
   {
-    EXPECT_CALL(observer, OnItemUpdated(item1)).Times(1);
-    EXPECT_CALL(observer, OnItemUpdated(item6)).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item1, Eq(absl::nullopt))).Times(1);
+    EXPECT_CALL(observer, OnItemUpdated(item6, Eq(absl::nullopt))).Times(1);
     provider_.set_last_update_time(base::TimeTicks::Now());
-    wrapped_provider_.NotifyOnItemUpdated(item1);
+    wrapped_provider_.NotifyOnItemUpdated(item1, absl::nullopt);
     provider_.ResumeDownload(id1, false);
   }
 }
