@@ -7,6 +7,7 @@
 #include <memory>
 
 #include "base/bind.h"
+#include "base/callback_helpers.h"
 #include "base/test/test_mock_time_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "components/offline_pages/core/background/change_requests_state_task.h"
@@ -20,9 +21,10 @@
 
 namespace offline_pages {
 namespace {
+
 const int64_t kRequestId1 = 42;
 const int64_t kRequestId2 = 44;
-const GURL kUrl1("http://example.com");
+
 const ClientId kClientId1("download", "1234");
 
 class MarkAttemptAbortedTaskTest : public RequestQueueTaskTestBase {
@@ -46,8 +48,8 @@ class MarkAttemptAbortedTaskTest : public RequestQueueTaskTestBase {
 
 void MarkAttemptAbortedTaskTest::AddItemToStore(RequestQueueStore* store) {
   base::Time creation_time = OfflineTimeNow();
-  SavePageRequest request_1(kRequestId1, kUrl1, kClientId1, creation_time,
-                            true);
+  SavePageRequest request_1(kRequestId1, GURL("http://example.com"), kClientId1,
+                            creation_time, true);
   store->AddRequest(request_1, RequestQueue::AddOptions(),
                     base::BindOnce(&MarkAttemptAbortedTaskTest::AddRequestDone,
                                    base::Unretained(this)));
@@ -78,7 +80,7 @@ TEST_F(MarkAttemptAbortedTaskTest, MarkAttemptAbortedWhenStoreEmpty) {
       &store_, kRequestId1,
       base::BindOnce(&MarkAttemptAbortedTaskTest::ChangeRequestsStateCallback,
                      base::Unretained(this)));
-  task.Run();
+  task.Execute(base::DoNothing());
   PumpLoop();
   ASSERT_TRUE(last_result());
   EXPECT_EQ(1UL, last_result()->item_statuses.size());
@@ -97,7 +99,7 @@ TEST_F(MarkAttemptAbortedTaskTest, MarkAttemptAbortedWhenExists) {
       &store_, kRequestId1,
       base::BindOnce(&MarkAttemptAbortedTaskTest::ChangeRequestsStateCallback,
                      base::Unretained(this)));
-  start_request_task.Run();
+  start_request_task.Execute(base::DoNothing());
   PumpLoop();
   ClearResults();
 
@@ -106,7 +108,7 @@ TEST_F(MarkAttemptAbortedTaskTest, MarkAttemptAbortedWhenExists) {
       base::BindOnce(&MarkAttemptAbortedTaskTest::ChangeRequestsStateCallback,
                      base::Unretained(this)));
 
-  task.Run();
+  task.Execute(base::DoNothing());
   PumpLoop();
   ASSERT_TRUE(last_result());
   EXPECT_EQ(1UL, last_result()->item_statuses.size());
@@ -126,7 +128,7 @@ TEST_F(MarkAttemptAbortedTaskTest, MarkAttemptAbortedWhenItemMissing) {
       &store_, kRequestId2,
       base::BindOnce(&MarkAttemptAbortedTaskTest::ChangeRequestsStateCallback,
                      base::Unretained(this)));
-  task.Run();
+  task.Execute(base::DoNothing());
   PumpLoop();
   ASSERT_TRUE(last_result());
   EXPECT_EQ(1UL, last_result()->item_statuses.size());
@@ -145,7 +147,7 @@ TEST_F(MarkAttemptAbortedTaskTest, MarkAttemptAbortedWhenPaused) {
       &store_, kRequestId1,
       base::BindOnce(&MarkAttemptAbortedTaskTest::ChangeRequestsStateCallback,
                      base::Unretained(this)));
-  start_request_task.Run();
+  start_request_task.Execute(base::DoNothing());
   PumpLoop();
   ClearResults();
 
@@ -156,7 +158,7 @@ TEST_F(MarkAttemptAbortedTaskTest, MarkAttemptAbortedWhenPaused) {
       &store_, requests, SavePageRequest::RequestState::PAUSED,
       base::BindOnce(&MarkAttemptAbortedTaskTest::ChangeRequestsStateCallback,
                      base::Unretained(this)));
-  pauseTask.Run();
+  pauseTask.Execute(base::DoNothing());
   PumpLoop();
 
   // Abort the task, the state should not change from PAUSED.
@@ -165,7 +167,7 @@ TEST_F(MarkAttemptAbortedTaskTest, MarkAttemptAbortedWhenPaused) {
       base::BindOnce(&MarkAttemptAbortedTaskTest::ChangeRequestsStateCallback,
                      base::Unretained(this)));
 
-  abortTask.Run();
+  abortTask.Execute(base::DoNothing());
   PumpLoop();
   ASSERT_TRUE(last_result());
   EXPECT_EQ(1UL, last_result()->item_statuses.size());
