@@ -5,27 +5,27 @@
 #include "components/offline_pages/core/background/request_coordinator.h"
 
 #include <limits>
+#include <memory>
 #include <utility>
 
 #include "base/bind.h"
 #include "base/callback.h"
+#include "base/callback_helpers.h"
 #include "base/logging.h"
 #include "base/metrics/histogram_functions.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/rand_util.h"
-#include "base/stl_util.h"
 #include "base/system/sys_info.h"
 #include "base/time/time.h"
 #include "components/offline_pages/core/background/offliner.h"
 #include "components/offline_pages/core/background/offliner_client.h"
 #include "components/offline_pages/core/background/offliner_policy.h"
 #include "components/offline_pages/core/background/save_page_request.h"
-#include "components/offline_pages/core/client_policy_controller.h"
 #include "components/offline_pages/core/offline_clock.h"
+#include "components/offline_pages/core/offline_page_client_policy.h"
 #include "components/offline_pages/core/offline_page_feature.h"
 #include "components/offline_pages/core/offline_page_item.h"
 #include "components/offline_pages/core/offline_page_model.h"
-#include "components/offline_pages/core/offline_pages_ukm_reporter.h"
 #include "components/offline_pages/core/offline_store_utils.h"
 #include "services/network/public/cpp/network_quality_tracker.h"
 
@@ -205,9 +205,9 @@ constexpr bool IsCanceledOrInternalFailure(Offliner::RequestStatus status) {
 }
 
 // Returns the |BackgroundSavePageResult| appropriate for a single attempt
-// status. Returns |base::nullopt| for indeterminate status values that can be
+// status. Returns |absl::nullopt| for indeterminate status values that can be
 // retried.
-base::Optional<RequestNotifier::BackgroundSavePageResult> SingleAttemptResult(
+absl::optional<RequestNotifier::BackgroundSavePageResult> SingleAttemptResult(
     Offliner::RequestStatus status) {
   switch (status) {
       // Success status values.
@@ -223,11 +223,14 @@ base::Optional<RequestNotifier::BackgroundSavePageResult> SingleAttemptResult(
     case Offliner::RequestStatus::LOADING_DEFERRED:
     case Offliner::RequestStatus::BACKGROUND_SCHEDULER_CANCELED:
     case Offliner::RequestStatus::REQUEST_COORDINATOR_CANCELED:
-      return base::nullopt;
+      return absl::nullopt;
 
       // Other failure status values.
     case Offliner::RequestStatus::LOADING_FAILED_NO_RETRY:
     case Offliner::RequestStatus::LOADING_FAILED_DOWNLOAD:
+    case Offliner::RequestStatus::LOADED_PAGE_HAS_CERTIFICATE_ERROR:
+    case Offliner::RequestStatus::LOADED_PAGE_IS_BLOCKED:
+    case Offliner::RequestStatus::LOADED_PAGE_IS_CHROME_INTERNAL:
       return RequestNotifier::BackgroundSavePageResult::LOADING_FAILURE;
     case Offliner::RequestStatus::DOWNLOAD_THROTTLED:
       return RequestNotifier::BackgroundSavePageResult::DOWNLOAD_THROTTLED;
@@ -237,7 +240,7 @@ base::Optional<RequestNotifier::BackgroundSavePageResult> SingleAttemptResult(
     case Offliner::RequestStatus::LOADING_FAILED_HTTP_ERROR:
     case Offliner::RequestStatus::LOADING_FAILED_NO_NEXT:
     case Offliner::RequestStatus::REQUEST_COORDINATOR_TIMED_OUT:
-      return base::nullopt;
+      return absl::nullopt;
 
     // Only used by |Offliner| internally.
     case Offliner::RequestStatus::UNKNOWN:
@@ -247,7 +250,7 @@ base::Optional<RequestNotifier::BackgroundSavePageResult> SingleAttemptResult(
     // Only recorded by |RequestCoordinator| directly.
     case Offliner::RequestStatus::BROWSER_KILLED:
       DCHECK(false) << "Received invalid status: " << static_cast<int>(status);
-      return base::nullopt;
+      return absl::nullopt;
   }
 }
 
@@ -268,7 +271,6 @@ RequestCoordinator::RequestCoordinator(
     std::unique_ptr<RequestQueue> queue,
     std::unique_ptr<Scheduler> scheduler,
     network::NetworkQualityTracker* network_quality_tracker,
-    std::unique_ptr<OfflinePagesUkmReporter> ukm_reporter,
     std::unique_ptr<ActiveTabInfo> active_tab_info)
     : is_low_end_device_(base::SysInfo::IsLowEndDevice()),
       state_(RequestCoordinatorState::IDLE),
@@ -277,16 +279,13 @@ RequestCoordinator::RequestCoordinator(
       policy_(std::move(policy)),
       queue_(std::move(queue)),
       scheduler_(std::move(scheduler)),
-      policy_controller_(new ClientPolicyController()),
       network_quality_tracker_(network_quality_tracker),
-      ukm_reporter_(std::move(ukm_reporter)),
       network_quality_at_request_start_(net::EFFECTIVE_CONNECTION_TYPE_UNKNOWN),
       last_offlining_status_(Offliner::RequestStatus::UNKNOWN),
       scheduler_callback_(base::DoNothing()),
       internal_start_processing_callback_(base::DoNothing()),
       pending_state_updater_(this),
-      active_tab_info_(std::move(active_tab_info)),
-      weak_ptr_factory_(this) {
+      active_tab_info_(std::move(active_tab_info)) {
   DCHECK(policy_ != nullptr);
   DCHECK(network_quality_tracker_);
   offliner_client_ = std::make_unique<OfflinerClient>(
@@ -341,25 +340,17 @@ int64_t RequestCoordinator::SavePageLater(
 
   // Put the request on the request queue.
   queue_->AddRequest(
-      request, base::BindOnce(&RequestCoordinator::AddRequestResultCallback,
-                              weak_ptr_factory_.GetWeakPtr(),
-                              std::move(save_page_later_callback),
-                              save_page_later_params.availability));
+      request, save_page_later_params.add_options,
+      base::BindOnce(&RequestCoordinator::AddRequestResultCallback,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(save_page_later_callback),
+                     save_page_later_params.availability));
 
   // Record the network quality when this request is made.
-
   RecordSavePageLaterNetworkQuality(
       save_page_later_params.client_id,
       network_quality_tracker_->GetEffectiveConnectionType());
 
-  // Record UKM for this page offlining attempt.
-  if (ukm_reporter_) {
-    ukm_reporter_->ReportUrlOfflineRequest(
-        save_page_later_params.url,
-        save_page_later_params.availability ==
-            RequestAvailability::DISABLED_FOR_OFFLINER);
-  }
-
   return id;
 }
 
@@ -554,22 +545,26 @@ void RequestCoordinator::AddRequestResultCallback(
     RequestAvailability availability,
     AddRequestResult result,
     const SavePageRequest& request) {
-  NotifyAdded(request);
-  // Inform the scheduler that we have an outstanding task.
-  scheduler_->Schedule(GetTriggerConditions(kUserRequest));
+  if (result == AddRequestResult::SUCCESS) {
+    NotifyAdded(request);
+    // Inform the scheduler that we have an outstanding task.
+    scheduler_->Schedule(GetTriggerConditions(kUserRequest));
 
-  if (availability == RequestAvailability::DISABLED_FOR_OFFLINER) {
-    // Mark attempt started (presuming it is disabled for background offliner
-    // because foreground offlining is happening).
-    queue_->MarkAttemptStarted(
-        request.request_id(),
-        base::BindOnce(&RequestCoordinator::MarkAttemptDone,
-                       weak_ptr_factory_.GetWeakPtr(), request.request_id(),
-                       request.client_id().name_space));
-  } else if (request.user_requested()) {
-    StartImmediatelyIfConnected();
+    if (availability == RequestAvailability::DISABLED_FOR_OFFLINER) {
+      // Mark attempt started (presuming it is disabled for background offliner
+      // because foreground offlining is happening).
+      queue_->MarkAttemptStarted(
+          request.request_id(),
+          base::BindOnce(&RequestCoordinator::MarkAttemptDone,
+                         weak_ptr_factory_.GetWeakPtr(), request.request_id(),
+                         request.client_id().name_space));
+    } else if (request.user_requested()) {
+      StartImmediatelyIfConnected();
+    }
+  } else {
+    event_logger_.RecordAddRequestFailed(request.client_id().name_space,
+                                         result);
   }
-
   std::move(save_page_later_callback).Run(result);
 }
 
@@ -649,7 +644,7 @@ bool RequestCoordinator::StartScheduledProcessing(
     const DeviceConditions& device_conditions,
     const base::RepeatingCallback<void(bool)>& callback) {
   DVLOG(2) << "Scheduled " << __func__;
-  current_conditions_.reset(new DeviceConditions(device_conditions));
+  current_conditions_ = std::make_unique<DeviceConditions>(device_conditions);
   return StartProcessingInternal(ProcessingWindowState::SCHEDULED_WINDOW,
                                  callback);
 }
@@ -698,8 +693,7 @@ RequestCoordinator::TryImmediateStart(
     return OfflinerImmediateStartStatus::BUSY;
 
   // Make sure we are not on svelte device to start immediately.
-  if (is_low_end_device_ &&
-      !offline_pages::IsOfflinePagesSvelteConcurrentLoadingEnabled()) {
+  if (is_low_end_device_) {
     DVLOG(2) << "low end device, returning";
     // Let the scheduler know we are done processing and failed due to svelte.
     callback.Run(false);
@@ -723,9 +717,9 @@ RequestCoordinator::TryImmediateStart(
 }
 
 void RequestCoordinator::RequestConnectedEventForStarting() {
-  connection_notifier_.reset(new ConnectionNotifier(
+  connection_notifier_ = std::make_unique<ConnectionNotifier>(
       base::BindOnce(&RequestCoordinator::HandleConnectedEventForStarting,
-                     weak_ptr_factory_.GetWeakPtr())));
+                     weak_ptr_factory_.GetWeakPtr()));
 }
 
 void RequestCoordinator::ClearConnectedEventRequest() {
@@ -810,7 +804,7 @@ void RequestCoordinator::TryNextRequest(bool is_start_of_processing) {
   // Ask request queue to make a new PickRequestTask object, then put it on
   // the task queue.
   queue_->PickNextRequest(
-      policy_.get(), policy_controller_.get(),
+      policy_.get(),
       base::BindOnce(&RequestCoordinator::RequestPicked,
                      weak_ptr_factory_.GetWeakPtr()),
       base::BindOnce(&RequestCoordinator::RequestNotPicked,
@@ -930,7 +924,7 @@ void RequestCoordinator::SendRequestToOffliner(const SavePageRequest& request) {
     RecordStartTimeUMA(request);
   }
   const OfflinePageClientPolicy& policy =
-      policy_controller_->GetPolicy(request.client_id().name_space);
+      GetPolicy(request.client_id().name_space);
   if (policy.defer_background_fetch_while_page_is_active &&
       active_tab_info_->DoesActiveTabMatch(request.url())) {
     queue_->MarkAttemptDeferred(
@@ -1017,7 +1011,7 @@ void RequestCoordinator::OfflinerDoneCallback(const SavePageRequest& request,
 void RequestCoordinator::UpdateRequestForAttempt(
     const SavePageRequest& request,
     Offliner::RequestStatus status) {
-  base::Optional<RequestNotifier::BackgroundSavePageResult> attempt_result =
+  absl::optional<RequestNotifier::BackgroundSavePageResult> attempt_result =
       SingleAttemptResult(status);
 
   // If the request failed, report the connection type as of the start of the
@@ -1028,6 +1022,7 @@ void RequestCoordinator::UpdateRequestForAttempt(
     RecordNetworkQualityAtRequestStartForFailedRequest(
         request.client_id(), network_quality_at_request_start_);
   }
+
   if (IsCanceledOrInternalFailure(status)) {
     UpdateRequestForAbortedAttempt(request);
   } else if (attempt_result) {
@@ -1082,6 +1077,9 @@ bool RequestCoordinator::ShouldTryNextRequest(
     case Offliner::RequestStatus::LOADING_FAILED_NO_RETRY:
     case Offliner::RequestStatus::LOADING_FAILED_DOWNLOAD:
     case Offliner::RequestStatus::DOWNLOAD_THROTTLED:
+    case Offliner::RequestStatus::LOADED_PAGE_HAS_CERTIFICATE_ERROR:
+    case Offliner::RequestStatus::LOADED_PAGE_IS_BLOCKED:
+    case Offliner::RequestStatus::LOADED_PAGE_IS_CHROME_INTERNAL:
       return true;
     case Offliner::RequestStatus::FOREGROUND_CANCELED:
     case Offliner::RequestStatus::LOADING_CANCELED:
@@ -1186,10 +1184,6 @@ void RequestCoordinator::NotifyNetworkProgress(const SavePageRequest& request,
     observer.OnNetworkProgress(request, received_bytes);
 }
 
-ClientPolicyController* RequestCoordinator::GetPolicyController() {
-  return policy_controller_.get();
-}
-
 void RequestCoordinator::RecordOfflinerResult(const SavePageRequest& request,
                                               Offliner::RequestStatus status) {
   event_logger_.RecordOfflinerResult(request.client_id().name_space, status,
