@@ -7,12 +7,9 @@
 
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
+#include "base/time/time.h"
 #include "components/offline_pages/task/task.h"
 
-namespace base {
-class Time;
-}  // namespace base
-
 namespace offline_pages {
 
 class OfflinePageMetadataStore;
@@ -27,10 +24,10 @@ class MarkPageAccessedTask : public Task {
                        const base::Time& access_time);
   ~MarkPageAccessedTask() override;
 
+ private:
   // Task implementation.
   void Run() override;
 
- private:
   void OnMarkPageAccessedDone(bool result);
 
   // The metadata store used to update the page. Not owned.
@@ -39,7 +36,7 @@ class MarkPageAccessedTask : public Task {
   int64_t offline_id_;
   base::Time access_time_;
 
-  base::WeakPtrFactory<MarkPageAccessedTask> weak_ptr_factory_;
+  base::WeakPtrFactory<MarkPageAccessedTask> weak_ptr_factory_{this};
   DISALLOW_COPY_AND_ASSIGN(MarkPageAccessedTask);
 };
 
