@@ -4,11 +4,11 @@
 
 #include "components/offline_pages/core/prefetch/prefetch_request_fetcher.h"
 
+#include "base/bind.h"
 #include "base/test/mock_callback.h"
 #include "components/offline_pages/core/prefetch/prefetch_request_test_base.h"
 #include "components/offline_pages/core/prefetch/prefetch_types.h"
 #include "net/http/http_status_code.h"
-#include "net/url_request/url_request_status.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "url/gurl.h"
 
@@ -19,22 +19,22 @@ using testing::SaveArg;
 
 namespace offline_pages {
 
-namespace {
-const GURL kTestURL("http://exmaple.org");
-const char kTestMessage[] = "Testing";
-}  // namespace
-
 class PrefetchRequestFetcherTest : public PrefetchRequestTestBase {
  public:
   PrefetchRequestStatus RunFetcherWithNetError(net::Error net_error);
   PrefetchRequestStatus RunFetcherWithHttpError(net::HttpStatusCode http_error);
   PrefetchRequestStatus RunFetcherWithData(const std::string& response_data,
                                            std::string* data_received);
+  PrefetchRequestStatus RunFetcherWithHttpErrorAndData(
+      net::HttpStatusCode http_error,
+      const std::string& response_data);
+  void SetEmptyRequest(bool empty_request) { empty_request_ = empty_request; }
 
  private:
   PrefetchRequestStatus RunFetcher(
       base::OnceCallback<void(void)> respond_callback,
       std::string* data_received);
+  bool empty_request_;
 };
 
 PrefetchRequestStatus PrefetchRequestFetcherTest::RunFetcherWithNetError(
@@ -67,13 +67,25 @@ PrefetchRequestStatus PrefetchRequestFetcherTest::RunFetcherWithData(
                     data_received);
 }
 
+PrefetchRequestStatus
+PrefetchRequestFetcherTest::RunFetcherWithHttpErrorAndData(
+    net::HttpStatusCode http_error,
+    const std::string& response_data) {
+  std::string data_received;
+  return RunFetcher(
+      base::BindOnce(&PrefetchRequestTestBase::RespondWithHttpErrorAndData,
+                     base::Unretained(this), http_error, response_data),
+      &data_received);
+}
+
 PrefetchRequestStatus PrefetchRequestFetcherTest::RunFetcher(
     base::OnceCallback<void(void)> respond_callback,
     std::string* data_received) {
   base::MockCallback<PrefetchRequestFetcher::FinishedCallback> callback;
   std::unique_ptr<PrefetchRequestFetcher> fetcher =
       PrefetchRequestFetcher::CreateForPost(
-          kTestURL, kTestMessage, shared_url_loader_factory(), callback.Get());
+          GURL("http://example.org"), "Testing", /*testing_header_value=*/"",
+          empty_request_, shared_url_loader_factory(), callback.Get());
 
   PrefetchRequestStatus status;
   std::string data;
@@ -124,6 +136,15 @@ TEST_F(PrefetchRequestFetcherTest, HttpErrors) {
             RunFetcherWithHttpError(net::HTTP_SERVICE_UNAVAILABLE));
   EXPECT_EQ(PrefetchRequestStatus::kShouldRetryWithBackoff,
             RunFetcherWithHttpError(net::HTTP_GATEWAY_TIMEOUT));
+
+  SetEmptyRequest(false);
+  EXPECT_EQ(PrefetchRequestStatus::kShouldSuspendNewlyForbiddenByOPS,
+            RunFetcherWithHttpErrorAndData(net::HTTP_FORBIDDEN,
+                                           "request forbidden by OPS"));
+  SetEmptyRequest(true);
+  EXPECT_EQ(PrefetchRequestStatus::kShouldSuspendForbiddenByOPS,
+            RunFetcherWithHttpErrorAndData(net::HTTP_FORBIDDEN,
+                                           "request forbidden by OPS"));
 }
 
 TEST_F(PrefetchRequestFetcherTest, EmptyResponse) {
@@ -133,8 +154,17 @@ TEST_F(PrefetchRequestFetcherTest, EmptyResponse) {
   EXPECT_TRUE(data.empty());
 }
 
-TEST_F(PrefetchRequestFetcherTest, Success) {
+TEST_F(PrefetchRequestFetcherTest, EmptyRequestSuccess) {
   std::string data;
+  SetEmptyRequest(true);
+  EXPECT_EQ(PrefetchRequestStatus::kEmptyRequestSuccess,
+            RunFetcherWithData("Any data.", &data));
+  EXPECT_FALSE(data.empty());
+}
+
+TEST_F(PrefetchRequestFetcherTest, NonEmptyRequestSuccess) {
+  std::string data;
+  SetEmptyRequest(false);
   EXPECT_EQ(PrefetchRequestStatus::kSuccess,
             RunFetcherWithData("Any data.", &data));
   EXPECT_FALSE(data.empty());
